//-----------------------------------------------------------------------------
// File: D3DBase.cpp
//
// Desc: DirectX window application created by the DirectX AppWizard
//-----------------------------------------------------------------------------
#define STRICT
#define DIRECTINPUT_VERSION 0x0800
//#include <windows.h>
#include <afxwin.h>

#include <commctrl.h>
#include <commdlg.h>
#include <basetsd.h>
#include <math.h>
#include <stdio.h>
#include <d3dx9.h>
#include <dxerr9.h>
#include <tchar.h>
#include <dinput.h>
#include "DXUtil.h"
#include "D3DEnumeration.h"
#include "D3DSettings.h"
#include "D3DApp.h"
#include "D3DFont.h"
#include "D3DFile.h"
#include "D3DUtil.h"
#include "DIUtil.h"
#include "resource.h"

// GAME
#include "game/ITime.h"
#include "game/IRand.h"
#include "game/IInput.h"
#include "game/CGameControl.h"
#include "game/CUberControl.h"
#include "game/globalconst.h"
#include "game/CUberControl.h"
#include "game/stateflagdefs.h"
#include "game/IHighScores.h"
//#include "game/glfunc.h"
#include "game/glaux.h"
#include <string.h>
// ENDGAME

// globals
ITime *glTime;
IRand *glRand;
IInput *glInput;
GTrackArray* glTrack;

const int OBSTACLE_VERT_COUNT = 36;
const int COIN_VERT_COUNT = 36;
const float SCALE_OBSTACLE = 1.4;
const float SCALE_COIN = 0.8;
const float FRAC = 0.0007;
const float RANGE = 60;
// end globals

#include "D3DBase.h"




//-----------------------------------------------------------------------------
// Defines, and constants
//-----------------------------------------------------------------------------
// This GUID must be unique for every game, and the same for 
// every instance of this app.  // {ACDF766F-FCF1-42F1-A9B8-A4DD1A483E17}
// The GUID allows DirectInput to remember input settings
GUID g_guidApp = { 0xACDF766F, 0xFCF1, 0x42F1, { 0xA9, 0xB8, 0xA4, 0xDD, 0x1A, 0x48, 0x3E, 0x17 } };


// Input semantics used by this app
enum INPUT_SEMANTICS
{
    // Gameplay semantics
    // TODO: change as needed    
    INPUT_LEFT = 1,     INPUT_RIGHT,    
    INPUT_UP,			INPUT_DOWN,
	INPUT_SPACE,		INPUT_ESCAPE,
	INPUT_CONFIG_DISPLAY,
};

// Actions used by this app
DIACTION g_rgGameAction[] =
{
    // TODO: change as needed.  Be sure to delete user map files 
    // (C:\Program Files\Common Files\DirectX\DirectInput\User Maps\*.ini)
    // after changing this, otherwise settings won't reset and will be read 
    // from the out of date ini files 

    // Device input (joystick, etc.) that is pre-defined by DInput, according
    // to genre type. The genre for this app is space simulators.
 //   { INPUT_ROTATE_AXIS_LR,  DIAXIS_3DCONTROL_LATERAL,      0, TEXT("Rotate left/right"), },
 //   { INPUT_ROTATE_AXIS_UD,  DIAXIS_3DCONTROL_MOVE,         0, TEXT("Rotate up/down"), },

    // Keyboard input mappings
    { INPUT_LEFT,     DIKEYBOARD_LEFT,               0, TEXT("Move left"), },
    { INPUT_RIGHT,    DIKEYBOARD_RIGHT,              0, TEXT("Move right"), },
    { INPUT_UP,       DIKEYBOARD_UP,                 0, TEXT("Move up"), },
    { INPUT_DOWN,     DIKEYBOARD_DOWN,               0, TEXT("Move down"), },
	{ INPUT_SPACE,	  DIKEYBOARD_SPACE,				 0, TEXT("Switch Mode"), },
	{ INPUT_ESCAPE,	  DIKEYBOARD_ESCAPE,			 0, TEXT("Back/Exit"), },
    { INPUT_CONFIG_DISPLAY,  DIKEYBOARD_F2,                 DIA_APPFIXED, TEXT("Configure Display"), },    
 //   { INPUT_CONFIG_INPUT,    DIKEYBOARD_F3,                 DIA_APPFIXED, TEXT("Configure Input"), },    
};

#define NUMBER_OF_GAMEACTIONS    (sizeof(g_rgGameAction)/sizeof(DIACTION))




//-----------------------------------------------------------------------------
// Global access to the app (needed for the global WndProc())
//-----------------------------------------------------------------------------
CMyD3DApplication* g_pApp  = NULL;
HINSTANCE          g_hInst = NULL;




//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything, and goes into a
//       message-processing loop. Idle time is used to render the scene.
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    CMyD3DApplication d3dApp;

    g_pApp  = &d3dApp;
    g_hInst = hInst;

    InitCommonControls();
    if( FAILED( d3dApp.Create( hInst ) ) )
        return 0;

    return d3dApp.Run();
}




//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor.   Paired with ~CMyD3DApplication()
//       Member variables should be initialized to a known state here.  
//       The application window has not yet been created and no Direct3D device 
//       has been created, so any initialization that depends on a window or 
//       Direct3D should be deferred to a later stage. 
//-----------------------------------------------------------------------------
CMyD3DApplication::CMyD3DApplication()
{
    m_dwCreationWidth           = 640;
    m_dwCreationHeight          = 480;
    m_strWindowTitle            = TEXT( "Sonic Zoom" );
    m_d3dEnumeration.AppUsesDepthBuffer   = TRUE;
	m_bStartFullscreen			= false;
	m_bShowCursorWhenFullscreen	= false;

    // Create a D3D font using d3dfont.cpp
    m_pFont                     = new CD3DFont( _T("Arial"), 12, D3DFONT_BOLD );
	m_pBigFont                  = new CD3DFont( _T("Impact"), 48, D3DFONT_ITALIC );
    m_bLoadingApp               = TRUE;
    m_pVB                       = NULL;
    m_pInputDeviceManager       = NULL;
    m_pDIConfigSurface          = NULL;

    ZeroMemory( &m_UserInput, sizeof(m_UserInput) );
    m_fWorldRotX                = 0.0f;
    m_fWorldRotY                = 0.0f;

	// GAME STUFF - initialize variables
	m_iTrackVertexCount = 0;
	m_fNextModeSwitchTime = 0;

	m_fBoxRotateX = 0.0f;
	m_fBoxRotateY = 0.0f;
	m_fBoxRotateZ = 0.0f;

	m_pObstacleVB = NULL;
	m_pCoinVB = NULL;
	m_pTrackQuadVB = NULL;
	m_pMenuVB = NULL;

	m_pCoinTex = NULL;
	m_pObstacleTex = NULL;
	m_pTrackMainTex = NULL;
	m_pLine1Tex = NULL;
	m_pLine2Tex = NULL;
	m_pSideLowTex = NULL;
	m_pSideHighTex = NULL;
	m_pCeilingTex = NULL;
	m_pMenuTex = NULL;

	m_pSkyBox = NULL;
	

	glTime = new ITime();
	glRand = new IRand();
	glInput = new IInput();
	glTrack = new GTrackArray();
	m_pUberCon = new CUberControl();

	m_dwAmbColor = D3DCOLOR_ARGB(255,255,255,255);
	m_bNeedTrackRefresh = false;
	// END GAME

    // Read settings from registry
    //ReadSettings();
}




//-----------------------------------------------------------------------------
// Name: ~CMyD3DApplication()
// Desc: Application destructor.  Paired with CMyD3DApplication()
//-----------------------------------------------------------------------------
CMyD3DApplication::~CMyD3DApplication()
{
	delete m_pUberCon;
}




//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Paired with FinalCleanup().
//       The window has been created and the IDirect3D9 interface has been
//       created, but the device has not been created yet.  Here you can
//       perform application-related initialization and cleanup that does
//       not depend on a device.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::OneTimeSceneInit()
{
    // TODO: perform one time initialization

    // Drawing loading status message until app finishes loading
    SendMessage( m_hWnd, WM_PAINT, 0, 0 );

    // Initialize DirectInput
    InitInput( m_hWnd );

    m_bLoadingApp = FALSE;

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: ReadSettings()
// Desc: Read the app settings from the registry
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::ReadSettings()
{
    HKEY hkey;
    if( ERROR_SUCCESS == RegCreateKeyEx( HKEY_CURRENT_USER, DXAPP_KEY, 
        0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, NULL ) )
    {
        // TODO: change as needed

        // Read the stored window width/height.  This is just an example,
        // of how to use DXUtil_Read*() functions.
        DXUtil_ReadIntRegKey( hkey, TEXT("Width"), &m_dwCreationWidth, m_dwCreationWidth );
        DXUtil_ReadIntRegKey( hkey, TEXT("Height"), &m_dwCreationHeight, m_dwCreationHeight );

        RegCloseKey( hkey );
    }
}




//-----------------------------------------------------------------------------
// Name: WriteSettings()
// Desc: Write the app settings to the registry
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::WriteSettings()
{
    HKEY hkey;

    if( ERROR_SUCCESS == RegCreateKeyEx( HKEY_CURRENT_USER, DXAPP_KEY, 
        0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, NULL ) )
    {
        // TODO: change as needed

        // Write the window width/height.  This is just an example,
        // of how to use DXUtil_Write*() functions.
        DXUtil_WriteIntRegKey( hkey, TEXT("Width"), m_rcWindowClient.right );
        DXUtil_WriteIntRegKey( hkey, TEXT("Height"), m_rcWindowClient.bottom );

        RegCloseKey( hkey );
    }
}





//-----------------------------------------------------------------------------
// Name: StaticInputAddDeviceCB()
// Desc: Static callback helper to call into CMyD3DApplication class
//-----------------------------------------------------------------------------
HRESULT CALLBACK CMyD3DApplication::StaticInputAddDeviceCB( 
                                         CInputDeviceManager::DeviceInfo* pDeviceInfo, 
                                         const DIDEVICEINSTANCE* pdidi, 
                                         LPVOID pParam )
{
    CMyD3DApplication* pApp = (CMyD3DApplication*) pParam;
    return pApp->InputAddDeviceCB( pDeviceInfo, pdidi );
}




//-----------------------------------------------------------------------------
// Name: InputAddDeviceCB()
// Desc: Called from CInputDeviceManager whenever a device is added. 
//       Set the dead zone, and creates a new InputDeviceState for each device
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InputAddDeviceCB( CInputDeviceManager::DeviceInfo* pDeviceInfo, 
                                                   const DIDEVICEINSTANCE* pdidi )
{
    UNREFERENCED_PARAMETER( pdidi );
    
    // Setup the deadzone 
    DIPROPDWORD dipdw;
    dipdw.diph.dwSize       = sizeof(DIPROPDWORD);
    dipdw.diph.dwHeaderSize = sizeof(DIPROPHEADER);
    dipdw.diph.dwObj        = 0;
    dipdw.diph.dwHow        = DIPH_DEVICE;
    dipdw.dwData            = 500;
    pDeviceInfo->pdidDevice->SetProperty( DIPROP_DEADZONE, &dipdw.diph );

    // Create a new InputDeviceState for each device so the 
    // app can record its state 
    InputDeviceState* pNewInputDeviceState = new InputDeviceState;
    ZeroMemory( pNewInputDeviceState, sizeof(InputDeviceState) );
    pDeviceInfo->pParam = (LPVOID) pNewInputDeviceState;

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: InitInput()
// Desc: Initialize DirectInput objects
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InitInput( HWND hWnd )
{
    HRESULT hr;

    // Setup action format for the actual gameplay
    ZeroMemory( &m_diafGame, sizeof(DIACTIONFORMAT) );
    m_diafGame.dwSize          = sizeof(DIACTIONFORMAT);
    m_diafGame.dwActionSize    = sizeof(DIACTION);
    m_diafGame.dwDataSize      = NUMBER_OF_GAMEACTIONS * sizeof(DWORD);
    m_diafGame.guidActionMap   = g_guidApp;

    // TODO: change the genre as needed
    m_diafGame.dwGenre         = DIVIRTUAL_CAD_3DCONTROL; 

    m_diafGame.dwNumActions    = NUMBER_OF_GAMEACTIONS;
    m_diafGame.rgoAction       = g_rgGameAction;
    m_diafGame.lAxisMin        = -100;
    m_diafGame.lAxisMax        = 100;
    m_diafGame.dwBufferSize    = 16;
    _tcscpy( m_diafGame.tszActionMap, _T("Sonic Zoom") );

    // Create a new input device manager
    m_pInputDeviceManager = new CInputDeviceManager();

    if( FAILED( hr = m_pInputDeviceManager->Create( hWnd, NULL, m_diafGame, 
                                                    StaticInputAddDeviceCB, this ) ) )
        return DXTRACE_ERR( "m_pInputDeviceManager->Create", hr );

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the display device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::ConfirmDevice( D3DCAPS9* pCaps, DWORD dwBehavior,
                                          D3DFORMAT Format )
{
    UNREFERENCED_PARAMETER( Format );
    UNREFERENCED_PARAMETER( dwBehavior );
    UNREFERENCED_PARAMETER( pCaps );
    
    BOOL bCapsAcceptable;

    // TODO: Perform checks to see if these display caps are acceptable.
    bCapsAcceptable = TRUE;

    if( bCapsAcceptable )         
        return S_OK;
    else
        return E_FAIL;
}




//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Paired with DeleteDeviceObjects()
//       The device has been created.  Resources that are not lost on
//       Reset() can be created here -- resources in D3DPOOL_MANAGED,
//       D3DPOOL_SCRATCH, or D3DPOOL_SYSTEMMEM.  Image surfaces created via
//       CreateImageSurface are never lost and can be created here.  Vertex
//       shaders and pixel shaders can also be created here as they are not
//       lost on Reset().
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InitDeviceObjects()
{
    // TODO: create device objects

    HRESULT hr;

    // Init the font
    m_pFont->InitDeviceObjects( m_pd3dDevice );
	m_pBigFont->InitDeviceObjects( m_pd3dDevice );

	// Init the skybox... SONICZOOM
	m_pSkyBox = new CD3DMesh();
	if( FAILED( m_pSkyBox->Create( m_pd3dDevice, _T("graphics/skybox2sz.x") ) ) )
        return D3DAPPERR_MEDIANOTFOUND;
	
	// LOAD TEXTURES - SONIC ZOOM
	if( FAILED( hr = D3DUtil_CreateTextureColorKey( m_pd3dDevice, _T("graphics/minetex.bmp"), &m_pObstacleTex, D3DFMT_UNKNOWN, 0xFF00FF ) ) )
    {
        return DXTRACE_ERR( TEXT("D3DUtil_CreateTexture"), hr );
    }
	if( FAILED( hr = D3DUtil_CreateTextureColorKey( m_pd3dDevice, _T("graphics/cointex.bmp"), &m_pCoinTex, D3DFMT_UNKNOWN, 0xFF00FF ) ) )
    {
        return DXTRACE_ERR( TEXT("D3DUtil_CreateTexture"), hr );
    }
	if( FAILED( hr = D3DUtil_CreateTextureColorKey( m_pd3dDevice, _T("graphics/yellowground.bmp"), &m_pTrackMainTex, D3DFMT_UNKNOWN, 0xFF00FF ) ) )
    {
        return DXTRACE_ERR( TEXT("D3DUtil_CreateTexture"), hr );
    }
	if( FAILED( hr = D3DUtil_CreateTextureColorKey( m_pd3dDevice, _T("graphics/line1.bmp"), &m_pLine1Tex, D3DFMT_UNKNOWN, 0xFF00FF ) ) )
    {
        return DXTRACE_ERR( TEXT("D3DUtil_CreateTexture"), hr );
    }
	if( FAILED( hr = D3DUtil_CreateTextureColorKey( m_pd3dDevice, _T("graphics/line2.bmp"), &m_pLine2Tex, D3DFMT_UNKNOWN, 0xFF00FF ) ) )
    {
        return DXTRACE_ERR( TEXT("D3DUtil_CreateTexture"), hr );
    }
	if( FAILED( hr = D3DUtil_CreateTextureColorKey( m_pd3dDevice, _T("graphics/woodfloor.bmp"), &m_pSideLowTex, D3DFMT_UNKNOWN, 0xFF00FF ) ) )
    {
        return DXTRACE_ERR( TEXT("D3DUtil_CreateTexture"), hr );
    }
	if( FAILED( hr = D3DUtil_CreateTextureColorKey( m_pd3dDevice, _T("graphics/creteyell.bmp"), &m_pSideHighTex, D3DFMT_UNKNOWN, 0xFF00FF ) ) )
    {
        return DXTRACE_ERR( TEXT("D3DUtil_CreateTexture"), hr );
    }
	if( FAILED( hr = D3DUtil_CreateTextureColorKey( m_pd3dDevice, _T("graphics/ceilingyellow.bmp"), &m_pCeilingTex, D3DFMT_UNKNOWN, 0xFF00FF ) ) )
    {
        return DXTRACE_ERR( TEXT("D3DUtil_CreateTexture"), hr );
    }
	if( FAILED( hr = D3DUtil_CreateTextureColorKey( m_pd3dDevice, _T("graphics/menu.jpg"), &m_pMenuTex, D3DFMT_UNKNOWN, 0xFF00FF ) ) )
    {
        return DXTRACE_ERR( TEXT("D3DUtil_CreateTexture"), hr );
    }
	// end textures SONICZOOM


    // Create the vertex buffer
    if( FAILED( hr = m_pd3dDevice->CreateVertexBuffer( 3*2*sizeof(CUSTOMVERTEX),
                                                  0, D3DFVF_CUSTOMVERTEX,
                                                  D3DPOOL_MANAGED, &m_pVB, NULL ) ) )
        return DXTRACE_ERR( "CreateVertexBuffer", hr );

    // Fill the vertex buffer with 2 triangles
    CUSTOMVERTEX* pVertices;

    if( FAILED( hr = m_pVB->Lock( 0, 0, (VOID**)&pVertices, 0 ) ) )
        return DXTRACE_ERR( "Lock", hr );

    // Front triangle
    pVertices[0].position = D3DXVECTOR3( -1.0f, -1.0f,  0.0f );
    pVertices[0].normal   = D3DXVECTOR3(  0.0f,  0.0f, -1.0f );
    pVertices[1].position = D3DXVECTOR3(  0.0f,  1.0f,  0.0f );
    pVertices[1].normal   = D3DXVECTOR3(  0.0f,  0.0f, -1.0f );
    pVertices[2].position = D3DXVECTOR3(  1.0f, -1.0f,  0.0f );
    pVertices[2].normal   = D3DXVECTOR3(  0.0f,  0.0f, -1.0f );

    // Back triangle
    pVertices[3].position = D3DXVECTOR3( -1.0f, -1.0f,  0.0f );
    pVertices[3].normal   = D3DXVECTOR3(  0.0f,  0.0f,  1.0f );
    pVertices[4].position = D3DXVECTOR3(  1.0f, -1.0f,  0.0f );
    pVertices[4].normal   = D3DXVECTOR3(  0.0f,  0.0f,  1.0f );
    pVertices[5].position = D3DXVECTOR3(  0.0f,  1.0f,  0.0f );
    pVertices[5].normal   = D3DXVECTOR3(  0.0f,  0.0f,  1.0f );

    m_pVB->Unlock();

	if( FAILED( m_pd3dDevice->CreateVertexBuffer( 4*sizeof(SCREENVERTEX),
                                       D3DUSAGE_WRITEONLY, D3DFVF_SCREENVERTEX,
                                       D3DPOOL_MANAGED, &m_pMenuVB, NULL ) ) )
        return DXTRACE_ERR( TEXT("m_pd3dDevice->CreateVertexBuffer"), hr );

	// SONICZOOM
	InitGameVertexBuffers(); // Dan - need to create and load the cube buffers

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Paired with InvalidateDeviceObjects()
//       The device exists, but may have just been Reset().  Resources in
//       D3DPOOL_DEFAULT and any other device state that persists during
//       rendering should be set here.  Render states, matrices, textures,
//       etc., that don't change during rendering can be set once here to
//       avoid redundant state setting during Render() or FrameMove().
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::RestoreDeviceObjects()
{
    // TODO: setup render states
    HRESULT hr;
	
	glTrack->bCreated = false; // need to recreate the track
	//m_bNeedTrackRefresh = true; // need to reload the track geometry

	// Set up the default texture states
	// Dan - this is the right combination of states
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
	m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE );
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
	m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE );

    m_pd3dDevice->SetSamplerState( 0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR );
    m_pd3dDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR );
    m_pd3dDevice->SetSamplerState( 0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR );

    // Set miscellaneous render states
    m_pd3dDevice->SetRenderState( D3DRS_DITHERENABLE,   FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_SPECULARENABLE, FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_ZENABLE,        TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_AMBIENT,  m_dwAmbColor );
	m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
	m_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,D3DBLEND_SRCALPHA);	//source blend factor
	m_pd3dDevice->SetRenderState( D3DRS_DESTBLEND,D3DBLEND_INVSRCALPHA);	//destination blend factor

	// X MODELS
	m_pSkyBox->RestoreDeviceObjects( m_pd3dDevice );

	// END

    // Set the world matrix
	// Dan - we do this every frame, but it can only hurt to take this out
    D3DXMATRIX matIdentity;
    D3DXMatrixIdentity( &matIdentity );
    m_pd3dDevice->SetTransform( D3DTS_WORLD,  &matIdentity );

    // Set the projection matrix
    D3DXMATRIX matProj;
    FLOAT fAspect = ((FLOAT)m_d3dsdBackBuffer.Width) / m_d3dsdBackBuffer.Height;
    D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, fAspect, 1.0f, 5000.0f ); // Dan - 5000.0f is also the cut-off for drawing geometry
    m_pd3dDevice->SetTransform( D3DTS_PROJECTION, &matProj );

    m_pd3dDevice->LightEnable( 0, TRUE ); // Dan - we want lighting even though we don't use lights
    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE ); // Dan - ditto

    // Restore the font
    m_pFont->RestoreDeviceObjects();
	m_pBigFont->RestoreDeviceObjects();
	
	// RESTORE THE SCREEN VERTEX BUFFER
	D3DVIEWPORT9 vp;
    m_pd3dDevice->GetViewport(&vp);
    FLOAT sx = (FLOAT)vp.Width;
    FLOAT sy = (FLOAT)vp.Height;
	if( m_pMenuVB )
    {
        // Setup dimensions for the viewport covering square
        SCREENVERTEX* v;
        m_pMenuVB->Lock( 0, 0, (void**)&v, 0 );
        v[0].color = v[1].color = v[2].color = v[3].color = 0xFFFFFFFF;
        v[0].p = D3DXVECTOR4( 0,sy,0.0f,1.0f);
        v[0].tu = 0.0f;  v[0].tv = 1.0f; 
        v[1].p = D3DXVECTOR4( 0, 0,0.0f,1.0f);
        v[1].tu = 0.0f;  v[1].tv = 0.0f; 
        v[2].p = D3DXVECTOR4(sx,sy,0.0f,1.0f);
        v[2].tu = 1.0f;  v[2].tv = 1.0f; 
        v[3].p = D3DXVECTOR4(sx, 0,0.0f,1.0f);
        v[3].tu = 1.0f;  v[3].tv = 0.0f; 
        m_pMenuVB->Unlock();
    }

    if( !m_bWindowed )
    {
        // Create a surface for configuring DInput devices
        if( FAILED( hr = m_pd3dDevice->CreateOffscreenPlainSurface( 640, 480, 
                                        m_d3dsdBackBuffer.Format, D3DPOOL_DEFAULT, 
										&m_pDIConfigSurface, NULL ) ) ) 
            return DXTRACE_ERR( "CreateOffscreenPlainSurface", hr );
    }

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: StaticConfigureInputDevicesCB()
// Desc: Static callback helper to call into CMyD3DApplication class
//-----------------------------------------------------------------------------
BOOL CALLBACK CMyD3DApplication::StaticConfigureInputDevicesCB( 
                                            IUnknown* pUnknown, VOID* pUserData )
{
    CMyD3DApplication* pApp = (CMyD3DApplication*) pUserData;
    return pApp->ConfigureInputDevicesCB( pUnknown );
}




//-----------------------------------------------------------------------------
// Name: ConfigureInputDevicesCB()
// Desc: Callback function for configuring input devices. This function is
//       called in fullscreen modes, so that the input device configuration
//       window can update the screen.
//-----------------------------------------------------------------------------
BOOL CMyD3DApplication::ConfigureInputDevicesCB( IUnknown* pUnknown )
{
    // Get access to the surface
    LPDIRECT3DSURFACE9 pConfigSurface = NULL;
    if( FAILED( pUnknown->QueryInterface( IID_IDirect3DSurface9,
                                          (VOID**)&pConfigSurface ) ) )
        return TRUE;

    // Render the scene, with the config surface blitted on top
    Render();

    RECT  rcSrc;
    SetRect( &rcSrc, 0, 0, 640, 480 );

    POINT ptDst;
    ptDst.x = (m_d3dsdBackBuffer.Width-640)/2;
    ptDst.y = (m_d3dsdBackBuffer.Height-480)/2;

    LPDIRECT3DSURFACE9 pBackBuffer;
    m_pd3dDevice->GetBackBuffer( 0, 0, D3DBACKBUFFER_TYPE_MONO, &pBackBuffer );
    m_pd3dDevice->UpdateSurface( pConfigSurface, &rcSrc, pBackBuffer, &ptDst );
    pBackBuffer->Release();

    // Present the backbuffer contents to the front buffer
    m_pd3dDevice->Present( 0, 0, 0, 0 );

    // Release the surface
    pConfigSurface->Release();

    return TRUE;
}




//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::FrameMove()
{
    
    // Update user input state
    UpdateInput( &m_UserInput );
	glTime->DXFrameUpdate( m_fTime, m_fElapsedTime ); // update our global time variable

	m_pUberCon->Draw(); // Advance the game a frame (MAIN CALL) ... does not actually draw anything

	// Is the level changing (and do we therefore need to remake our track?
	if ( m_pUberCon->exState == EX_GAME /*&& m_pUberCon->gameControl->levelLength > 0*/ )
	{
		if ( m_pUberCon->gameControl->gameState == GAME_LEVELINIT )
			m_bNeedTrackRefresh = true; // we're going to need to refresh the track next frame

		if ( m_pUberCon->gameControl->gameState == GAME_STARTING && m_bNeedTrackRefresh )
		{
			float trackLength = m_pUberCon->gameControl->levelLength; // this is the length the track should be
			bool bNeedCleanup = false; // do we need to do cleanup (no if it's our first time)
			int iLevel = m_pUberCon->gameControl->level % 11; // what type of level (trackwise) is this?
			const int iNORMAL = 0; // to decide which create function to call - only ground track
			const int iTUNNEL = 1; // to decide which create function to call - tunnel
			int iType = 0; // normal or tunnel?
			int iWarp = WARP_NONE; // which type of warp, initialze to none.

			if ( glTrack->bCreated )
				bNeedCleanup = true; // we're already created, need to free memory

			switch ( iLevel )
			{
			case 0: // bright, in the open, straight track
				iWarp = WARP_NONE;
				iType = iNORMAL;
				m_dwAmbColor = D3DCOLOR_ARGB(255,255,255,255);
				break;
			case 1:	// bright, in the tunnel, straight track
				iWarp = WARP_NONE;
				iType = iTUNNEL;
				m_dwAmbColor = D3DCOLOR_ARGB(255,255,255,255);
				break;
			case 2: // dark, in the tunnel, straight track
				iWarp = WARP_NONE;
				iType = iTUNNEL;
				m_dwAmbColor = D3DCOLOR_ARGB(255,100,100,100);
				break;
			case 3: // dark, in the tunnel, rollercoaster
				iWarp = WARP_CURVY;
				iType = iTUNNEL;
				m_dwAmbColor = D3DCOLOR_ARGB(255,100,100,100);
				break;
			case 4: // bright, in the open, rollercoaster
				iWarp = WARP_CURVY;
				iType = iNORMAL;
				m_dwAmbColor = D3DCOLOR_ARGB(255,255,255,255);
				break;
			case 5: // bright, in the tunnel, rollercoaster
				iWarp = WARP_CURVY;
				iType = iTUNNEL;
				m_dwAmbColor = D3DCOLOR_ARGB(255,255,255,255);
				break;
			case 6: // bright, in the tunnel, perp spiral
				iWarp = WARP_PERPSPIRAL;
				iType = iTUNNEL;
				m_dwAmbColor = D3DCOLOR_ARGB(255,255,255,255);
				break;
			case 7: // dark, in the tunnel, perp spiral
				iWarp = WARP_PERPSPIRAL;
				iType = iTUNNEL;
				m_dwAmbColor = D3DCOLOR_ARGB(255,100,100,100);
				break;
			case 8: // dark, in the tunnel, axial spiral
				iWarp = WARP_AXIALSPIRAL;
				iType = iTUNNEL;
				m_dwAmbColor = D3DCOLOR_ARGB(255,100,100,100);
				break;
			case 9: // bright, in the tunnel, axial spiral
				iWarp = WARP_AXIALSPIRAL;
				iType = iTUNNEL;
				m_dwAmbColor = D3DCOLOR_ARGB(255,255,255,255);
				break;
			case 10: // bright, outside,  axialspiral
				iWarp = WARP_AXIALSPIRAL;
				iType = iNORMAL;
				m_dwAmbColor = D3DCOLOR_ARGB(255,255,255,255);
				break;
			}

			if ( bNeedCleanup )
				glTrack->Cleanup(); // do cleanup

			if ( iType == iNORMAL )
				glTrack->Create( trackLength ); // create normal
			else
				glTrack->Create2( trackLength ); // create tunnel

			glTrack->DoWarp(RANGE,FRAC,iWarp); // do our chosen warp
			LoadStaticTrackGeometry(); // load track into its vertex buffer
			m_pd3dDevice->SetRenderState( D3DRS_AMBIENT,  m_dwAmbColor ); // reset the light level


			m_bNeedTrackRefresh = false; // don't need to do this until the next level change
		}
	}
	if ( m_pUberCon->exState == EX_QUIT )
	{
		SendMessage( m_hWnd, WM_CLOSE, 0, 0 ); // we're quitting!
	}

	/*
    // Respond to input
    if( m_UserInput.bDoConfigureInput && m_bWindowed )  // full-screen configure disabled for now
    {
        // One-shot per keypress
        m_UserInput.bDoConfigureInput = FALSE;

        Pause( true );

        // Get access to the list of semantically-mapped input devices
        // to delete all InputDeviceState structs before calling ConfigureDevices()
        CInputDeviceManager::DeviceInfo* pDeviceInfos;
        DWORD dwNumDevices;
        m_pInputDeviceManager->GetDevices( &pDeviceInfos, &dwNumDevices );

        for( DWORD i=0; i<dwNumDevices; i++ )
        {
            InputDeviceState* pInputDeviceState = (InputDeviceState*) pDeviceInfos[i].pParam;
            SAFE_DELETE( pInputDeviceState );
            pDeviceInfos[i].pParam = NULL;
        }

        // Configure the devices (with edit capability)
        if( m_bWindowed )
            m_pInputDeviceManager->ConfigureDevices( m_hWnd, NULL, NULL, DICD_EDIT, NULL );
        else
            m_pInputDeviceManager->ConfigureDevices( m_hWnd,
                                                     m_pDIConfigSurface,
                                                     (VOID*)StaticConfigureInputDevicesCB,
                                                     DICD_EDIT, (LPVOID) this );

        Pause( false );
    }
*/
    if( m_UserInput.bDoConfigureDisplay )
    {
        // One-shot per keypress
        m_UserInput.bDoConfigureDisplay = FALSE;

        Pause(true);

        // Configure the display device
        UserSelectNewDevice();

        Pause(false);
    }

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: UpdateInput()
// Desc: Update the user input.  Called once per frame 
//-----------------------------------------------------------------------------
void CMyD3DApplication::UpdateInput( UserInput* pUserInput )
{
    if( NULL == m_pInputDeviceManager )
        return;

    // Get access to the list of semantically-mapped input devices
    CInputDeviceManager::DeviceInfo* pDeviceInfos;
    DWORD dwNumDevices;
    m_pInputDeviceManager->GetDevices( &pDeviceInfos, &dwNumDevices );

    // Loop through all devices and check game input
    for( DWORD i=0; i<dwNumDevices; i++ )
    {
        DIDEVICEOBJECTDATA rgdod[10];
        DWORD   dwItems = 10;
        HRESULT hr;
        LPDIRECTINPUTDEVICE8 pdidDevice = pDeviceInfos[i].pdidDevice;
        InputDeviceState* pInputDeviceState = (InputDeviceState*) pDeviceInfos[i].pParam;

        hr = pdidDevice->Acquire();
        hr = pdidDevice->Poll();
        hr = pdidDevice->GetDeviceData( sizeof(DIDEVICEOBJECTDATA),
                                        rgdod, &dwItems, 0 );
        if( FAILED(hr) )
            continue;

        // Get the sematics codes for the game menu
        for( DWORD j=0; j<dwItems; j++ )
        {
            BOOL  bButtonState = (rgdod[j].dwData==0x80) ? TRUE : FALSE;
            FLOAT fButtonState = (rgdod[j].dwData==0x80) ? 1.0f : 0.0f;
            FLOAT fAxisState   = (FLOAT)((int)rgdod[j].dwData)/100.0f;
            UNREFERENCED_PARAMETER( fButtonState );

            switch( rgdod[j].uAppData )
            {
                // TODO: Handle semantics for the game 

      /*          // Handle relative axis data
                case INPUT_ROTATE_AXIS_LR: 
                    pInputDeviceState->fAxisRotateLR = -fAxisState;
                    break;
                case INPUT_ROTATE_AXIS_UD:
                    pInputDeviceState->fAxisRotateUD = -fAxisState;
                    break;
*/
                // Handle buttons separately so the button state data
                // doesn't overwrite the axis state data, and handle
                // each button separately so they don't overwrite each other

				// Dan - we use this
                case INPUT_LEFT:  pInputDeviceState->bButtonLeft  = bButtonState; break;
                case INPUT_RIGHT: pInputDeviceState->bButtonRight = bButtonState; break;
                case INPUT_UP:    pInputDeviceState->bButtonUp    = bButtonState; break;
                case INPUT_DOWN:  pInputDeviceState->bButtonDown  = bButtonState; break;
				case INPUT_SPACE: pInputDeviceState->bButtonSpace = bButtonState; break;
				case INPUT_ESCAPE: pInputDeviceState->bButtonEscape = bButtonState; break;

                // Handle one-shot buttons
//                case INPUT_CONFIG_INPUT:   if( bButtonState ) pUserInput->bDoConfigureInput = TRUE; break;
                case INPUT_CONFIG_DISPLAY: if( bButtonState ) pUserInput->bDoConfigureDisplay = TRUE; break;
            }
        }
    }

    // TODO: change process code as needed

    // Process user input and store result into pUserInput struct
    pUserInput->fAxisRotateLR = 0.0f;
    pUserInput->fAxisRotateUD = 0.0f;

	// SONICZOOM
	// Dan - we go through each device, and if any say a button is down, we take that as a yes
	// otherwise, we assume that the button is not down.
	bool bUp = false;
	bool bDown = false;
	bool bRight = false;
	bool bLeft = false;
	bool bSpace = false;
	bool bEscape = false;
	// ENDSZ

    // Combine the data from all the DirectInput devices
    for( i=0; i<dwNumDevices; i++ )
    {
        InputDeviceState* pInputDeviceState = (InputDeviceState*) pDeviceInfos[i].pParam;

        // Process the button data 
		// SONICZOOM
        if( pInputDeviceState->bButtonLeft )
			bRight = true;
        
		if( pInputDeviceState->bButtonRight )
			bLeft = true; // GAME

        if( pInputDeviceState->bButtonUp )
			bUp = true;
      
        if( pInputDeviceState->bButtonDown )
			bDown = true;
    
		if( pInputDeviceState->bButtonSpace )
			bSpace = true;

		if( pInputDeviceState->bButtonEscape )
			bEscape = true;
    } 
 
	// Dan - Our input system, IInput, is set up more as a messaging system (you call when a button is pressed down or comes up_
	// whereas DirectInput does checks every frame. This translates between the two formats.
	if( bRight )
		glInput->pressDown(GKEY_RIGHT); // GAME
	else 
		glInput->pressUp(GKEY_RIGHT);

	if( bLeft )
		glInput->pressDown(GKEY_LEFT); // GAME
	else
		glInput->pressUp(GKEY_LEFT);

	if( bUp )
		glInput->pressDown(GKEY_UP); // GAME
	else 
		glInput->pressUp(GKEY_UP);

	if( bDown )
		glInput->pressDown(GKEY_DOWN); // GAME
	else 
		glInput->pressUp(GKEY_DOWN);

	if( bSpace )
		glInput->pressDown(GKEY_SPACE); // GAME
	else 
		glInput->pressUp(GKEY_SPACE);

	if( bEscape )
		glInput->pressDown(GKEY_ESC); // GAME
	else 
		glInput->pressUp(GKEY_ESC);
}




//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::Render()
{
	if ( !m_pUberCon || m_pUberCon->exState == EX_QUIT || !m_pd3dDevice )
		return E_FAIL;

    // Clear the viewport
	 m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
                         0x000000, 1.0f, 0L );

    // Begin the scene
    if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
    {
		m_dwPolyCount = 0;			// reset the polycount
		m_dwDeferredPolyCount = 0;	// reset the deferred count
        
		if ( m_pUberCon->exState == EX_MENU )
		{
			RenderMenu(); // render the menu
		}
		if ( m_pUberCon->exState == EX_GAME )
		{
			// this if statement is ridiculously long
			if ( m_pUberCon->gameControl->gameState == GAME_STARTING || m_pUberCon->gameControl->gameState == GAME_RACING || m_pUberCon->gameControl->gameState == GAME_PAUSE || m_pUberCon->gameControl->gameState == GAME_HIGHSCORES || m_pUberCon->gameControl->gameState == GAME_LEVEL_SUCCESS || m_pUberCon->gameControl->gameState == GAME_DEATH )
			{
				RenderSky();			// render the sky
				RenderUpdateView();		// update the view matrix according to the player
				RenderTrackArray();		// render the track
				RenderGameObjects();	// render objects last (so you can see through them)
			}
		}
       RenderText(); // render the text (does the HUD system)

        // End the scene.
        m_pd3dDevice->EndScene();
    }

    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: RenderMenu()
// Desc: Renders stats and help text to the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::RenderMenu()
{
	D3DCOLOR fontColor        = D3DCOLOR_ARGB(255,255,0,0); // initialize color
	TCHAR szMsg[MAX_PATH] = TEXT(""); // text for our D3Dfonts
	FLOAT fNextLine = -0.2f; // utility
	FLOAT fX = 0.2f; // where on the screen are we drawing? from -1.0 to 1.0
	FLOAT fY = 0.0f; // where on the screen are we drawing? from -1.0 to 1.0
	bool bDrawBox = false; // should we render a box?
	int iBoxType = 0;	// is the box a coin or obstacle?
	float fBoxX = 0.0f; // where is the box?
	float fBoxY = 0.0f;
	float fBoxZ = 20.5f;
	float fBoxScale = 1.0f; // how big is the box?
	float fTextScale = 0.08f; // how big is our text (0 to 1)

	m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE ); // don't do Z for the menu screen
	m_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, FALSE ); // same
	m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE ); // do lighting though
	m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE ); // same

	m_pd3dDevice->SetTexture( 0, m_pMenuTex );		// set texture to the menu texture
    m_pd3dDevice->SetFVF( D3DFVF_SCREENVERTEX );	 // set our vertex format to the screenvertex
    m_pd3dDevice->SetStreamSource( 0, m_pMenuVB, 0, sizeof(SCREENVERTEX) );		// stream from the menuVB
	m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP,0, 2 ); // draw a two-triangle trianglestrip

	m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE ); // reset settings
	m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE );
	m_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, TRUE );

	// DECIDE WHAT TEXT/BOX TO DRAW

	if ( m_pUberCon->menuControl->menuState == MENU_SELECTING )  // we're on the main menu
	{
		fontColor = D3DCOLOR_ARGB(255,255,0,0); // reset the color
		if ( m_pUberCon->menuControl->menuMainSelect == MENU_SELECT_TRAINING ) // we're currently on "training"
		{
			fontColor = D3DCOLOR_ARGB(255,255,255,0); // set this text to yellow instead of red
			fBoxX = 1.0f; // the box is next to "training"
			fBoxY = 1.3f;
			fBoxScale = 0.5f;
		}
		sprintf( szMsg, TEXT("Training") ); // get a TCHAR
		m_pBigFont->DrawTextScaled( fX, fNextLine, 0, fTextScale, fTextScale, fontColor, szMsg); // get a TCHAR
		fNextLine += 0.2f; // go to the next line

		fontColor = D3DCOLOR_ARGB(255,255,0,0); // reset the color
		if ( m_pUberCon->menuControl->menuMainSelect == MENU_SELECT_STARTGAME ) // we're currently selecting
		{
			fontColor = D3DCOLOR_ARGB(255,255,255,0); // set this text to yellow instead of red
			fBoxX = 1.0f; // the box is next to "start game"
			fBoxY = -0.5f;
			fBoxScale = 0.5f;
		}
		sprintf( szMsg, TEXT("Start Game") ); // get a TCHAR
		m_pBigFont->DrawTextScaled( fX, fNextLine, 0, fTextScale, fTextScale, fontColor, szMsg); // draw the text
		fNextLine += 0.2f; // go to the next line
		
		fontColor = D3DCOLOR_ARGB(255,255,0,0); // reset the color
		if ( m_pUberCon->menuControl->menuMainSelect == MENU_SELECT_HIGHSCORES ) // we're on "high scores"
		{
			fontColor = D3DCOLOR_ARGB(255,255,255,0);
			fBoxX = 1.0f;
			fBoxY = -2.2f;
			fBoxScale = 0.5f;
		}
		sprintf( szMsg, TEXT("High Scores") ); // get a TCHAR
		m_pBigFont->DrawTextScaled( fX, fNextLine, 0, fTextScale, fTextScale, fontColor, szMsg); // draw the text
		fNextLine += 0.2f; // go to the next line
		
		fontColor = D3DCOLOR_ARGB(255,255,0,0); // reset the color 
		if ( m_pUberCon->menuControl->menuMainSelect == MENU_SELECT_QUIT ) // we're on "quit"
		{
			fontColor = D3DCOLOR_ARGB(255,255,255,0);
			fBoxX = 1.0f;
			fBoxY = -4.0f;
			fBoxScale = 0.5f;
		}
		sprintf( szMsg, TEXT("Quit") ); // get the TCHAR
		m_pBigFont->DrawTextScaled( fX, fNextLine, 0, fTextScale, fTextScale, fontColor, szMsg); // draw the text
		fNextLine += 0.2f; // go to the next line

		bDrawBox = true; // yes, draw the box on the main menu
		iBoxType = 0; // make it a coin
		m_fBoxRotateX += 1.0f*PI*glTime->reportFrameInterval(); // rotate the box some every frame
		m_fBoxRotateY += 0.33f*PI*glTime->reportFrameInterval();  // rotate the box some every frame
		m_fBoxRotateZ += 0.66f*PI*glTime->reportFrameInterval(); // rotate the box some every frame
	}
	else if ( m_pUberCon->menuControl->menuState == MENU_GAMESELECT ) // we're selecting a level
	{
		FLOAT fStartX = -0.05f;
		FLOAT fDistX = 0.2f;
		FLOAT fNumY = 0.0f;
		fontColor = D3DCOLOR_ARGB(255,255,0,0); // set color to red
		sprintf( szMsg, TEXT("Choose Level:") ); // TCHAR of "Choose Level:"
		m_pBigFont->DrawTextScaled( -0.23f, -0.2f, 0, fTextScale, fTextScale, fontColor, szMsg); // write it too the screen

		const int iOffSet = -4; // we start four lower than the number currently selected
		int iNumber = 0;
		FLOAT fScale = 0.05f; // the scale for the text
		FLOAT fDigitOffset = 0; // do we need to offset for the fact that we've got more than one digit?
		fX = 0;
		for ( int i = 0; i < 9; i++ ) // do 9, one in the center, four on each side
		{
			iNumber = m_pUberCon->menuControl->levelStart + iOffSet + i; // what number are we drawing?
			if ( iNumber < 1 || iNumber > 20 )
				continue; // don't draw above the limit

			fontColor = D3DCOLOR_ARGB(255,255,0,0); // reset the color to red
			fScale = 0.05f; // reset the scale
			
			if ( iNumber > 9 )
				fDigitOffset = -0.05f; // need to shift over because we've got more than one digit
			else
				fDigitOffset = 0.0f; // we don't need to shift

			if ( i == 4 ) // this is the one in the center, and the one we're currently selecting
			{
				fontColor = D3DCOLOR_ARGB(255,255,255,0); // make it yellow
				fScale = 0.1f; // make it big
			} 
			else if ( i == 5 || i == 3 )
				fScale = 0.08f; // slightly smaller
			else if ( i == 6 || i == 2 )
				fScale = 0.07f; // slightly smaller
			else if ( i == 7 || i == 1 )
				fScale = 0.06f; // slightly smaller
			else if ( i == 8 || i == 0 ) 
				fScale = 0.05f; // slightly smaller
			
			fX = fStartX + fDistX * (iOffSet+i) + fDigitOffset; // compute the X value
			sprintf( szMsg, TEXT("%i"), iNumber); // write the text to screen
			m_pBigFont->DrawTextScaled( fX, fNumY, 0, fScale, fScale, fontColor, szMsg); // write text to screen
		}

		bDrawBox = true; // draw a box
		fBoxY = -4.0f; // draw it lower than the numbers
		iBoxType = 1; // draw an obstacle
		m_fBoxRotateX += (0.66)*PI*glTime->reportFrameInterval(); // spin it along one axis
		m_fBoxRotateY = (float)PI/4; // offset the rotation
		m_fBoxRotateZ = (float)PI/4; // offset the rotation
	}
	else if ( m_pUberCon->menuControl->menuState == MENU_HIGHSCORES ) // we're viewing the high scores
	{
		if ( m_pUberCon->menuControl->m_pHighScores )
		{
			IHighScores* pScores = m_pUberCon->menuControl->m_pHighScores; // get a quicker pointer
			
			fNextLine = -0.2f; // reset the nextline
			int iG = 255; // start at full green
			for ( int i = 0; i < NUM_IN_LIST; i++ )
			{	
				// write the initials
				fontColor = D3DCOLOR_ARGB(255,255,iG,0);
				sprintf( szMsg, TEXT("%s"), pScores->nameList[i].c_str() );
				m_pBigFont->DrawTextScaled( -0.25f, fNextLine, 0, fTextScale, fTextScale, fontColor, szMsg);

				// write the score
				fontColor = D3DCOLOR_ARGB(255,255,iG,0);
				sprintf( szMsg, TEXT("%i"), pScores->scoreList[i] );
				m_pBigFont->DrawTextScaled( 0.25f, fNextLine, 0, fTextScale, fTextScale, fontColor, szMsg);

				fNextLine += 0.2f; // go down a line
				iG -= 255/NUM_IN_LIST; // decrease the amount of green, thus going from yellow to red
			}
		}
	}

	if ( bDrawBox ) // we need to draw a box
	{
		// initialize matrices
		D3DXMATRIX matScale;
		D3DXMATRIX matTranslate;
		D3DXMATRIX matWorld;
		D3DXMATRIX matRotY;
		D3DXMATRIX matRotX;
		D3DXMATRIX matRotZ;
		D3DXMATRIX mat;
		D3DXMATRIX matRot;
		D3DXMATRIX matTransScale;
		D3DMATERIAL9 mtrl;

		// do the rotation NOTE: this code seems to be mathematically incorrect. don't have time/need to fix
		D3DXMatrixRotationX( &matRotX, m_fBoxRotateX );
		D3DXMatrixRotationY( &matRotY, m_fBoxRotateY );
		D3DXMatrixRotationZ( &matRotZ, m_fBoxRotateZ );
		D3DXMatrixMultiply( &matRot, &matRotX, &matRotY );
		D3DXMatrixMultiply( &matRot, &matRot, &matRotZ );

		D3DXMatrixScaling( &matScale, fBoxScale, fBoxScale, fBoxScale ); // scale the box
		D3DXMatrixTranslation( &matTranslate, fBoxX, fBoxY, fBoxZ ); // translate the box
		D3DXMatrixMultiply( &matTransScale, &matScale, &matTranslate ); // add both into a matrix

		// initialize a simple view matrix
		D3DXMATRIX matView;
		D3DXVECTOR3 vFromPt   = D3DXVECTOR3( 0.0f, 0.0f, -1.0f );
		D3DXVECTOR3 vLookatPt = D3DXVECTOR3( 0.0f, 0.0f, 0.0f ); 
		D3DXVECTOR3 vUpVec    = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
		D3DXMatrixLookAtLH( &matView, &vFromPt, &vLookatPt, &vUpVec );
		m_pd3dDevice->SetTransform( D3DTS_VIEW, &matView );

		D3DXMatrixMultiply( &mat, &matRot, &matTransScale ); // multiply to get the final world matrix

		int vertexCount = 0;
		m_pd3dDevice->SetTransform( D3DTS_WORLD, &mat ); // set the transform
		if ( iBoxType == 0 )
		{
			m_pd3dDevice->SetStreamSource( 0, m_pCoinVB, 0, sizeof(CUSTOMVERTEX) ); // use the coin VB
			vertexCount = COIN_VERT_COUNT; // use the amount of coin vertices
			D3DUtil_InitMaterial( mtrl, 1.0f, 1.0f, 1.0f, 0.8f ); // draw the coin at 20% transparency
			m_pd3dDevice->SetMaterial( &mtrl ); // set the material
			m_pd3dDevice->SetTexture(0,m_pCoinTex ); // set the coin texture
		}
		else 
		{
			m_pd3dDevice->SetStreamSource( 0, m_pObstacleVB, 0, sizeof(CUSTOMVERTEX) ); // use the obstacle VB
			vertexCount = OBSTACLE_VERT_COUNT; // set the number of obstacle vertices
			D3DUtil_InitMaterial( mtrl, 1.0f, 1.0f, 1.0f, 1.0f ); // draw it solid
			m_pd3dDevice->SetMaterial( &mtrl ); // set the material
			m_pd3dDevice->SetTexture(0,m_pObstacleTex ); // set the obstacle texture
		}
		m_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX ); // set the right vertex format
		m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, vertexCount/3 ); // draw the triangles
		m_dwPolyCount+=vertexCount/3; // debug only
		
	}
	
	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: RenderUpdateView()
// Desc: Update the view from the player's perspective
//-----------------------------------------------------------------------------

HRESULT CMyD3DApplication::RenderUpdateView()
{
	CBaseEntity** pArray = NULL; // quick pointer to the CBaseEntity array
	CBaseEntity* pEnt = NULL; // quick pointer to a given CBaseEntity
	int arrayCount = 0; // how many are in the array?
	bool bFound = false; // initialize

	pArray = m_pUberCon->gameControl->entArray;
	arrayCount = m_pUberCon->gameControl->entCount;

	// go through the entire array
	for ( int i = 0; i < arrayCount; i++ )
	{
		pEnt = pArray[i]; // get the current entity

		if ( m_pUberCon->gameControl->gameState == GAME_STARTING || ( m_pUberCon->gameControl->gameState == GAME_PAUSE && m_pUberCon->gameControl->prePauseState == GAME_STARTING ) )
		{
			// if we're starting, we need to get the CCamera (should be second entity)
			if ( pEnt->pvars->obj_type == OBJ_CAMERA )
			{
				bFound = true;
				break;
			}
		}
		else
		{
			// otherwise, we need to get the CPlayer (should be first entity)
			if ( pEnt->isPlayer() )
			{
				bFound = true;
				break;
			}
		}
	}

	if ( !bFound )
		return E_FAIL;

	// pEnt is now our viewer
	Vec3d up = glTrack->getNormal(pEnt->pvars->origin.x,pEnt->pvars->origin.y,pEnt->pvars->origin.z,RANGE,FRAC); // get the normal from the GTrackArray

	float fOffX = glTrack->getXOffsetFromXYZ(pEnt->pvars->origin.x, pEnt->pvars->origin.y, pEnt->pvars->origin.z, RANGE, FRAC ); // get the X offset from the GTrackArray
	float fOffY = glTrack->getYOffsetFromXYZ(pEnt->pvars->origin.x, pEnt->pvars->origin.y, pEnt->pvars->origin.z, RANGE, FRAC ); // get the Y offset from the GTrackArray
	float fAtX = pEnt->pvars->origin.x + pEnt->pvars->orientation.x; // the point to look at 
	float fAtY = pEnt->pvars->origin.y + pEnt->pvars->orientation.y; // the point to look at
	float fAtZ = pEnt->pvars->origin.z + pEnt->pvars->orientation.z; // the point to look at 
	
	if ( glTrack->bCreated && (glTrack->warpType == WARP_PERPSPIRAL || glTrack->warpType == WARP_AXIALSPIRAL || glTrack->warpType == WARP_CURVY ) )
	{
		// we need to look at a point that takes into account the warping of the GTrackArray
		fAtX = pEnt->pvars->origin.x;
		fAtY = pEnt->pvars->origin.y;
		fAtZ = pEnt->pvars->origin.z + 50; // look at where we'll be 50 points down the track
		fAtX += glTrack->getXOffsetFromXYZ(pEnt->pvars->origin.x, pEnt->pvars->origin.y, fAtZ, RANGE, FRAC ); // get the offset for the point we're looking at
		fAtY += glTrack->getYOffsetFromXYZ(pEnt->pvars->origin.x, pEnt->pvars->origin.y, fAtZ, RANGE, FRAC ); // get the offset for the point we're looking at
	}

	// set up the view matrix
	D3DXMATRIX matView;
	D3DXVECTOR3 vFromPt   = D3DXVECTOR3(pEnt->pvars->origin.x + fOffX, pEnt->pvars->origin.y + fOffY,pEnt->pvars->origin.z);
	D3DXVECTOR3 vLookatPt = D3DXVECTOR3( fAtX, fAtY, fAtZ );
	D3DXVECTOR3 vUpVec    = D3DXVECTOR3( up.x, up.y, up.z );
	D3DXMatrixLookAtLH( &matView, &vFromPt, &vLookatPt, &vUpVec );
	m_pd3dDevice->SetTransform( D3DTS_VIEW, &matView );

	// the sky needs to know where we are
	m_fSkyX = pEnt->pvars->origin.x + fOffX;
	m_fSkyY = pEnt->pvars->origin.y + fOffY;
	m_fSkyZ = pEnt->pvars->origin.z;

	// need to store this information
	m_vPoint = vLookatPt - vFromPt;
	m_vPos = vFromPt;

	return S_OK;
}



//-----------------------------------------------------------------------------
// Name: RenderGameObjects()
// Desc: Renders mines/coins
//-----------------------------------------------------------------------------

HRESULT CMyD3DApplication::RenderGameObjects()
{
	// initialize matrices
	D3DXMATRIX matScale;
    D3DXMATRIX matTranslate;
	D3DXMATRIX matWorld;
    D3DXMATRIX matRotY;
    D3DXMATRIX matRotX;
	D3DXMATRIX matRotZ;
    D3DXMATRIX mat;
	D3DXMATRIX matRot;
	D3DXMATRIX matTransScale;
	
	// create a temporary vertex buffer pointer
	LPDIRECT3DVERTEXBUFFER9 pTempVB = NULL;
	int	vertexCount = 0;

	CBaseEntity** pArray = NULL; // quick pointer to the array of CBaseEntities
	CBaseEntity* pEnt = NULL; // quick pointer to a given CBaseEntity
	int arrayCount = 0; // size of the array
	
	pArray = m_pUberCon->gameControl->entArray; // load it up
	arrayCount = m_pUberCon->gameControl->entCount; // and the size

	// scaling and offsets
	float fScaleX = 1; 
	float fScaleY = 1;
	float fScaleZ = 1;
	float fOffX = 0;
	float fOffY = 0;

	// go through every entity
	for ( int i = arrayCount-1; i >= 0; i-- )
	{
		pEnt = pArray[i]; // get the current entity

		if ( pEnt->pvars->ent_flags & ENT_VISIBLE ) // this entity can be seen, must be drawn
		{
			D3DMATERIAL9 mtrl; // create a material

			if ( pEnt->pvars->obj_type == OBJ_OBSTACLE )
			{
				pTempVB = m_pObstacleVB; // use the obstacle vertexbuffer
				vertexCount = OBSTACLE_VERT_COUNT; // sync the vertex count
	
				//m_pd3dDevice->GetMaterial(&mtrl);
				D3DUtil_InitMaterial( mtrl, 1.0f, 1.0f, 1.0f, 1.0f ); // init the material
				m_pd3dDevice->SetMaterial( &mtrl ); // set the material

				m_pd3dDevice->SetTexture(0,m_pObstacleTex); // set the texture to the obstacles'

				// set the scaling
				fScaleX = SCALE_OBSTACLE; 
				fScaleY = SCALE_OBSTACLE;
				fScaleZ = SCALE_OBSTACLE;
			}
			else if ( pEnt->pvars->obj_type == OBJ_COIN )
			{
				pTempVB = m_pCoinVB; // use the coin obstacle vertexbuffer
				vertexCount = COIN_VERT_COUNT; // sync the vertex count

				//m_pd3dDevice->GetMaterial(&mtrl);
				D3DUtil_InitMaterial( mtrl, 1.0f, 1.0f, 1.0f, 0.8f ); // init the material
				m_pd3dDevice->SetMaterial( &mtrl ); // set the material

				m_pd3dDevice->SetTexture(0,m_pCoinTex); // use the coin texture

				// set the scaling
				fScaleX = SCALE_COIN;
				fScaleY = SCALE_COIN;
				fScaleZ = SCALE_COIN;
			}
			else
				continue;

			// get the offsets from the GTrackArray
			fOffX = glTrack->getXOffsetFromXYZ(pEnt->pvars->origin.x, pEnt->pvars->origin.y, pEnt->pvars->origin.z, RANGE, FRAC );
			fOffY = glTrack->getYOffsetFromXYZ(pEnt->pvars->origin.x, pEnt->pvars->origin.y, pEnt->pvars->origin.z, RANGE, FRAC );
			
			// if the obstacle is behind the player, or out of range, we don't need to draw
			D3DXVECTOR3 vToObj = D3DXVECTOR3(pEnt->pvars->origin.x+fOffX,pEnt->pvars->origin.y+fOffY,pEnt->pvars->origin.z) - m_vPos;
			if ( (D3DXVec3Dot( &m_vPoint, &vToObj ) < 0) || (D3DXVec3Length(&vToObj) > MAX_DRAW_DISTANCE) )
			{
				m_dwDeferredPolyCount+=vertexCount/3; // debug only
				continue; // don't draw this
			}

			// do the rotation matrix math NOTE: this isn't right somehow. don't have the time/need to figure out the math
			D3DXMatrixRotationX( &matRotX, pEnt->pvars->orientation.x );
			D3DXMatrixRotationY( &matRotY, pEnt->pvars->orientation.y );
			D3DXMatrixRotationZ( &matRotZ, pEnt->pvars->orientation.z );
			D3DXMatrixMultiply( &matRot, &matRotX, &matRotY );
			D3DXMatrixMultiply( &matRot, &matRot, &matRotZ );
	
			// do the scaling and translation
			D3DXMatrixScaling( &matScale, fScaleX, fScaleY, fScaleZ ); 
            D3DXMatrixTranslation( &matTranslate, pEnt->pvars->origin.x + fOffX, pEnt->pvars->origin.y + fOffY, pEnt->pvars->origin.z );
            D3DXMatrixMultiply( &matTransScale, &matScale, &matTranslate );

			// combine them into the world matrix
			D3DXMatrixMultiply( &mat, &matRot, &matTransScale );

			// do the drawing
			m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE ); // we're using alpha
			m_pd3dDevice->SetTransform( D3DTS_WORLD, &mat ); // set the new world matrix
			m_pd3dDevice->SetStreamSource( 0, pTempVB, 0, sizeof(CUSTOMVERTEX) ); // set the vertex buffer to use
			m_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX ); // set the vertex format
			m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, vertexCount/3 ); // draw the vertex buffer's contents
			m_dwPolyCount+=vertexCount/3; // debug only

		}
	}
	
	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: RenderTrackArray()
// Desc: Renders the track.
//-----------------------------------------------------------------------------

HRESULT CMyD3DApplication::RenderTrackArray()
{
	if ( !glTrack->bCreated )
		return S_OK; // don't draw if there's nothing to draw

    D3DXMATRIX mat;
	
	HRESULT hr;

	// go through every quad in the track
	for ( int i = 0; i < glTrack->segNum; i++ )
	{
		D3DXMatrixIdentity( &mat );

		// init the material/set the color
		D3DMATERIAL9 mtrl;

		// if the track segment is behind us or out of range, we don't need to draw it
		D3DXVECTOR3 vToObj = D3DXVECTOR3(glTrack->track[i].first[0], glTrack->track[i].first[1], glTrack->track[i].first[2]) - m_vPos;
		if ( ((D3DXVec3Dot( &m_vPoint, &vToObj ) < 0) && (D3DXVec3Length(&vToObj) > TRACK_SEG_LENGTH)) || (D3DXVec3Length(&vToObj) > MAX_DRAW_DISTANCE) )
		{
			m_dwDeferredPolyCount+=2; // debug only
			continue; // skip this track quad
		}

		//m_pd3dDevice->GetMaterial(&mtrl);
		//D3DUtil_InitMaterial( mtrl, glTrack->track[i].R, glTrack->track[i].G, glTrack->track[i].B  );
		D3DUtil_InitMaterial( mtrl, 1.0f,1.0f,1.0f,1.0f ); // set everything to full bright
		m_pd3dDevice->SetMaterial( &mtrl ); // set the material
		
		// get the texture from the texture index
		switch ( glTrack->track[i].textureIndex )
		{
		case TEXTURE_TRACKMAIN:
			m_pd3dDevice->SetTexture(0,m_pTrackMainTex);
			break;
		case TEXTURE_LINE1:
			m_pd3dDevice->SetTexture(0,m_pLine1Tex);
			break;
		case TEXTURE_LINE2:
			m_pd3dDevice->SetTexture(0,m_pLine2Tex);
			break;
		case TEXTURE_SIDELOW:
			m_pd3dDevice->SetTexture(0,m_pSideLowTex);
			break;
		case TEXTURE_SIDEHIGH:
			m_pd3dDevice->SetTexture(0,m_pSideHighTex);
			break;
		case TEXTURE_CEILING:
			m_pd3dDevice->SetTexture(0,m_pCeilingTex);
			break;
		}

		// set up the draw
		m_pd3dDevice->SetTransform( D3DTS_WORLD, &mat );
		m_pd3dDevice->SetStreamSource( 0, m_pTrackQuadVB, 0, sizeof(CUSTOMVERTEX) );
		m_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX );
		m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, i*6, 2 ); // 2 triangles * 3 vertices = 6, draws 2 triangles after the offset, i*6
		m_dwPolyCount+=2; // debug only

	}

	return S_OK;

}

//-----------------------------------------------------------------------------
// Name: RenderSky()
// Desc: Renders sky and stuff
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::RenderSky()
{
	// set up matrices
	D3DXMATRIX mat;
	D3DXMATRIX matTrans, matRot;

	// translate to the player's last known position, hopefully updated this frame
	D3DXMatrixTranslation( &matTrans, m_fSkyX, m_fSkyY, m_fSkyZ );
	D3DXMatrixRotationY( &matRot, -PI/2 ); // HACK: our current skymap looks better in this direction.
	D3DXMatrixMultiply( &mat, &matRot, &matTrans ); 
	m_pd3dDevice->SetTransform( D3DTS_WORLD, &mat ); // use this world matrix

	m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE ); // NO Z
	// Some cards do not disable writing to Z when 
	// D3DRS_ZENABLE is FALSE. So do it explicitly
	m_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, FALSE );
	m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE ); // no lighting

	// Render the skybox
	m_pSkyBox->Render( m_pd3dDevice );
	m_dwPolyCount += 4;

	// Restore the render states
	m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );
	m_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, TRUE);
	m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE );


	return S_OK;
}


//-----------------------------------------------------------------------------
// Name: RenderText()
// Desc: Renders stats and help text to the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::RenderText()
{
	D3DCOLOR fontColor        = D3DCOLOR_ARGB(255,255,255,0);
	TCHAR szMsg[MAX_PATH] = TEXT("");

	// get the player entity
	CBaseEntity* pEnt = NULL;
	if (  m_pUberCon->exState == EX_GAME && m_pUberCon->gameControl->entArray != NULL )
	{
		for (int i = 0; i < m_pUberCon->gameControl->entCount; i++ )
		{
			if ( m_pUberCon->gameControl->entArray[i]->isPlayer() )
			{
				pEnt = m_pUberCon->gameControl->entArray[i];
				break;
			}
		}
	}

	// Output display stats
	FLOAT fNextLine = 40.0f; 
/*	// DEBUG: FRAMERATE, DEVICE
	lstrcpy( szMsg, m_strDeviceStats );
	fNextLine -= 20.0f;
	m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

	lstrcpy( szMsg, m_strFrameStats );
	fNextLine -= 20.0f;
	m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );
*/
	fNextLine = 40.0f; 

/*	// THIS IS DEBUG TEXT
	if ( pEnt )
	{
		sprintf( szMsg, TEXT("Time: %0.2f" ), glTime->getTime() );
		fNextLine += 20.0f;
		m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

		sprintf( szMsg, TEXT("Distance: %0.2f" ), pEnt->pvars->origin.z );
		fNextLine += 20.0f;
		m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

		sprintf( szMsg, TEXT("Speed: %0.2f" ), pEnt->pvars->speed );
		fNextLine += 20.0f;
		m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

		sprintf( szMsg, TEXT("Coins: %i" ), m_pUberCon->gameControl->coins );
		fNextLine += 20.0f;
		m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

		sprintf( szMsg, TEXT("Score: %i" ), m_pUberCon->gameControl->score );
		fNextLine += 20.0f;
		m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

		sprintf( szMsg, TEXT("Lives: %i" ), m_pUberCon->gameControl->lives );
		fNextLine += 20.0f;
		m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

		sprintf( szMsg, TEXT("PolyCount: %i" ), m_dwPolyCount );
		fNextLine += 20.0f;
		m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

		sprintf( szMsg, TEXT("Skipped Polys: %i" ), m_dwDeferredPolyCount );
		fNextLine += 20.0f;
		m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );
	
		float fRatio = (float)m_dwPolyCount/(m_dwDeferredPolyCount+m_dwPolyCount);
		sprintf( szMsg, TEXT("Drawn/Total: %0.2f" ), fRatio );
		fNextLine += 20.0f;
		m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );
	}
*/
	if ( pEnt )
	{
		// draw the Heads Up Display -- 

		float fTextScale = 0.1f;

		// "Coins: "
		fontColor = D3DCOLOR_ARGB(255,255,50,0);
		sprintf( szMsg, TEXT("Coins: %i"), m_pUberCon->gameControl->coins );
		m_pBigFont->DrawTextScaled( -0.95f, 0.80f, 0.0f, fTextScale, fTextScale, fontColor, szMsg);

		// "Score: "
		fontColor = D3DCOLOR_ARGB(255,255,50,0);
		sprintf( szMsg, TEXT("Score: %i"), m_pUberCon->gameControl->score );
		m_pBigFont->DrawTextScaled( -0.3f, 0.80f, 0.0f, fTextScale, fTextScale, fontColor, szMsg);

		// "Lives: "
		fontColor = D3DCOLOR_ARGB(255,255,50,0);
		sprintf( szMsg, TEXT("Lives: %i"), m_pUberCon->gameControl->lives );
		m_pBigFont->DrawTextScaled( 0.55f, 0.80f, 0.0f, fTextScale, fTextScale, fontColor, szMsg);
	
		// "Speed: ", at the low speeds, it's red, at the high speeds, it's yellow
		int iG = (int)255*(pEnt->pvars->speed-MIN_PLAYER_SPEED)/(MAX_PLAYER_SPEED-MIN_PLAYER_SPEED); // get the amount of green to add to the color
		if ( pEnt->pvars->speed < MIN_PLAYER_SPEED )
			iG = 0;
		fontColor = D3DCOLOR_ARGB(255,255,iG,0);
		sprintf( szMsg, TEXT("Speed: %0.0f"), pEnt->pvars->speed );
		m_pBigFont->DrawTextScaled( -0.95f, -0.80f, 0.0f, fTextScale, fTextScale, fontColor, szMsg);

		// "Time: "
		fontColor = D3DCOLOR_ARGB(255,255,50,0);
		if ( m_pUberCon->gameControl->gameState == GAME_PAUSE && !(m_pUberCon->gameControl->prePauseState == GAME_STARTING) )
			sprintf( szMsg, TEXT("Time: %0.2f"), m_pUberCon->gameControl->pauseBeginTime - m_pUberCon->gameControl->levelStartTime - m_pUberCon->gameControl->pauseTime); // get the time during pause
		else if ( m_pUberCon->gameControl->gameState == GAME_STARTING || (m_pUberCon->gameControl->gameState == GAME_PAUSE && m_pUberCon->gameControl->prePauseState == GAME_STARTING) )
			sprintf( szMsg, TEXT("Time: %0.2f"), 0.0f); // at the start of the game, time is always 0
		else if ( m_pUberCon->gameControl->gameState == GAME_HIGHSCORES || m_pUberCon->gameControl->gameState == GAME_DEATH || m_pUberCon->gameControl->gameState == GAME_LEVEL_SUCCESS ) 
			sprintf( szMsg, TEXT("Time: %0.2f"), m_pUberCon->gameControl->levelEndTime - m_pUberCon->gameControl->levelStartTime - m_pUberCon->gameControl->pauseTime); // get the time after the level has stopped
		else
			sprintf( szMsg, TEXT("Time: %0.2f"), glTime->getTime() - m_pUberCon->gameControl->levelStartTime - m_pUberCon->gameControl->pauseTime); // get the time if we're playing normally
		m_pBigFont->DrawTextScaled( -0.95f, -0.98f, 0.0f, fTextScale, fTextScale, fontColor, szMsg);

		// "Level: "
		fontColor = D3DCOLOR_ARGB(255,255,50,0);
		sprintf( szMsg, TEXT("Level: %i"), m_pUberCon->gameControl->level );
		m_pBigFont->DrawTextScaled( 0.25f, -0.98f, 0.0f, fTextScale, fTextScale, fontColor, szMsg);

		// WE'RE PAUSED, WRITE PAUSE TEXT
		if ( m_pUberCon->gameControl->gameState == GAME_PAUSE )
		{
			// "PAUSED"
			fontColor = D3DCOLOR_ARGB(255,255,0,0);
			sprintf( szMsg, TEXT("PAUSED") );
			m_pBigFont->DrawTextScaled( -0.35f, -0.3f, 0.0f, 0.2f, 0.2f, fontColor, szMsg);

			// "Press Space to continue..."
			fontColor = D3DCOLOR_ARGB(255,255,255,0);
			sprintf( szMsg, TEXT("Press Space to continue...") );
			m_pBigFont->DrawTextScaled( -0.6f, 0.1f, 0.0f, 0.1f, 0.1f, fontColor, szMsg);

			// "Press Escape to exit..."
			fontColor = D3DCOLOR_ARGB(255,255,255,0);
			sprintf( szMsg, TEXT("Press Escape to exit...") );
			m_pBigFont->DrawTextScaled( -0.5f, 0.3f, 0.0f, 0.1f, 0.1f, fontColor, szMsg);
		}

		// WE'RE SELECTING OUR INITIALS
		if ( m_pUberCon->gameControl->gameState == GAME_HIGHSCORES )
		{
			// "You got a high score!"
			fontColor = D3DCOLOR_ARGB(255,255,255,0);
			sprintf( szMsg, TEXT("You got a high score!") );
			m_pBigFont->DrawTextScaled( -0.4f, -0.3f, 0.0f, 0.1f, 0.1f, fontColor, szMsg);

			// FIRST LETTER
			if ( m_pUberCon->gameControl->nameState == 0 )
				fontColor = D3DCOLOR_ARGB(255,255,255,0); // yellow if we're currently editing this one
			else
				fontColor = D3DCOLOR_ARGB(255,255,0,0); // red otherwise
			sprintf( szMsg, TEXT("%c"), m_pUberCon->gameControl->name[0] );
			m_pBigFont->DrawTextScaled( -0.3f, -0.1f, 0.0f, 0.2f, 0.2f, fontColor, szMsg);

			// SECOND LETTER
			if ( m_pUberCon->gameControl->nameState == 1 )
				fontColor = D3DCOLOR_ARGB(255,255,255,0); // yellow if we're currently editing this one
			else
				fontColor = D3DCOLOR_ARGB(255,255,0,0); // red otherwise
			sprintf( szMsg, TEXT("%c"), m_pUberCon->gameControl->name[1] );
			m_pBigFont->DrawTextScaled( -0.1f, -0.1f, 0.0f, 0.2f, 0.2f, fontColor, szMsg);

			// THIRD LETTER
			if ( m_pUberCon->gameControl->nameState == 2 )
				fontColor = D3DCOLOR_ARGB(255,255,255,0); // yellow if we're currently editing this one
			else
				fontColor = D3DCOLOR_ARGB(255,255,0,0); // red otherwise
			sprintf( szMsg, TEXT("%c"), m_pUberCon->gameControl->name[2] );
			m_pBigFont->DrawTextScaled( 0.1f, -0.1f, 0.0f, 0.2f, 0.2f, fontColor, szMsg);
		}

		// WE'VE FINISHED A LEVEL
		if ( m_pUberCon->gameControl->gameState == GAME_LEVEL_SUCCESS )
		{
			// "LEVEL SUCCESS!!!"
			fontColor = D3DCOLOR_ARGB(255,255,255,0);
			sprintf( szMsg, TEXT("LEVEL SUCCESS!!!") );
			m_pBigFont->DrawTextScaled( -0.4f, -0.3f, 0.0f, 0.1f, 0.1f, fontColor, szMsg);

			// "Score: "
			fontColor = D3DCOLOR_ARGB(255,255,0,0);
			sprintf( szMsg, TEXT("Score: %i"), m_pUberCon->gameControl->score );
			m_pBigFont->DrawTextScaled( -0.4f, 0.0f, 0.0f, 0.15f, 0.15f, fontColor, szMsg);

		}

		// WE'RE DEAD
		if ( m_pUberCon->gameControl->gameState == GAME_DEATH )
		{
			// "GAME OVER."
			fontColor = D3DCOLOR_ARGB(255,255,0,0);
			sprintf( szMsg, TEXT("GAME OVER") );
			m_pBigFont->DrawTextScaled( -0.4f, -0.3f, 0.0f, 0.15f, 0.15f, fontColor, szMsg);

			// "Press Space or Escape to go back to the menu."
			fontColor = D3DCOLOR_ARGB(255,255,255,0);
			sprintf( szMsg, TEXT("Press Space or Escape to go back to the menu.") );
			m_pBigFont->DrawTextScaled( -0.5f, 0.0f, 0.0f, 0.05f, 0.05f, fontColor, szMsg);

		}
	}

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: Pause()
// Desc: Called in to toggle the pause state of the app.
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::Pause( bool bPause )
{
    // Get access to the list of semantically-mapped input devices
    // to zero the state of all InputDeviceState structs.  This is needed
    // because when using DISCL_FOREGROUND, the action mapper will not 
    // record actions when the focus switches, for example if a dialog appears.
    // This causes a problem when a button held down when loosing focus, and let
    // go when the focus is lost.  The app will not record that the button 
    // has been let go, so the state will be incorrect when focus returns.  
    // To fix this either use DISCL_BACKGROUND or zero the state when 
    // loosing focus.
    CInputDeviceManager::DeviceInfo* pDeviceInfos;
    DWORD dwNumDevices;
    m_pInputDeviceManager->GetDevices( &pDeviceInfos, &dwNumDevices );

    for( DWORD i=0; i<dwNumDevices; i++ )
    {
        InputDeviceState* pInputDeviceState = (InputDeviceState*) pDeviceInfos[i].pParam;
        ZeroMemory( pInputDeviceState, sizeof(InputDeviceState) );
    }

    CD3DApplication::Pause( bPause );
}




//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Overrrides the main WndProc, so the sample can do custom message
//       handling (e.g. processing mouse, keyboard, or menu commands).
//-----------------------------------------------------------------------------
LRESULT CMyD3DApplication::MsgProc( HWND hWnd, UINT msg, WPARAM wParam,
                                    LPARAM lParam )
{
    switch( msg )
    {
        case WM_PAINT:
        {
            if( m_bLoadingApp )
            {
                // Draw on the window tell the user that the app is loading
                // TODO: change as needed
                HDC hDC = GetDC( hWnd );
                TCHAR strMsg[MAX_PATH];
                wsprintf( strMsg, TEXT("Loading... Please wait") );
                RECT rct;
                GetClientRect( hWnd, &rct );
                DrawText( hDC, strMsg, -1, &rct, DT_CENTER|DT_VCENTER|DT_SINGLELINE );
                ReleaseDC( hWnd, hDC );
            }
            break;
        }

        case WM_COMMAND:
        {
            switch( LOWORD(wParam) )
            {
                case IDM_CONFIGINPUT:
                    m_UserInput.bDoConfigureInput = TRUE;
                    break;

                case IDM_CHANGEDEVICE:
                    m_UserInput.bDoConfigureDisplay = TRUE;
                    return 0; // Don't hand off to parent
            }
            break;
        }

    }

    return CD3DApplication::MsgProc( hWnd, msg, wParam, lParam );
}




//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Invalidates device objects.  Paired with RestoreDeviceObjects()
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InvalidateDeviceObjects()
{
    // TODO: Cleanup any objects created in RestoreDeviceObjects()
    m_pFont->InvalidateDeviceObjects();
	m_pBigFont->InvalidateDeviceObjects();
	m_pSkyBox->InvalidateDeviceObjects();
    SAFE_RELEASE( m_pDIConfigSurface );

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Paired with InitDeviceObjects()
//       Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.  
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::DeleteDeviceObjects()
{
    // Cleanup any objects created in InitDeviceObjects()
    m_pFont->DeleteDeviceObjects();
	m_pBigFont->DeleteDeviceObjects();
	m_pSkyBox->Destroy();

    SAFE_RELEASE( m_pVB );
	SAFE_RELEASE( m_pCoinVB );
	SAFE_RELEASE( m_pObstacleVB );
	SAFE_RELEASE( m_pTrackQuadVB );
	SAFE_RELEASE( m_pMenuVB );
	SAFE_RELEASE( m_pCoinTex );
	SAFE_RELEASE( m_pObstacleTex );
	SAFE_RELEASE( m_pTrackMainTex );
	SAFE_RELEASE( m_pLine1Tex );
	SAFE_RELEASE( m_pLine2Tex );
	SAFE_RELEASE( m_pSideLowTex );
	SAFE_RELEASE( m_pSideHighTex );
	SAFE_RELEASE( m_pCeilingTex );
	SAFE_RELEASE( m_pMenuTex );
	
	if ( glTrack->bCreated )
		glTrack->Cleanup();
	glTrack->bCreated = false;

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Paired with OneTimeSceneInit()
//       Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::FinalCleanup()
{
    // Perform any final cleanup needed
    // Cleanup D3D font
    SAFE_DELETE( m_pFont );
	SAFE_DELETE( m_pBigFont );
	SAFE_DELETE( m_pSkyBox );

    // Cleanup DirectInput
    CleanupDirectInput();

    // Write the settings to the registry
    WriteSettings();

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: CleanupDirectInput()
// Desc: Cleanup DirectInput 
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::CleanupDirectInput()
{
    if( NULL == m_pInputDeviceManager )
        return;

    // Get access to the list of semantically-mapped input devices
    // to delete all InputDeviceState structs
    CInputDeviceManager::DeviceInfo* pDeviceInfos;
    DWORD dwNumDevices;
    m_pInputDeviceManager->GetDevices( &pDeviceInfos, &dwNumDevices );

    for( DWORD i=0; i<dwNumDevices; i++ )
    {
        InputDeviceState* pInputDeviceState = (InputDeviceState*) pDeviceInfos[i].pParam;
        SAFE_DELETE( pInputDeviceState );
        pDeviceInfos[i].pParam = NULL;
    }

    // Cleanup DirectX input objects
    SAFE_DELETE( m_pInputDeviceManager );

}

//-----------------------------------------------------------------------------
// Name: LoadStaticTrackGeometry()
//
//
//
//-----------------------------------------------------------------------------

HRESULT CMyD3DApplication::LoadStaticTrackGeometry()
{
	HRESULT hr = S_OK;

	if ( !glTrack->bCreated )
		return DXTRACE_ERR( "CreateVertexBuffer", hr );

	int vertexCount = glTrack->segNum * 6; // two triangles times three vertices times number of quads

	// TRACK QUAD BUFFER
	if ( m_pTrackQuadVB )
		SAFE_RELEASE( m_pTrackQuadVB );

	if( FAILED( hr = m_pd3dDevice->CreateVertexBuffer( vertexCount*sizeof(CUSTOMVERTEX),
													0, D3DFVF_CUSTOMVERTEX,
													D3DPOOL_MANAGED, &m_pTrackQuadVB, NULL ) ) )
		return DXTRACE_ERR( "CreateVertexBuffer", hr );

	CUSTOMVERTEX* pVertices;
	// Add the triangle into the buffer
	if( FAILED( hr = m_pTrackQuadVB->Lock( 0, 0, (VOID**)&pVertices, 0 ) ) )
		return DXTRACE_ERR( "Lock", hr );
	
	int vertIndex = 0;
	for ( int i = 0; i < glTrack->segNum; i++ )
	{

		pVertices[vertIndex].position = D3DXVECTOR3(glTrack->track[i].first);
		pVertices[vertIndex].normal   = D3DXVECTOR3(glTrack->track[i].firstNorm);
		pVertices[vertIndex].fU		  = 0.0f;
		pVertices[vertIndex].fV		  = 0.0f;
		vertIndex++;
		pVertices[vertIndex].position = D3DXVECTOR3(glTrack->track[i].third);
		pVertices[vertIndex].normal   = D3DXVECTOR3(glTrack->track[i].thirdNorm);
		pVertices[vertIndex].fU		  = 1.0f;
		pVertices[vertIndex].fV		  = 1.0f;
		vertIndex++;
		pVertices[vertIndex].position = D3DXVECTOR3(glTrack->track[i].fourth);
		pVertices[vertIndex].normal   = D3DXVECTOR3(glTrack->track[i].fourthNorm);
		pVertices[vertIndex].fU		  = 0.0f;
		pVertices[vertIndex].fV		  = 1.0f;
		vertIndex++;

		pVertices[vertIndex].position = D3DXVECTOR3(glTrack->track[i].first);
		pVertices[vertIndex].normal   = D3DXVECTOR3(glTrack->track[i].firstNorm);
		pVertices[vertIndex].fU		  = 0.0f;
		pVertices[vertIndex].fV		  = 0.0f;
		vertIndex++;
		pVertices[vertIndex].position = D3DXVECTOR3(glTrack->track[i].second);
		pVertices[vertIndex].normal   = D3DXVECTOR3(glTrack->track[i].secondNorm);
		pVertices[vertIndex].fU		  = 1.0f;
		pVertices[vertIndex].fV		  = 0.0f;
		vertIndex++;
		pVertices[vertIndex].position = D3DXVECTOR3(glTrack->track[i].third);
		pVertices[vertIndex].normal   = D3DXVECTOR3(glTrack->track[i].thirdNorm);
		pVertices[vertIndex].fU		  = 1.0f;
		pVertices[vertIndex].fV		  = 1.0f;
		vertIndex++;

	}
	m_pTrackQuadVB->Unlock();

	m_iTrackVertexCount = vertexCount;

	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: InitGameVertexBuffers()
//
//
//
//-----------------------------------------------------------------------------

HRESULT CMyD3DApplication::InitGameVertexBuffers()
{
	CUSTOMVERTEX* pVertices;
	HRESULT hr;
	
	// OBSTACLE BUFFER
	if( FAILED( hr = m_pd3dDevice->CreateVertexBuffer( OBSTACLE_VERT_COUNT*sizeof(CUSTOMVERTEX),
                                                  0, D3DFVF_CUSTOMVERTEX,
                                                  D3DPOOL_MANAGED, &m_pObstacleVB, NULL ) ) )
        return DXTRACE_ERR( "CreateVertexBuffer", hr );

	// COIN BUFFER
	if( FAILED( hr = m_pd3dDevice->CreateVertexBuffer( COIN_VERT_COUNT*sizeof(CUSTOMVERTEX),
                                                  0, D3DFVF_CUSTOMVERTEX,
                                                  D3DPOOL_MANAGED, &m_pCoinVB, NULL ) ) )
        return DXTRACE_ERR( "CreateVertexBuffer", hr );


   
	// OBSTACLE LOCK
    if( FAILED( hr = m_pObstacleVB->Lock( 0, 0, (VOID**)&pVertices, 0 ) ) )
        return DXTRACE_ERR( "Lock", hr );
	
	// FRONT TRIANGLE 1
	pVertices[0].position = D3DXVECTOR3(-1.0f,1.0f,-1.0f);
	pVertices[0].normal = D3DXVECTOR3(0.0f,0.0f,-1.0f);
	pVertices[0].fU = 0.0f;
	pVertices[0].fV = 0.0f;
	pVertices[1].position = D3DXVECTOR3(1.0f,-1.0f,-1.0f);
	pVertices[1].normal = D3DXVECTOR3(0.0f,0.0f,-1.0f);
	pVertices[1].fU = 1.0f;
	pVertices[1].fV = 1.0f;
	pVertices[2].position = D3DXVECTOR3(-1.0f,-1.0f,-1.0f);
	pVertices[2].normal = D3DXVECTOR3(0.0f,0.0f,-1.0f);
	pVertices[2].fU = 0.0f;
	pVertices[2].fV = 1.0f;
	
	// FRONT TRIANGLE 2
	pVertices[3].position = D3DXVECTOR3(-1.0f,1.0f,-1.0f);
	pVertices[3].normal = D3DXVECTOR3(0.0f,0.0f,-1.0f);
	pVertices[3].fU = 0.0f;
	pVertices[3].fV = 0.0f;
	pVertices[4].position = D3DXVECTOR3(1.0f,1.0f,-1.0f);
	pVertices[4].normal = D3DXVECTOR3(0.0f,0.0f,-1.0f);
	pVertices[4].fU = 1.0f;
	pVertices[4].fV = 0.0f;
	pVertices[5].position = D3DXVECTOR3(1.0f,-1.0f,-1.0f);
	pVertices[5].normal = D3DXVECTOR3(0.0f,0.0f,-1.0f);
	pVertices[5].fU = 1.0f;
	pVertices[5].fV = 1.0f;

	// RIGHT TRIANGLE 1
	pVertices[6].position = D3DXVECTOR3(1.0f,1.0f,-1.0f);
	pVertices[6].normal = D3DXVECTOR3(1.0f,0.0f,0.0f);
	pVertices[6].fU = 0.0f;
	pVertices[6].fV = 0.0f;
	pVertices[7].position = D3DXVECTOR3(1.0f,-1.0f,1.0f);
	pVertices[7].normal = D3DXVECTOR3(1.0f,0.0f,0.0f);
	pVertices[7].fU = 1.0f;
	pVertices[7].fV = 1.0f;
	pVertices[8].position = D3DXVECTOR3(1.0f,-1.0f,-1.0f);
	pVertices[8].normal = D3DXVECTOR3(1.0f,0.0f,0.0f);
	pVertices[8].fU = 0.0f;
	pVertices[8].fV = 1.0f;

	// RIGHT TRIANGLE 2
	pVertices[9].position = D3DXVECTOR3(1.0f,1.0f,-1.0f);
	pVertices[9].normal = D3DXVECTOR3(1.0f,0.0f,0.0f);
	pVertices[9].fU = 0.0f;
	pVertices[9].fV = 0.0f;
	pVertices[10].position = D3DXVECTOR3(1.0f,1.0f,1.0f);
	pVertices[10].normal = D3DXVECTOR3(1.0f,0.0f,0.0f);
	pVertices[10].fU = 1.0f;
	pVertices[10].fV = 0.0f;
	pVertices[11].position = D3DXVECTOR3(1.0f,-1.0f,1.0f);
	pVertices[11].normal = D3DXVECTOR3(1.0f,0.0f,0.0f);
	pVertices[11].fU = 1.0f;
	pVertices[11].fV = 1.0f;

	// BACK TRIANGLE 1
	pVertices[12].position = D3DXVECTOR3(1.0f,1.0f,1.0f);
	pVertices[12].normal = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[12].fU = 0.0f;
	pVertices[12].fV = 0.0f;
	pVertices[13].position = D3DXVECTOR3(-1.0f,-1.0f,1.0f);
	pVertices[13].normal = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[13].fU = 1.0f;
	pVertices[13].fV = 1.0f;
	pVertices[14].position = D3DXVECTOR3(1.0f,-1.0f,1.0f);
	pVertices[14].normal = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[14].fU = 0.0f;
	pVertices[14].fV = 1.0f;

	// BACK TRIANGLE 2
	pVertices[15].position = D3DXVECTOR3(1.0f,1.0f,1.0f);
	pVertices[15].normal = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[15].fU = 0.0f;
	pVertices[15].fV = 0.0f;
	pVertices[16].position = D3DXVECTOR3(-1.0f,1.0f,1.0f);
	pVertices[16].normal = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[16].fU = 1.0f;
	pVertices[16].fV = 0.0f;
	pVertices[17].position = D3DXVECTOR3(-1.0f,-1.0f,1.0f);
	pVertices[17].normal = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[17].fU = 1.0f;
	pVertices[17].fV = 1.0f;

	// LEFT TRIANGLE 1
	pVertices[18].position = D3DXVECTOR3(-1.0f,1.0f,1.0f);
	pVertices[18].normal = D3DXVECTOR3(-1.0f,0.0f,0.0f);
	pVertices[18].fU = 0.0f;
	pVertices[18].fV = 0.0f;
	pVertices[19].position = D3DXVECTOR3(-1.0f,-1.0f,-1.0f);
	pVertices[19].normal = D3DXVECTOR3(-1.0f,0.0f,0.0f);
	pVertices[19].fU = 1.0f;
	pVertices[19].fV = 1.0f;
	pVertices[20].position = D3DXVECTOR3(-1.0f,-1.0f,1.0f);
	pVertices[20].normal = D3DXVECTOR3(-1.0f,0.0f,0.0f);
	pVertices[20].fU = 0.0f;
	pVertices[20].fV = 1.0f;

	// LEFT TRIANGLE 2
	pVertices[21].position = D3DXVECTOR3(-1.0f,1.0f,1.0f);
	pVertices[21].normal = D3DXVECTOR3(-1.0f,0.0f,0.0f);
	pVertices[21].fU = 0.0f;
	pVertices[21].fV = 0.0f;
	pVertices[22].position = D3DXVECTOR3(-1.0f,1.0f,-1.0f);
	pVertices[22].normal = D3DXVECTOR3(-1.0f,0.0f,0.0f);
	pVertices[22].fU = 1.0f;
	pVertices[22].fV = 0.0f;
	pVertices[23].position = D3DXVECTOR3(-1.0f,-1.0f,-1.0f);
	pVertices[23].normal = D3DXVECTOR3(-1.0f,0.0f,0.0f);
	pVertices[23].fU = 1.0f;
	pVertices[23].fV = 1.0f;

	// TOP TRIANGLE 1
	pVertices[24].position = D3DXVECTOR3(-1.0f,1.0f,1.0f);
	pVertices[24].normal = D3DXVECTOR3(0.0f,1.0f,0.0f);
	pVertices[24].fU = 0.0f;
	pVertices[24].fV = 0.0f;
	pVertices[25].position = D3DXVECTOR3(1.0f,1.0f,-1.0f);
	pVertices[25].normal = D3DXVECTOR3(0.0f,1.0f,0.0f);
	pVertices[25].fU = 1.0f;
	pVertices[25].fV = 1.0f;
	pVertices[26].position = D3DXVECTOR3(-1.0f,1.0f,-1.0f);
	pVertices[26].normal = D3DXVECTOR3(0.0f,1.0f,0.0f);
	pVertices[26].fU = 0.0f;
	pVertices[26].fV = 1.0f;

	// TOP TRIANGLE 2
	pVertices[27].position = D3DXVECTOR3(-1.0f,1.0f,1.0f);
	pVertices[27].normal = D3DXVECTOR3(0.0f,1.0f,0.0f);
	pVertices[27].fU = 0.0f;
	pVertices[27].fV = 0.0f;
	pVertices[28].position = D3DXVECTOR3(1.0f,1.0f,1.0f);
	pVertices[28].normal = D3DXVECTOR3(0.0f,1.0f,0.0f);
	pVertices[28].fU = 1.0f;
	pVertices[28].fV = 0.0f;
	pVertices[29].position = D3DXVECTOR3(1.0f,1.0f,-1.0f);
	pVertices[29].normal = D3DXVECTOR3(0.0f,1.0f,0.0f);
	pVertices[29].fU = 1.0f;
	pVertices[29].fV = 1.0f;

	// BOTTOM TRIANGLE 1
	pVertices[30].position = D3DXVECTOR3(1.0f,-1.0f,1.0f);
	pVertices[30].normal = D3DXVECTOR3(0.0f,-1.0f,0.0f);
	pVertices[30].fU = 0.0f;
	pVertices[30].fV = 0.0f;
	pVertices[31].position = D3DXVECTOR3(-1.0f,-1.0f,-1.0f);
	pVertices[31].normal = D3DXVECTOR3(0.0f,-1.0f,0.0f);
	pVertices[31].fU = 1.0f;
	pVertices[31].fV = 1.0f;
	pVertices[32].position = D3DXVECTOR3(1.0f,-1.0f,-1.0f);
	pVertices[32].normal = D3DXVECTOR3(0.0f,-1.0f,0.0f);
	pVertices[32].fU = 0.0f;
	pVertices[32].fV = 1.0f;

	// BOTTOM TRIANGLE 2
	pVertices[33].position = D3DXVECTOR3(1.0f,-1.0f,1.0f);
	pVertices[33].normal = D3DXVECTOR3(0.0f,-1.0f,0.0f);
	pVertices[33].fU = 0.0f;
	pVertices[33].fV = 0.0f;
	pVertices[34].position = D3DXVECTOR3(-1.0f,-1.0f,1.0f);
	pVertices[34].normal = D3DXVECTOR3(0.0f,-1.0f,0.0f);
	pVertices[34].fU = 1.0f;
	pVertices[34].fV = 0.0f;
	pVertices[35].position = D3DXVECTOR3(-1.0f,-1.0f,-1.0f);
	pVertices[35].normal = D3DXVECTOR3(0.0f,-1.0f,0.0f);
	pVertices[35].fU = 1.0f;
	pVertices[35].fV = 1.0f;

	 m_pVB->Unlock(); // END OBSTACLE

	 // COIN LOCK
    if( FAILED( hr = m_pCoinVB->Lock( 0, 0, (VOID**)&pVertices, 0 ) ) )
        return DXTRACE_ERR( "Lock", hr );

	// FRONT TRIANGLE 1
	pVertices[0].position = D3DXVECTOR3(-1.0f,1.0f,-1.0f);
	pVertices[0].normal = D3DXVECTOR3(0.0f,0.0f,-1.0f);
	pVertices[0].fU = 0.0f;
	pVertices[0].fV = 0.0f;
	pVertices[1].position = D3DXVECTOR3(1.0f,-1.0f,-1.0f);
	pVertices[1].normal = D3DXVECTOR3(0.0f,0.0f,-1.0f);
	pVertices[1].fU = 1.0f;
	pVertices[1].fV = 1.0f;
	pVertices[2].position = D3DXVECTOR3(-1.0f,-1.0f,-1.0f);
	pVertices[2].normal = D3DXVECTOR3(0.0f,0.0f,-1.0f);
	pVertices[2].fU = 0.0f;
	pVertices[2].fV = 1.0f;
	
	// FRONT TRIANGLE 2
	pVertices[3].position = D3DXVECTOR3(-1.0f,1.0f,-1.0f);
	pVertices[3].normal = D3DXVECTOR3(0.0f,0.0f,-1.0f);
	pVertices[3].fU = 0.0f;
	pVertices[3].fV = 0.0f;
	pVertices[4].position = D3DXVECTOR3(1.0f,1.0f,-1.0f);
	pVertices[4].normal = D3DXVECTOR3(0.0f,0.0f,-1.0f);
	pVertices[4].fU = 1.0f;
	pVertices[4].fV = 0.0f;
	pVertices[5].position = D3DXVECTOR3(1.0f,-1.0f,-1.0f);
	pVertices[5].normal = D3DXVECTOR3(0.0f,0.0f,-1.0f);
	pVertices[5].fU = 1.0f;
	pVertices[5].fV = 1.0f;

	// RIGHT TRIANGLE 1
	pVertices[6].position = D3DXVECTOR3(1.0f,1.0f,-1.0f);
	pVertices[6].normal = D3DXVECTOR3(1.0f,0.0f,0.0f);
	pVertices[6].fU = 0.0f;
	pVertices[6].fV = 0.0f;
	pVertices[7].position = D3DXVECTOR3(1.0f,-1.0f,1.0f);
	pVertices[7].normal = D3DXVECTOR3(1.0f,0.0f,0.0f);
	pVertices[7].fU = 1.0f;
	pVertices[7].fV = 1.0f;
	pVertices[8].position = D3DXVECTOR3(1.0f,-1.0f,-1.0f);
	pVertices[8].normal = D3DXVECTOR3(1.0f,0.0f,0.0f);
	pVertices[8].fU = 0.0f;
	pVertices[8].fV = 1.0f;

	// RIGHT TRIANGLE 2
	pVertices[9].position = D3DXVECTOR3(1.0f,1.0f,-1.0f);
	pVertices[9].normal = D3DXVECTOR3(1.0f,0.0f,0.0f);
	pVertices[9].fU = 0.0f;
	pVertices[9].fV = 0.0f;
	pVertices[10].position = D3DXVECTOR3(1.0f,1.0f,1.0f);
	pVertices[10].normal = D3DXVECTOR3(1.0f,0.0f,0.0f);
	pVertices[10].fU = 1.0f;
	pVertices[10].fV = 0.0f;
	pVertices[11].position = D3DXVECTOR3(1.0f,-1.0f,1.0f);
	pVertices[11].normal = D3DXVECTOR3(1.0f,0.0f,0.0f);
	pVertices[11].fU = 1.0f;
	pVertices[11].fV = 1.0f;

	// BACK TRIANGLE 1
	pVertices[12].position = D3DXVECTOR3(1.0f,1.0f,1.0f);
	pVertices[12].normal = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[12].fU = 0.0f;
	pVertices[12].fV = 0.0f;
	pVertices[13].position = D3DXVECTOR3(-1.0f,-1.0f,1.0f);
	pVertices[13].normal = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[13].fU = 1.0f;
	pVertices[13].fV = 1.0f;
	pVertices[14].position = D3DXVECTOR3(1.0f,-1.0f,1.0f);
	pVertices[14].normal = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[14].fU = 0.0f;
	pVertices[14].fV = 1.0f;

	// BACK TRIANGLE 2
	pVertices[15].position = D3DXVECTOR3(1.0f,1.0f,1.0f);
	pVertices[15].normal = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[15].fU = 0.0f;
	pVertices[15].fV = 0.0f;
	pVertices[16].position = D3DXVECTOR3(-1.0f,1.0f,1.0f);
	pVertices[16].normal = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[16].fU = 1.0f;
	pVertices[16].fV = 0.0f;
	pVertices[17].position = D3DXVECTOR3(-1.0f,-1.0f,1.0f);
	pVertices[17].normal = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[17].fU = 1.0f;
	pVertices[17].fV = 1.0f;

	// LEFT TRIANGLE 1
	pVertices[18].position = D3DXVECTOR3(-1.0f,1.0f,1.0f);
	pVertices[18].normal = D3DXVECTOR3(-1.0f,0.0f,0.0f);
	pVertices[18].fU = 0.0f;
	pVertices[18].fV = 0.0f;
	pVertices[19].position = D3DXVECTOR3(-1.0f,-1.0f,-1.0f);
	pVertices[19].normal = D3DXVECTOR3(-1.0f,0.0f,0.0f);
	pVertices[19].fU = 1.0f;
	pVertices[19].fV = 1.0f;
	pVertices[20].position = D3DXVECTOR3(-1.0f,-1.0f,1.0f);
	pVertices[20].normal = D3DXVECTOR3(-1.0f,0.0f,0.0f);
	pVertices[20].fU = 0.0f;
	pVertices[20].fV = 1.0f;

	// LEFT TRIANGLE 2
	pVertices[21].position = D3DXVECTOR3(-1.0f,1.0f,1.0f);
	pVertices[21].normal = D3DXVECTOR3(-1.0f,0.0f,0.0f);
	pVertices[21].fU = 0.0f;
	pVertices[21].fV = 0.0f;
	pVertices[22].position = D3DXVECTOR3(-1.0f,1.0f,-1.0f);
	pVertices[22].normal = D3DXVECTOR3(-1.0f,0.0f,0.0f);
	pVertices[22].fU = 1.0f;
	pVertices[22].fV = 0.0f;
	pVertices[23].position = D3DXVECTOR3(-1.0f,-1.0f,-1.0f);
	pVertices[23].normal = D3DXVECTOR3(-1.0f,0.0f,0.0f);
	pVertices[23].fU = 1.0f;
	pVertices[23].fV = 1.0f;

	// TOP TRIANGLE 1
	pVertices[24].position = D3DXVECTOR3(-1.0f,1.0f,1.0f);
	pVertices[24].normal = D3DXVECTOR3(0.0f,1.0f,0.0f);
	pVertices[24].fU = 0.0f;
	pVertices[24].fV = 0.0f;
	pVertices[25].position = D3DXVECTOR3(1.0f,1.0f,-1.0f);
	pVertices[25].normal = D3DXVECTOR3(0.0f,1.0f,0.0f);
	pVertices[25].fU = 1.0f;
	pVertices[25].fV = 1.0f;
	pVertices[26].position = D3DXVECTOR3(-1.0f,1.0f,-1.0f);
	pVertices[26].normal = D3DXVECTOR3(0.0f,1.0f,0.0f);
	pVertices[26].fU = 0.0f;
	pVertices[26].fV = 1.0f;

	// TOP TRIANGLE 2
	pVertices[27].position = D3DXVECTOR3(-1.0f,1.0f,1.0f);
	pVertices[27].normal = D3DXVECTOR3(0.0f,1.0f,0.0f);
	pVertices[27].fU = 0.0f;
	pVertices[27].fV = 0.0f;
	pVertices[28].position = D3DXVECTOR3(1.0f,1.0f,1.0f);
	pVertices[28].normal = D3DXVECTOR3(0.0f,1.0f,0.0f);
	pVertices[28].fU = 1.0f;
	pVertices[28].fV = 0.0f;
	pVertices[29].position = D3DXVECTOR3(1.0f,1.0f,-1.0f);
	pVertices[29].normal = D3DXVECTOR3(0.0f,1.0f,0.0f);
	pVertices[29].fU = 1.0f;
	pVertices[29].fV = 1.0f;

	// BOTTOM TRIANGLE 1
	pVertices[30].position = D3DXVECTOR3(1.0f,-1.0f,1.0f);
	pVertices[30].normal = D3DXVECTOR3(0.0f,-1.0f,0.0f);
	pVertices[30].fU = 0.0f;
	pVertices[30].fV = 0.0f;
	pVertices[31].position = D3DXVECTOR3(-1.0f,-1.0f,-1.0f);
	pVertices[31].normal = D3DXVECTOR3(0.0f,-1.0f,0.0f);
	pVertices[31].fU = 1.0f;
	pVertices[31].fV = 1.0f;
	pVertices[32].position = D3DXVECTOR3(1.0f,-1.0f,-1.0f);
	pVertices[32].normal = D3DXVECTOR3(0.0f,-1.0f,0.0f);
	pVertices[32].fU = 0.0f;
	pVertices[32].fV = 1.0f;

	// BOTTOM TRIANGLE 2
	pVertices[33].position = D3DXVECTOR3(1.0f,-1.0f,1.0f);
	pVertices[33].normal = D3DXVECTOR3(0.0f,-1.0f,0.0f);
	pVertices[33].fU = 0.0f;
	pVertices[33].fV = 0.0f;
	pVertices[34].position = D3DXVECTOR3(-1.0f,-1.0f,1.0f);
	pVertices[34].normal = D3DXVECTOR3(0.0f,-1.0f,0.0f);
	pVertices[34].fU = 1.0f;
	pVertices[34].fV = 0.0f;
	pVertices[35].position = D3DXVECTOR3(-1.0f,-1.0f,-1.0f);
	pVertices[35].normal = D3DXVECTOR3(0.0f,-1.0f,0.0f);
	pVertices[35].fU = 1.0f;
	pVertices[35].fV = 1.0f;

	 m_pVB->Unlock(); // END COIN



	return S_OK;
}


