
#ifndef __CBASEENTITY_H_
#define __CBASEENTITY_H_

#include <math.h>
#include "Vec3.h"
#include "IEntVars.h"
#include "globalconst.h"
#include "stateflagdefs.h"
#include "ITime.h"
#include "IInput.h"

//-----------------------------------------------------------
// Class CBaseEntity
//
//	This is the class from which all game objects are derived.
// Gameplay mostly consists of querying each CBaseEntity subclass
// each frame, and then drawing/playing the sounds that result.
//
//
//
//	UPDATES:
//	2/4/04 - Daniel: first pass, data
//	2/6/04 - Daniel: moved data to IEntVars
//	2/9/04 - Daniel: Physics
//	2/11/04 - Daniel: physics now uses global frame intervals, not an entvars value
//  2/24/04 - Daniel: angular velocity that actually makes sense/works
//
//-----------------------------------------------------------

// global variables
extern ITime* glTime;
extern IInput* glInput;

class CBaseEntity
{
public:
	// Functions
	virtual void	Think() { return; } // logic every frame...
	virtual void	Spawn() { return; } // begin
	virtual void	Remove() { return; }
	virtual void	Hit( CBaseEntity* victim ) { return; } // called when collision detects that we've hit another entity
	virtual bool	isPlayer() { return false; } // assume it's not

	// I can't foresee a reason to overload this in subclasses, as the purpose
	// is ONLY to update origin/orientation, physics things that change EVERY FRAME
	// rather than in think intervals
	virtual void	Physics()  
	{
		float interval = glTime->reportFrameInterval();
		if ( pvars->ent_flags & ENT_MOVABLE )
		{
			if ( pvars->speed != 0 ) // save ourselves some trouble 
			{
				// distance = velocity * time
				// origin = origin + ( velocity direction * floats/s * seconds 
				pvars->origin = pvars->origin + pvars->velocity * pvars->speed * interval;
			}
		}
		if ( pvars->ang_vel.Length() != 0 )
			{
				// angular distance = angular velocity * time
				if( isPlayer() )
				{
					// HACK: if we're the player, we handle orientation like it's our view angle
					// the discrepancy is due to the openGL->D3D switch, but it would be clumsy anyway. live and learn.
					float curAng = (float)atan2(pvars->orientation.z,pvars->orientation.x);
					curAng += pvars->ang_vel.x * interval;
					pvars->orientation.x = (float)cos(curAng);
					pvars->orientation.z = (float)sin(curAng);
				}
				else
				{
					// whereas, if we're not the player, orientation is the rotation along each axis
					pvars->orientation = pvars->orientation + pvars->ang_vel * interval;
				}
			}
	}

	// Data
	IEntVars	*pvars;
};



#endif