

#include "CCamera.h"
#include "IEntVars.h"
#include "CBaseEntity.h"
#include "IRand.h"
#include "globalconst.h"
#include "stateflagdefs.h"
#include <math.h>

extern IRand* glRand;

void CCamera :: Spawn()
{
	// set it up
	pvars->obj_type				= OBJ_CAMERA; // object type
	pvars->player_state			= PL_NOT_PLAYER; // we're at the beginning state
	pvars->ent_flags			= 0;			 // initialize
	pvars->radius				= RADIUS_PLAYER; // radius for collision
	pvars->speed				= 0;
	pvars->thinktime			= glTime->getTime();
	pvars->velocity.z			= 0; 
	pvars->velocity.x			= 0; 
	pvars->velocity.y			= 0;
	pvars->orientation = pvars->velocity;
	pvars->orientation.z = 1;
	pvars->switch_state			= SWITCH_NO;

	camState = CAMSTATE_INIT;
	camType = -1;


}

//
// Think
//	... called by: CGameControl->StartThink
//	... description: called every frame while the camera is active
//
void CCamera :: Think()
{
	if ( camState == CAMSTATE_INIT )
	{
		int nextType = glRand->Rand() % NUM_CAMTYPES; // get a random type
		switch ( nextType )
		{
		case CAMTYPE_OVERLOOK:
			InitOverlook(); // we're a higher, sweeping camera
			break;
		case CAMTYPE_STARTER:
			InitStarter(); // we're a more gentle, falling camera
			break;
		}
	}

	if ( camState == CAMSTATE_ACTIVE )
	{
		// equation: add the total distance divided by the total time, taking into acount our frame interval
		//pvars->origin = pvars->origin + ((targOrigin - pvars->origin) * FRAME_INTERVAL * camTime);
		if ( glTime->reportFrameInterval() < 0.1f ) // HACK: this nixes the time distortion caused by Text To Speech, specifically, the "Level 5!" part
			pvars->origin = pvars->origin + ((targOrigin - pvars->origin) * glTime->reportFrameInterval() * camTime);
	}

	if ( glTime->getTime() > camEndTime )
		camState = CAMSTATE_DEAD; // we're not going to be used anymore

	//pvars->thinktime = glTime->getTime() + FRAME_INTERVAL;
	pvars->thinktime = glTime->getTime();
}

//
// InitOverlook()
//	... called by: Think
//	... description: sets up the first type of camera, this one sweeps backwards into the starting position
//
void CCamera :: InitOverlook()
{
	// start far above and ahead of the playerstart point
	pvars->origin.x = TRACK_2_X;
	pvars->origin.y = 100;
	pvars->origin.z = 300;

	// go back to the player start
	targOrigin.x = TRACK_2_X;
	targOrigin.y = 0;
	targOrigin.z = 0;

	camTime = 3; // arbitrary
	camEndTime = glTime->getTime() + camTime;

	camState = CAMSTATE_ACTIVE; // we're now active
	camType = CAMTYPE_OVERLOOK; // set our type
}

//
// InitStarter()
//	... called by: Think
//	... description: sets up the second type of camera, this one just goes down a little
//
void CCamera :: InitStarter()
{
	// start us a little bit higher, a little bit behind
	pvars->origin.x = TRACK_2_X;
	pvars->origin.y = 30;
	pvars->origin.z = -30;

	// to the player start
	targOrigin.x = TRACK_2_X;
	targOrigin.y = 0;
	targOrigin.z = 0;

	camTime = 3; // arbitrary
	camEndTime = glTime->getTime() + camTime; // at this point we should be at targOrigin

	camState = CAMSTATE_ACTIVE; // set our state as active
	camType = CAMTYPE_STARTER; // set the type
}