
#include "CCoin.h"
#include "CBaseEntity.h"
#include "CGameControl.h"
#include "globalconst.h"
#include "stateflagdefs.h"
#include "soundfunc.h"
#include "Vec3.h"
#include "ITime.h"

// -------------------------------------
// CCoin - functions definitions
//
// ... see header for more information
//
//	UPDATES:
//		- 2/9/04, Daniel: created, sort of finished (first pass)
//
// -------------------------------------

//
// Default Constructor
//
//
CCoin :: CCoin()
{
	//Spawn();
}

//
// Origin Constructor
//
//
CCoin :: CCoin( Vec3d orig )
{
	//Spawn();
	//pvars->origin = orig;
}

//
// Spawn
//	... called by: constructor (why not just in constructor? because
//			there's always a chance we'd want to reset.)
//	... description: initialize pvars to what this object requires
//
void CCoin :: Spawn()
{
	// object-specific traits
	pvars->obj_type				= OBJ_COIN; // object type
	pvars->player_state			= PL_NOT_PLAYER; // this is no player
	pvars->switch_state			= SWITCH_NO;
	pvars->ent_flags			= 0;			 // initialize
	pvars->ent_flags			= ((( pvars->ent_flags | ENT_COLLIDABLE )
									| ENT_VISIBLE )
									| ENT_AUDIBLE ); // flags
	pvars->ang_vel.y			= 0.66*PI;			// spinning (depends on whether implemented)
	pvars->ang_vel.x			= 0.1*PI;
	pvars->ang_vel.z			= 0;
	pvars->radius				= RADIUS_COIN; // radius for collision
	pvars->speed				= 0;
	pvars->thinktime			= glTime->getTime();
	
	// origin handled once, when created, and outside this function
}


//
// Think
//	... called by: CGameControl->Think()
//	...	description: do game logic at intervals, check for updates/state change, etc.
//
void CCoin :: Think()
{
	if ( !( pvars->ent_flags & ENT_DEAD ) ) // we're not dead, right?
	{
		// nothing to do here yet...


		pvars->thinktime = glTime->getTime() + NT_COIN; // think next
	}
}

//
// Hit
//	... called by: CGameControl->CheckCollisions()
//	... description: do game logic when a object collides with this one
//
void CCoin :: Hit( CBaseEntity* other )
{
	// if we're not a player, do nothing... that technically shouldn't happen,
	// and other things can't pick up coins anyway.
	if ( other->isPlayer() ) // the player hit us
	{
		IPlayStationarySound(NULL,30,SOUND_EVENT_HITCOIN);
		//CGameControl::CoinPickedUp( other ); // alert game that player picked up coin
		Remove(); // destroy this coin
	}
}

//
// Remove
//	... called by: any, but in this case probably Hit()
//	... description: removes the coin from interaction
//
void CCoin :: Remove()
{
		// we must disappear...
		pvars->ent_flags = (((pvars->ent_flags ^ ENT_COLLIDABLE )
								^ ENT_VISIBLE )
								^ ENT_AUDIBLE );
		// and die.
		pvars->ent_flags = pvars->ent_flags | ENT_DEAD;
}

// end file