#include "IFileIO.h"

//-----------------------------------------------------------
// IFileIO - functions definitions
//
//	... see header for more information
//
//
//	UPDATES:
//	3/24/04 - Matt: created (decided it would be better
//					programming to split IFileIO into a .h
//					and a .cpp)
//  3/24/04 - Matt: added default constructor and added OpenFile function
//
//-----------------------------------------------------------


//Default Constructor
IFileIO::IFileIO()
{
	
}

//
//Constructor
//	... description: opens specified file, tests for error, and creates object IFileIO

IFileIO :: IFileIO( string filePath )
{
	//opens filePath
	f.open(filePath.c_str());	
	
	//test for error when opening filePath
	if(!f)							
	{
		//usually put a return 0 here, but function is void, so had
		//to use abort()-- this function is not really necessary
		abort();		//aborts program if there was error opening file
	}

}

//
//OpenFile
//	... called by: any
//	... description: opens filePath to be read from
void IFileIO :: OpenFile( string filePath )
{
	f.open(filePath.c_str());	//opens filePath

	if(!f)
		abort();		

}

//
//ReadLine()
//	... called by: any
//	... description: Reads the current line of the input text file. 
//					 It then uses a simple tokenizer to break up the 
//					 string into words, placing them into a vector<string>, 
//					 and returns the vector. The next call of ReadLine() 
//					 automatically reads the next line.
//

vector<string> IFileIO :: ReadLine()
{
	string line;				
	
	if (!f.eof())
	{	
		getline(f, line);

		string buf;				// Buffer string for tokenizer
	    stringstream ss(line);	// Insert line into a stream

		vector<string> tokens;	// Create vector to hold words in line

	    while (ss >> buf)		//tokenizes line into words and adds them to tokens
		    tokens.push_back(buf);
	
	
		return tokens;

	}
	
}


//
//isFileEnded()
//	... called by: any
//	... description: Returns true if file is at the end, false otherwise.
//

bool IFileIO :: isFileEnded()
{
	if (f.eof())
		return true;
	else
		return false;

}

//
//CloseFile()
//	... called by: any
//	... description: Closes file stream.
//

void IFileIO :: CloseFile()
{
	f.close();
}