
#ifndef __IHIGHSCORES_H_
#define __IHIGHSCORES_H_

#include "IFileIO.h"
#include "OFileIO.h"
#include "ITextToSpeech.h"

//-----------------------------------------------------------
// Class IHighScores
//
//	This class reads a text file of current high scores and 
//  stores them into two arrays: nameList, which is a string
//  array and holds the top names, and scoreList, which is an
//  int array and holds the top scores. A score can be checked
//  whether it makes the top score list, and if so, the name and 
//  score is added to two arrays and the text file is updated. 
//  A congratulatory message (or a sorry message if the score was
//  not high enough) is played. Finally, this class allows the
//  text-to-speech reading of the whole high score list.
//
//
//	UPDATES:
//	3/24/04 - Matt: created, constructors, fillLists() completed
//	3/25/04 - Matt: writeList() completed
//	3/29/04 - Matt: speakList() first draft completed
//	3/31/04 - Matt: speakList() completed, checkScore() completed
//	4/07/04 - Matt: added a checkScore that only takes an int, made
//					the database arrays public so other classes could 
//					use the info stored in them
//
//
//-----------------------------------------------------------

const int NUM_IN_LIST = 5;


class IHighScores
{
public:

	// constructors
	IHighScores();
	IHighScores(string filePath);

	// functions
	void writeList();
	void writeList(string fileName);
	void speakList();
	bool checkScore(int playerScore);
	void checkScore(string playerName, int playerScore);

	// data
	string d_fileName;
	string nameList[NUM_IN_LIST];
	int scoreList[NUM_IN_LIST];

private:
	// internal functions
	void fillLists();
	void writeListToFile(string fileName);

	
};



#endif

		


