
#include "IScriptSegment.h"
#include "IScriptLine.h"
#include "globalconst.h"

// -------------------------------------
// IScriptSegment - functions definitions
//
// ... see header for more information
//
//	UPDATES:
//		2/16/04	- Daniel: created
//		2/17/04	- Daniel: changed array format
//
// -------------------------------------

const int START_CAP_SCRIPT = 5;

//
// constructor
//
//
IScriptSegment :: IScriptSegment( )
{
	capacity = START_CAP_SCRIPT; // starter out
	lineArray = new IScriptLine*[START_CAP_SCRIPT]; // link it up
	lineCount = 0; // none yet
	difficulty = 0;
}

//
// Cleanup
//
//
void IScriptSegment :: Cleanup( )
{
	for ( int i = 0; i < lineCount; i++ )
	{
		delete lineArray[i];
	}
	delete lineArray;
}

//
// AddLine
//	... called by: ILevelLoader::Load()
//	... description: adds an IScriptLine into the array
//
void IScriptSegment :: AddLine( IScriptLine *s )
{
	if ( lineCount >= capacity ) // do we need to increase capacity?
		IncreaseCap();

	lineArray[lineCount] = s; // lineCount represents right above last index
	lineCount++; // up it
}

//
// IncreaseCap
//	... called by: AddLine
//	... description: increases array capacity
//
void IScriptSegment :: IncreaseCap()
{
	
	capacity += 5; // increase 5 at a time... overhead isn't too much of a worry here
	IScriptLine** tempArray = new IScriptLine*[capacity]; 
	for ( int i = 0; i < lineCount; i++ )
		tempArray[i] = lineArray[i]; // copy over pointers

	delete [] lineArray;
	lineArray = tempArray; // change the pointer
}