#include "soundfunc.h"
#include "Vec3.h"
#include "IEntVars.h"
#include "CBaseEntity.h"
#include "fmod.h"


// -------------------------------------
// soundfunc - functions definitions
//
// ... see header for more information
//
//	UPDATES:
//		- 2/21/04, Eden: created
//		- 2/22/04, Daniel: in the course of getting it to compile: no soundfunc, 
//					can't switch floats, need entCount in IInitializeSounds
//		- 2/24/04, Eden: changed the way samples are loaded to reduce channels
//		- 3/15/04, Eden: changed IUpdateSounds and IInitializeSounds to only load 
//					channels when objects are within "hearing range"
//		- 3/21/04, Eden: added IPlayStationarySounds and IPlay3DSounds
//		- 3/22/04, Daniel: needed to initialize has_sound is IEntVars
//		- 3/23/04, Eden: added IPauseSounds, IUnpauseSounds, and IResetSounds
//		- 3/31/04, Daniel: with D3D graphics, don't need to switch x values anymore
//		- 4/5/04,  Eden: added IInitializeMenuSounds, IInitializeRacingSounds, 
//					IPlayMenuSounds, IFreeRacingSamples, IFreeMenuSamples, ICloseFmod
//		- 4/13/04, Eden: added more sound samples, took out some of the pops and buzzes
//		- 4/20/04, Eden: equalized the sound levels in the menu and the race, took out
//					some of the buzzes and pops, added more samples
//		- 4/21/04, Eden: removed bugs where crashed obstacles would still make sounds,
//					added more samples
//		- 4/26/04, Eden: cleaned up the code a bit, put in some more sound samples
// -------------------------------------

//Pointers to samples
//Racing samples
FSOUND_SAMPLE *sample_coin_1;
FSOUND_SAMPLE *sample_coin_2;
FSOUND_SAMPLE *sample_coin_3;
FSOUND_SAMPLE *sample_obstacle_1;
FSOUND_SAMPLE *sample_obstacle_2;
FSOUND_SAMPLE *sample_obstacle_3;
FSOUND_SAMPLE *sample_hitobstacle;
FSOUND_SAMPLE *sample_hitcoin;
FSOUND_SAMPLE *sample_death;
FSOUND_SAMPLE *sample_levelend;
FSOUND_SAMPLE *sample_levelstart;
FSOUND_SAMPLE *sample_warning;
FSOUND_SAMPLE *sample_leftlane;
FSOUND_SAMPLE *sample_centerlane;
FSOUND_SAMPLE *sample_rightlane;
FSOUND_SAMPLE *sample_donemoving;
FSOUND_SAMPLE *sample_moving;
FSOUND_SAMPLE *sample_pause;
FSOUND_SAMPLE *sample_unpause;
FSOUND_SAMPLE *sample_engine;
FSOUND_SAMPLE *sample_firstinitial;
FSOUND_SAMPLE *sample_secondinitial;
FSOUND_SAMPLE *sample_thirdinitial;
FSOUND_SAMPLE *sample_gothighscore;
FSOUND_SAMPLE *sample_returntomenu;
FSOUND_SAMPLE *sample_startnext;
//Menu samples
FSOUND_SAMPLE *sample_training;
FSOUND_SAMPLE *sample_startgame;
FSOUND_SAMPLE *sample_highscores;
FSOUND_SAMPLE *sample_quit;
FSOUND_SAMPLE *sample_soniczoom;
FSOUND_SAMPLE *sample_menuinstructions;
FSOUND_SAMPLE *sample_selectlevelbeep;
FSOUND_SAMPLE *sample_chooselevel;
//Menu streams
FSOUND_STREAM *stream_training;
FSOUND_STREAM *stream_credits;
FSOUND_STREAM *stream_music;

//
// IInitializeMenuSounds
//  ... called by: CGameControl
//	... description: Initializes Fmod
//					 loads all samples used in the menu
//
void IInitializeMenuSounds()
{
	//Initialize FMOD
	FSOUND_Init(44100, 32, 0);

	//Load samples into memory
	sample_training			= FSOUND_Sample_Load(FSOUND_FREE, "sounds/training.wav", FSOUND_2D, 0, 0);
	sample_startgame		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/startgame.wav", FSOUND_2D, 0, 0);
	sample_highscores		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/highscores.wav", FSOUND_2D, 0, 0);
	sample_quit				= FSOUND_Sample_Load(FSOUND_FREE, "sounds/quit.wav", FSOUND_2D, 0, 0);
	sample_soniczoom		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/soniczoom.wav", FSOUND_2D, 0, 0);
	sample_menuinstructions	= FSOUND_Sample_Load(FSOUND_FREE, "sounds/menuinstructions.wav", FSOUND_2D, 0, 0);
	sample_selectlevelbeep	= FSOUND_Sample_Load(FSOUND_FREE, "sounds/beep4.wav", FSOUND_2D, 0, 0);
	sample_chooselevel		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/chooselevel.wav", FSOUND_2D, 0, 0);
	//Open streams
	stream_music			= FSOUND_Stream_Open("sounds/music.wav", FSOUND_2D | FSOUND_LOOP_NORMAL, 0, 0);
	stream_training			= FSOUND_Stream_Open("sounds/training.mp3", FSOUND_2D, 0, 0);
}

//
// IInitializeRaceSounds
//  ... called by: CGameControl
//	... description: Assigns each to a separate sound channel.
//					 Player is initialized to face forward.
//
void IInitializeRaceSounds( CBaseEntity** entArray, int entCount )
{
	sample_coin_1			= FSOUND_Sample_Load(FSOUND_FREE, "sounds/coin1.wav", FSOUND_HW3D | FSOUND_LOOP_NORMAL, 0, 0);
	sample_coin_2			= FSOUND_Sample_Load(FSOUND_FREE, "sounds/coin2.wav", FSOUND_HW3D | FSOUND_LOOP_NORMAL, 0, 0);
	sample_coin_3			= FSOUND_Sample_Load(FSOUND_FREE, "sounds/coin3.wav", FSOUND_HW3D | FSOUND_LOOP_NORMAL, 0, 0);
	sample_obstacle_1		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/obstacle1.wav", FSOUND_HW3D | FSOUND_LOOP_NORMAL, 0, 0);
	sample_obstacle_2		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/obstacle2.wav", FSOUND_HW3D | FSOUND_LOOP_NORMAL, 0, 0);
	sample_obstacle_3		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/obstacle3.wav", FSOUND_HW3D | FSOUND_LOOP_NORMAL, 0, 0);

	sample_hitobstacle		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/hitobstacle.wav", FSOUND_2D, 0, 0);
	sample_hitcoin			= FSOUND_Sample_Load(FSOUND_FREE, "sounds/hitcoin.wav", FSOUND_2D, 0, 0);
	sample_death			= FSOUND_Sample_Load(FSOUND_FREE, "sounds/death.wav", FSOUND_2D, 0, 0);
	sample_levelend			= FSOUND_Sample_Load(FSOUND_FREE, "sounds/levelend.wav", FSOUND_2D, 0, 0);
	sample_levelstart		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/levelstart.wav", FSOUND_2D, 0, 0);
	sample_pause			= FSOUND_Sample_Load(FSOUND_FREE, "sounds/pause.wav", FSOUND_2D, 0, 0);
	sample_unpause			= FSOUND_Sample_Load(FSOUND_FREE, "sounds/unpause.wav", FSOUND_2D, 0, 0);

	sample_donemoving		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/beep1.wav", FSOUND_2D, 0, 0);
	sample_moving			= FSOUND_Sample_Load(FSOUND_FREE, "sounds/beep4.wav", FSOUND_HW3D, 0, 0);

	sample_leftlane			= FSOUND_Sample_Load(FSOUND_FREE, "sounds/leftlane.wav", FSOUND_2D, 0, 0);
	sample_centerlane		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/centerlane.wav", FSOUND_2D, 0, 0);
	sample_rightlane		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/rightlane.wav", FSOUND_2D, 0, 0);

	sample_warning			= FSOUND_Sample_Load(FSOUND_FREE, "sounds/beep2.wav", FSOUND_2D, 0, 0);

	sample_engine			= FSOUND_Sample_Load(FSOUND_FREE, "sounds/engine.wav", FSOUND_2D | FSOUND_LOOP_NORMAL , 0, 0);

	sample_firstinitial		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/firstinitial.wav", FSOUND_2D, 0, 0);
	sample_secondinitial	= FSOUND_Sample_Load(FSOUND_FREE, "sounds/secondinitial.wav", FSOUND_2D, 0, 0);
	sample_thirdinitial		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/thirdinitial.wav", FSOUND_2D, 0, 0);
	sample_gothighscore		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/gothighscore.wav", FSOUND_2D, 0, 0);
	sample_returntomenu		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/returntomenu.wav", FSOUND_2D, 0, 0);
	sample_startnext		= FSOUND_Sample_Load(FSOUND_FREE, "sounds/startnext.wav", FSOUND_2D, 0, 0);

	stream_credits			= FSOUND_Stream_Open("sounds/credits.mp3", FSOUND_2D, 0, 0);

	//Set min max distances for the coins and obstacles so that they'll be louder
	//They will be at max volume and remain at max once they are 50 units ahead
	FSOUND_Sample_SetMinMaxDistance( sample_coin_1, 50, 1000000000 );
	FSOUND_Sample_SetMinMaxDistance( sample_coin_2, 50, 1000000000 );
	FSOUND_Sample_SetMinMaxDistance( sample_coin_3, 50, 1000000000 );
	FSOUND_Sample_SetMinMaxDistance( sample_obstacle_1, 50, 1000000000 );
	FSOUND_Sample_SetMinMaxDistance( sample_obstacle_2, 50, 1000000000 );
	FSOUND_Sample_SetMinMaxDistance( sample_obstacle_3, 50, 1000000000 );
}

//
// IPlayMenuSound
//  ... called by: CGameControl
//	... description: plays menu sounds
//					 only uses one channel for sounds and one channel for streams
//					 any new sounds stop any currently playing sounds, only one stream at a time
//					
void IPlayMenuSound( int type )
{
	FSOUND_SAMPLE *sample = NULL;
	FSOUND_STREAM *stream = NULL;

	if(type == SOUND_MENU_TRAINING )
		sample = sample_training;
	else if(type == SOUND_MENU_STARTGAME )
		sample = sample_startgame;
	else if(type == SOUND_MENU_HIGHSCORES )
		sample = sample_highscores;
	else if(type == SOUND_MENU_QUIT )
		sample = sample_quit;
	else if(type == SOUND_MENU_SONICZOOM )
		sample = sample_soniczoom;
	else if(type == SOUND_MENU_INSTRUCTIONS )
		sample = sample_menuinstructions;
	else if(type == SOUND_MENU_SELECTLEVELBEEP )
		sample = sample_selectlevelbeep;
	else if(type == SOUND_MENU_CHOOSELEVEL )
		sample = sample_chooselevel;
	else if( type == SOUND_STREAM_TRAINING )
		stream = stream_training;
	else if( type == SOUND_STREAM_MUSIC )
		stream = stream_music;
	if ( !sample && !stream)
		return;

	//all menu sounds are only played in channel 0 so that sounds will cancel each other out
	//same with streams all in channel 1    
	// If the sample is not null, then play the sample, otherwise, it's a stream
	if(sample != NULL)
	{
		//Free channel 0
		FSOUND_StopSound( 0 );
		FSOUND_PlaySound(0, sample);
	}
	else
	{
		//Free channel 1
		FSOUND_StopSound( 1 );
		FSOUND_Stream_Play(1, stream);
	}
}

//
// IPlayStationarySound
//  ... called by: CGameControl
//	... description: plays a single sound once with no 3D support
//
void IPlayStationarySound( float pos[3], int volume, int type )
{
	FSOUND_SAMPLE *sample = NULL;
	FSOUND_STREAM *stream = NULL;
	int channel;

	if(type == SOUND_EVENT_HITOBSTACLE )
		sample = sample_hitobstacle;
	else if(type == SOUND_EVENT_HITCOIN )
		sample = sample_hitcoin;
	else if(type == SOUND_EVENT_DEATH )
		sample = sample_death;
	else if(type == SOUND_EVENT_LEVELEND )
		sample = sample_levelend;
	else if(type == SOUND_EVENT_LEVELSTART )
		sample = sample_levelstart;
	else if( type == SOUND_EVENT_DONEMOVING )
		sample = sample_donemoving;
	else if( type == SOUND_EVENT_WARNING )
		sample = sample_warning;
	else if( type == SOUND_EVENT_LEFTLANE )
		sample = sample_leftlane;
	else if( type == SOUND_EVENT_CENTERLANE )
		sample = sample_centerlane;
	else if( type == SOUND_EVENT_RIGHTLANE )
		sample = sample_rightlane;
	else if( type == SOUND_EVENT_PAUSE )
		sample = sample_pause;
	else if( type == SOUND_EVENT_FIRSTINITIAL )
		sample = sample_firstinitial;
	else if( type == SOUND_EVENT_SECONDINITIAL )
		sample = sample_secondinitial;
	else if( type == SOUND_EVENT_THIRDINITIAL )
		sample = sample_thirdinitial;
	else if( type == SOUND_EVENT_GOTHIGHSCORE )
		sample = sample_gothighscore;
	else if( type == SOUND_EVENT_RETURNTOMENU )
		sample = sample_returntomenu;
	else if( type == SOUND_EVENT_STARTNEXT )
		sample = sample_startnext;
	else if( type == SOUND_STREAM_CREDITS )
		stream = stream_credits;
	if ( !sample && !stream)
		return;

	channel = FSOUND_PlaySound(FSOUND_FREE, sample);

	if(sample != NULL)
	{
		channel = FSOUND_PlaySound(FSOUND_FREE, sample);
	}
	else
	{
		FSOUND_Stream_Play(FSOUND_FREE, stream);
	}
}

//
// IPlay3DSound
//  ... called by: CGameControl
//	... description: plays a single sound once with 3D support
//
void IPlay3DSound( float pos[3], float vel[3], int volume, int type )
{
	FSOUND_SAMPLE *sample = NULL;
	int channel;

	if(type == SOUND_3D_MOVING )
		sample = sample_moving;

	if ( !sample )
		return;

	float secPos[3] = { pos[0], pos[1], -pos[2] };
	float secVel[3] = { vel[0], vel[1], -vel[2] };

	channel = FSOUND_PlaySound(FSOUND_FREE, sample);
	//Taking out setting volume for now
	//FSOUND_SetVolume(channel, volume);
	FSOUND_3D_SetAttributes( channel, secPos, secVel );
}

//
// IResetRaceSounds
//  ... called by : CGameControl
//	... description: Resets entities after each new level starts
//
void IResetRaceSounds( CBaseEntity** entArray, int entCount )
{
	float pos[3] = {0, 0, 0};
	//float vel[3] = {0, 0, 0}; //won't be needed since all coins/obstacles stationary 

	//Vectors for player
	float listener_pos[3] = {TRACK_2_X, 0, 0}; //player initial position
	float listener_vel[3] = {0, 0, 0}; //player initially stationary
	
	//Setting player position
	FSOUND_3D_Listener_SetAttributes(listener_pos, listener_vel, 0, 0, 1.0f, 0, 1.0f, 0);

	// initialize booleans
	for ( int i = 0; i < entCount; i++ )
	{
		entArray[i]->pvars->has_sound = false;
	}

	//Engine sound start, engine is always channel 0
	FSOUND_PlaySound( 0, sample_engine);
}

//
// IUpdateSounds
//  ... called by: CGameControl
//	... description: Updates the player position for sound purposes
//
void IUpdateSounds( CBaseEntity** entArray, int entCount, IEntVars * player )
{
	float pos[3] = {0, 0, 0};

	//Go through all entities
	//Coins and obstacles will be given sounds and positions
	for ( int i = 0; i < entCount; i++ )
	{
		IEntVars *pVar = entArray[i]->pvars;

		//If the object is within "hearing range" and doesn't have a sound yet, and it hasn't been crashed into yet, give it a sound
		if( ( pVar->origin.z - player->origin.z < MAX_PLAYER_SPEED * 3 ) && ( pVar->origin.z - player->origin.z > 0 ) && !(pVar->has_sound) && (pVar->ent_flags & ENT_AUDIBLE) )
		{

			if( pVar->obj_type == OBJ_COIN )	//If it's a coin
			{
				if ( pVar->origin.x == TRACK_1_X )
				{
					pVar->channel_id = FSOUND_PlaySound(FSOUND_FREE, sample_coin_1);
				} 
				else if ( pVar->origin.x == TRACK_2_X )
				{
					pVar->channel_id = FSOUND_PlaySound(FSOUND_FREE, sample_coin_2);
				}
				else if ( pVar->origin.x == TRACK_3_X )
				{
					pVar->channel_id = FSOUND_PlaySound(FSOUND_FREE, sample_coin_3);
				}

				//Get and set the position of the coin
				pos[0] = pVar->origin.x;
				pos[1] = pVar->origin.y;
				pos[2] = -pVar->origin.z;
	
				FSOUND_3D_SetAttributes( pVar->channel_id, pos, NULL );
				pVar->has_sound = TRUE;
			}
			else if( pVar->obj_type == OBJ_OBSTACLE)	//If it's an obstacles
			{
				//Play the obstacle sound determined by the lane
				if ( pVar->origin.x == TRACK_1_X )
				{
					pVar->channel_id = FSOUND_PlaySound(FSOUND_FREE, sample_obstacle_1);
				} 
				else if ( pVar->origin.x == TRACK_2_X )
				{
					pVar->channel_id = FSOUND_PlaySound(FSOUND_FREE, sample_obstacle_2);
				}
				else if ( pVar->origin.x == TRACK_3_X )
				{
					pVar->channel_id = FSOUND_PlaySound(FSOUND_FREE, sample_obstacle_3);
				}

				//Get and set the position of the obstacle
				pos[0] = pVar->origin.x;
				pos[1] = pVar->origin.y;
				pos[2] = -pVar->origin.z;

				FSOUND_3D_SetAttributes( pVar->channel_id, pos, NULL );
				pVar->has_sound = TRUE;
			}
		}
		// Else if the coin is out of hearing range, or has been collided with, stop the sound and free the channel
		else if( (pVar->origin.z - player->origin.z <= 0) || (!(pVar->ent_flags & ENT_AUDIBLE) && (pVar->has_sound))  )
		{
			FSOUND_StopSound( pVar->channel_id );
			pVar->has_sound = FALSE;
		}
	}

	//The engine sound's frequency is based on player's speed, so that player can gauge his speed based on the engine's pitch
	FSOUND_SetFrequency( 0, (player->speed) * 100 );

	//update player position
	float listener_pos[3] = {0, 0, 0};
	listener_pos[0] = player->origin.x+0.1;
	listener_pos[1] = player->origin.y;
	listener_pos[2] = -player->origin.z;

	//update player velocity
	float listener_vel[3] = {0, 0, 0};
	listener_vel[0] = player->velocity.x;
	listener_vel[1] = player->velocity.y;
	listener_vel[2] = -player->velocity.z;

	//Set and Update
	FSOUND_3D_Listener_SetAttributes(listener_pos, listener_vel, 0, 0, 1.0f, 0, 1.0f, 0);

	FSOUND_Update();
}

//
// IPauseSound
//  ... called by: CGameControl
//	... description: pauses all sounds, used when game is paused
//
void IPauseSounds()
{
	FSOUND_SAMPLE *sample;
	int channel;

	//play the pause sound so that player knows game is paused
	sample = FSOUND_Sample_Load(FSOUND_FREE, "pause.wav", FSOUND_2D , 0, 0);
	channel = FSOUND_PlaySound(FSOUND_FREE, sample);

	//pause all sound channels
	FSOUND_SetPaused( FSOUND_ALL, TRUE );
}

//
// IUnpauseSound
//  ... called by: CGameControl
//	... description: resumes all sound after player resumes game after pause
//
void IUnpauseSounds()
{
	//unpause all sound channels
	FSOUND_SetPaused( FSOUND_ALL, FALSE );
}

//
// IResetSound
//  ... called by: CGameControl
//	... description: stops all sounds and frees all sound channels. 
//					 used when level ends or when player quits game or returns to menu
//
void IResetSounds()
{
	FSOUND_StopSound( FSOUND_ALL );
}

//
// IFreeRacingSamples
//  ... called by: soundfunc.cpp
//	... description: frees racing samples from memory
//
void IFreeRacingSamples()
{
	FSOUND_Sample_Free( sample_coin_1 );
	FSOUND_Sample_Free( sample_coin_2 );
	FSOUND_Sample_Free( sample_coin_3 );
	FSOUND_Sample_Free( sample_obstacle_1 );
	FSOUND_Sample_Free( sample_obstacle_2 );
	FSOUND_Sample_Free( sample_obstacle_3 );
	FSOUND_Sample_Free( sample_hitobstacle );
	FSOUND_Sample_Free( sample_hitcoin );
	FSOUND_Sample_Free( sample_death );
	FSOUND_Sample_Free( sample_levelend );
	FSOUND_Sample_Free( sample_levelstart );
	FSOUND_Sample_Free( sample_warning );
	FSOUND_Sample_Free( sample_leftlane );
	FSOUND_Sample_Free( sample_centerlane );
	FSOUND_Sample_Free( sample_rightlane );
	FSOUND_Sample_Free( sample_pause );
	FSOUND_Sample_Free( sample_unpause );
}

//
// IFreeMenuSamples
//  ... called by: soundfunc.cpp
//	... description: frees menu samples from memory and closes menu streams
//
void IFreeMenuSamples()
{
	FSOUND_Sample_Free( sample_training );
	FSOUND_Sample_Free( sample_startgame );
	FSOUND_Sample_Free( sample_highscores );
	FSOUND_Sample_Free( sample_quit );
	FSOUND_Sample_Free( sample_soniczoom );
	FSOUND_Stream_Close( stream_training );
	FSOUND_Stream_Close( stream_credits );
	FSOUND_Stream_Close( stream_music );
}

//
// ICloseFmod
//  ... called by: CGameControl
//	... description: closes all streams, frees all samples, stops all sounds and closes fmod
//					 used when player quits game
//
void ICloseFmod()
{
	FSOUND_StopSound( FSOUND_ALL );
	IFreeRacingSamples();
	IFreeMenuSamples();
	FSOUND_Close();
}