//-----------------------------------------------------------------------------
// File: D3DBase.h
//
// Desc: Header file D3DBase sample app
//-----------------------------------------------------------------------------
#pragma once




//-----------------------------------------------------------------------------
// Defines, and constants
//-----------------------------------------------------------------------------
// TODO: change "DirectX AppWizard Apps" to your name or the company name
//#define DXAPP_KEY        TEXT("Software\\DirectX AppWizard Apps\\D3DBase")
#define DXAPP_KEY        TEXT("Software\\SoundsLikeFun\\SonicZoom")

const float MAX_DRAW_DISTANCE = 1000.0f;

// Custom D3D vertex format used by the vertex buffer
// Dan - we're using lights but not lighting, still need normal/might as well have it
struct CUSTOMVERTEX
{
    D3DXVECTOR3 position;       // vertex position
    D3DXVECTOR3 normal;         // vertex normal
	float fU;
	float fV;
};

// Screen vertex
// Dan - for the menu
struct SCREENVERTEX
{
    D3DXVECTOR4 p;
    DWORD       color;
    FLOAT       tu, tv;
};

#define D3DFVF_CUSTOMVERTEX (D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_TEX1) // Dan - normal vertex, xyz, with a normal for lighting, and texture coordinates
#define D3DFVF_SCREENVERTEX (D3DFVF_XYZRHW|D3DFVF_DIFFUSE|D3DFVF_TEX1) // Dan - pre-computed vertices


// DirectInput action mapper reports events only when buttons/axis change
// so we need to remember the present state of relevant axis/buttons for 
// each DirectInput device.  The CInputDeviceManager will store a 
// pointer for each device that points to this struct
struct InputDeviceState
{
    // Dan - changed to what we use in this game
    BOOL  bButtonLeft;
    BOOL  bButtonRight;
    BOOL  bButtonUp;
    BOOL  bButtonDown;
	BOOL  bButtonSpace;
	BOOL  bButtonEscape;
};


// Struct to store the current input state
struct UserInput
{
	// Dan - everything except the display one is useless to us now. No need to break something by taking them out
    FLOAT fAxisRotateUD;
    FLOAT fAxisRotateLR;
    BOOL bDoConfigureInput;
    BOOL bDoConfigureDisplay;
};



//-----------------------------------------------------------------------------
// Name: class CMyD3DApplication
// Desc: Application class. The base class (CD3DApplication) provides the 
//       generic functionality needed in all Direct3D samples. CMyD3DApplication 
//       adds functionality specific to this sample program.
//-----------------------------------------------------------------------------
class CMyD3DApplication : public CD3DApplication
{
    BOOL                    m_bLoadingApp;          // TRUE, if the app is loading
    LPDIRECT3DVERTEXBUFFER9 m_pVB;                  // Vextex buffer 
    CD3DFont*               m_pFont;                // Font for drawing text
    CInputDeviceManager*    m_pInputDeviceManager;  // DirectInput device manager
    DIACTIONFORMAT          m_diafGame;             // Action format for game play
    LPDIRECT3DSURFACE9      m_pDIConfigSurface;     // Surface for config'ing DInput devices
    UserInput               m_UserInput;            // Struct for storing user input 

    FLOAT                   m_fWorldRotX;           // World rotation state X-axis
    FLOAT                   m_fWorldRotY;           // World rotation state Y-axis

	// SONICZOOM STUFF
	DWORD m_dwPolyCount;			// DEBUG: how many polys are we drawing?
	DWORD m_dwDeferredPolyCount;	// DEBUG: how many polys did we avoid drawing?
	float m_fNextModeSwitchTime;	// DEBUG: was used to switch between track warps

	int m_iTrackVertexCount;	// the number of vertices (3 * num of triangles) in the track VB

	float m_fSkyX;				// X coordinate of the skybox (also the player)
	float m_fSkyY;				// Y coordinate of the skybox (also the player)
	float m_fSkyZ;				// Z coordinate of the skybox (also the player)

	float m_fBoxRotateX;		// how much to rotate the box in the menu
	float m_fBoxRotateY;
	float m_fBoxRotateZ;

	D3DXVECTOR3 m_vPoint;		// in what direction is the player pointing in?
	D3DXVECTOR3 m_vPos;			// where is the player?

	CD3DMesh* m_pSkyBox;		// the skybox mesh, loaded from a file

	CD3DFont* m_pBigFont;		// the font used for the menu/HUD

	LPDIRECT3DVERTEXBUFFER9	m_pObstacleVB;		// vertex buffer for the obstacle (cube)
	LPDIRECT3DVERTEXBUFFER9 m_pCoinVB;			// vertex buffer for the coin (cube)
	LPDIRECT3DVERTEXBUFFER9 m_pTrackQuadVB;		// vertex buffer for the track
	LPDIRECT3DVERTEXBUFFER9 m_pMenuVB;			// vertex buffer for the menu (quad the size of the screen)

	LPDIRECT3DTEXTURE9		m_pObstacleTex;		// obstacle texture
	LPDIRECT3DTEXTURE9		m_pCoinTex;			// coin texture
	LPDIRECT3DTEXTURE9		m_pTrackMainTex;	// main track texture (ground)
	LPDIRECT3DTEXTURE9		m_pLine1Tex;		// texture of the first line
	LPDIRECT3DTEXTURE9		m_pLine2Tex;		// texture of the second line
	LPDIRECT3DTEXTURE9		m_pSideLowTex;		// texture of the lower wall
	LPDIRECT3DTEXTURE9		m_pSideHighTex;		// texture of the higher wall
	LPDIRECT3DTEXTURE9		m_pCeilingTex;		// texture of the ceiling
	LPDIRECT3DTEXTURE9		m_pMenuTex;			// menu texture

	CUberControl*			m_pUberCon;			// UberControl of the game

	bool m_bNeedTrackRefresh;	// do we need to reload the track geometry?
	D3DCOLOR m_dwAmbColor;		// what is the lighting level/color we're using?
	// END SONICZOOM STUFF
  

protected:  
	virtual HRESULT OneTimeSceneInit();
    virtual HRESULT InitDeviceObjects(); // Dan - we use this heavily even though it's a template-generated function
    virtual HRESULT RestoreDeviceObjects(); // Dan - we use this heavily even though it's a template-generated function
    virtual HRESULT InvalidateDeviceObjects(); // Dan - we use this heavily even though it's a template-generated function
    virtual HRESULT DeleteDeviceObjects(); // Dan - we use this heavily even though it's a template-generated function
    virtual HRESULT Render(); // Dan - we use this heavily even though it's a template-generated function
    virtual HRESULT FrameMove(); // Dan - we use this heavily even though it's a template-generated function
    virtual HRESULT FinalCleanup();
    virtual HRESULT ConfirmDevice( D3DCAPS9*, DWORD, D3DFORMAT );
    VOID    Pause( bool bPause );

    HRESULT RenderText(); // Dan - we use this heavily even though it's a template-generated function

    HRESULT InitInput( HWND hWnd );
    void    UpdateInput( UserInput* pUserInput );
    void    CleanupDirectInput();
    VOID    ReadSettings(); // Dan - neutered
    VOID    WriteSettings(); // Dan - neutered, we don't want settings to be written or read

	// SONIC ZOOM
	HRESULT RenderUpdateView(); // update the view matrix from the player's perspective
	HRESULT RenderGameObjects(); // render the coins and obstacles
	HRESULT RenderSky(); // render the sky box
	HRESULT InitGameVertexBuffers(); // initialize our vertex buffers
	HRESULT LoadStaticTrackGeometry(); // load the GTrackArray into a vertex buffer
	HRESULT RenderTrackArray(); // render our track
	HRESULT RenderMenu(); // render the menu
	// END SONIC ZOOM


public:
    LRESULT MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
    CMyD3DApplication(); // Dan - we use this heavily even though it's a template-generated function
    virtual ~CMyD3DApplication();

    HRESULT InputAddDeviceCB( CInputDeviceManager::DeviceInfo* pDeviceInfo, const DIDEVICEINSTANCE* pdidi );
    static HRESULT CALLBACK StaticInputAddDeviceCB( CInputDeviceManager::DeviceInfo* pDeviceInfo, const DIDEVICEINSTANCE* pdidi, LPVOID pParam );   
    BOOL    ConfigureInputDevicesCB( IUnknown* pUnknown );
    static BOOL CALLBACK StaticConfigureInputDevicesCB( IUnknown* pUnknown, VOID* pUserData );
};

