
#include "CMenuControl.h"
#include "stateflagdefs.h"
#include "IInput.h"
#include "ITime.h"
#include "CBaseEntity.h"
#include "globalconst.h"
#include "soundfunc.h"
#include "ITextToSpeech.h"
#include "IHighScores.h"
#include "filedefs.h"
#include <string.h>

// -------------------------------------
// CMenuControl - functions definitions
//
// ... see header for more information
//
//	UPDATES:
//		4/2/04 - Daniel: created
//		4/21/04 - Daniel: this header didn't get updated, sorry. basically finished on this date
//
// -------------------------------------

//
// Constructors
//
//
//

CMenuControl::CMenuControl()
{
	levelStart = 1; // start on level 1
	menuState = MENU_INIT; // start in init
	menuMainSelect = MENU_SELECT_TRAINING; // main menu

	bTrainingPlaying = false; // not initially

	// initialize all input to false
	bUp = bDown = bLeft = bRight = bBack = bConfirm = false; 

	// set timers to zero
	timerNextUp = 0;
	timerNextDown = 0;
	timerNextRight = 0;
	timerNextLeft = 0;
	timerNextBack = 0;
	timerNextConfirm = 0;
	
	timerTransition = 0;
	timerReminder = 0;
	timerNum = 0;
	timerTrainingDone = 0;

	bSaidInstructions = false;

	// don't have a highscores database yet
	m_pHighScores = NULL;
}


//
// Think
//	... called by: CUberControl::Think()
//	... description: frame-by-frame logic
//
void CMenuControl::Think()
{
	UpdateInputCommands();
	
	if ( timerTransition < glTime->getTime() )
	{
		switch( menuState )
		{
		case MENU_INTRO:
			//	IntroThink();
			// NOTE: as of 4/21/04, this never got implemented - Daniel
			// it was going to be a flashscreen before the game came up with a logo or something.
			// it basically was rendered unnecessary.
			menuState = MENU_INIT;
			break;
		case MENU_INIT:
			IInitializeMenuSounds();
			IPlayMenuSound( SOUND_MENU_SONICZOOM ); // "SONIC ZOOOOM!"
			IPlayMenuSound( SOUND_STREAM_MUSIC ); // the main menu theme
			//IPlayMenuSound( SOUND_MENU_INSTRUCTIONS);
			menuState = MENU_SELECTING; // go to the main menu
			timerTransition = glTime->getTime()+MENU_INTERVAL_SONICZOOM; // a short interval before the menu_selecting code kicks in, to allow the sonic zoom sound to play
			break;
		case MENU_SELECTING:
			SelectThink(); // Main menu think
			break;
		case MENU_GAMESELECT:
			GameSelectThink(); // Picking a level
			break;
		case MENU_HIGHSCORES:
			HighScoresThink(); // Viewing the high scores
			break;
		case MENU_DOTRAINING:
			TrainingThink(); // Listening to the training mode
			break;
		case MENU_DOGAMEPLAY: // this is for CUberControl to pick up on, do nothing
		case MENU_DOQUIT: // esta, tambien
			break;
		}
	}

}

//
// UpdateInputCommands
//	... called by: Think()
//	... description: Do input here, because we need to make sure that input is allowed only at intervals, 
//						otherwise we'd move "up" sixty-some times or more in a frame. The boolean values
//						reset every frame, and under this system, if one is true, we can be sure to act on it.
//
void CMenuControl :: UpdateInputCommands()
{
	bUp = bDown = bLeft = bRight = bBack = bConfirm = false; // it's a new frame, everything is now reset
	
	// if a key is pressed, accounts for beind held down
	if ( glInput->isKeyPressed( GKEY_UP ) && timerNextUp < glTime->getTime() )
	{
		bUp = true;
		timerNextUp = glTime->getTime() + MENU_INTERVAL_MOVE;
	}
	if ( glInput->isKeyPressed( GKEY_DOWN ) && timerNextDown < glTime->getTime() )
	{
		bDown = true;
		timerNextDown = glTime->getTime() + MENU_INTERVAL_MOVE;
	}
	if ( glInput->isKeyPressed( GKEY_RIGHT ) && timerNextRight < glTime->getTime() )
	{
		bRight = true;
		timerNextRight = glTime->getTime() + MENU_INTERVAL_MOVE;
	}
	if ( glInput->isKeyPressed( GKEY_LEFT ) && timerNextLeft < glTime->getTime() )
	{
		bLeft = true;
		timerNextLeft = glTime->getTime() + MENU_INTERVAL_MOVE;
	}
 	if ( glInput->isKeyPressed( GKEY_ESC ) && timerNextBack < glTime->getTime() )
	{
		bBack = true;
		timerNextBack = glTime->getTime() + MENU_INTERVAL_CHOOSE;
	}
	if ( glInput->isKeyPressed( GKEY_SPACE ) && timerNextUp < glTime->getTime() )
	{
		bConfirm = true;
		timerNextBack = glTime->getTime() + MENU_INTERVAL_CHOOSE;
	}

	// If the key is NOT pressed, we can reset the timer and let the player use the key at will
	if ( !glInput->isKeyPressed( GKEY_UP ) )
		timerNextUp = 0;
	if ( !glInput->isKeyPressed( GKEY_DOWN ) )
		timerNextDown = 0;
	if ( !glInput->isKeyPressed( GKEY_RIGHT ) )
		timerNextRight = 0;
	if ( !glInput->isKeyPressed( GKEY_LEFT ) )
		timerNextLeft = 0;
	if ( !glInput->isKeyPressed( GKEY_ESC ) )
		timerNextBack = 0;
	if ( !glInput->isKeyPressed( GKEY_SPACE ) )
		timerNextBack = 0;
}


//
// SelectThink
//	... called by: Think()
//	... description: we're on the main menu
//
void CMenuControl::SelectThink()
{
	
	if ( !bSaidInstructions )
	{
		IPlayMenuSound(SOUND_MENU_INSTRUCTIONS);
		bSaidInstructions = true;
		timerReminder = glTime->getTime() + 7.1f;
	}
	
	if ( bBack )
	{
		// the user is quitting out
		menuState = MENU_DOQUIT;
		IFreeMenuSamples();
		return;
	}

	if ( bConfirm )
	{
		// the user has chosen an option
		if ( menuMainSelect == MENU_SELECT_TRAINING )
		{
			menuState = MENU_DOTRAINING; // this will be caught by UberControl, which will handle the transition
			timerTrainingDone = glTime->getTime() + 72.0f; // HACK: would be nice if we could tell what the length of a stream is
		}
		else if ( menuMainSelect == MENU_SELECT_STARTGAME )
		{
			menuState = MENU_GAMESELECT; // go to the game-select submenu
			IPlayMenuSound(SOUND_MENU_CHOOSELEVEL);
			timerTransition = glTime->getTime() + MENU_INTERVAL_TRANSITION; 
		}
		else if ( menuMainSelect == MENU_SELECT_HIGHSCORES )
		{
			menuState = MENU_HIGHSCORES; // go to highscores 
			timerTransition = glTime->getTime() + MENU_INTERVAL_TRANSITION;
			timerScores = timerTransition + 0.25f;
		}
		else if ( menuMainSelect == MENU_SELECT_QUIT )
		{
			menuState = MENU_DOQUIT; // this will be caught by UberControl, which will kill the app
			IFreeMenuSamples();
		}
	}

	if ( bDown )
	{
		timerReminder = 0; // we want to "remind" them immediately

		// move down (and wrap around)
		if ( menuMainSelect == MENU_SELECT_QUIT )
		{
			menuMainSelect = MENU_SELECT_TRAINING;
		}
		else if ( menuMainSelect == MENU_SELECT_TRAINING )
		{
			menuMainSelect = MENU_SELECT_STARTGAME;
		}
		else if ( menuMainSelect == MENU_SELECT_STARTGAME )
		{
			menuMainSelect = MENU_SELECT_HIGHSCORES;
		}
		else if ( menuMainSelect == MENU_SELECT_HIGHSCORES )
		{
			menuMainSelect = MENU_SELECT_QUIT;
		}
	}

	if ( bUp)
	{
		timerReminder = 0; // we want to "remind" them immediately

		// move down (and wrap around)
		if ( menuMainSelect == MENU_SELECT_QUIT )
		{
			menuMainSelect = MENU_SELECT_HIGHSCORES;
		}
		else if ( menuMainSelect == MENU_SELECT_HIGHSCORES )
		{
			menuMainSelect = MENU_SELECT_STARTGAME;
		}
		else if ( menuMainSelect == MENU_SELECT_STARTGAME )
		{
			menuMainSelect = MENU_SELECT_TRAINING; 
		}
		else if ( menuMainSelect == MENU_SELECT_TRAINING )
		{
			menuMainSelect = MENU_SELECT_QUIT;
		}
	}

	// CHECK REMINDER TIMER HERE
	if ( timerReminder < glTime->getTime() )
	{
		IPlayMenuSound( GetSoundFromMenuSelect() );
		timerReminder = glTime->getTime() + MENU_INTERVAL_REMINDER;
	}
}

//
// GameSelectThink
//	... called by: Think()
//	... description: we're selecting which level we want to start at
//
void CMenuControl::GameSelectThink()
{
	// Are we exiting?
	if ( bBack )
	{
		menuState = MENU_SELECTING; // go back to the main menu
		timerTransition = glTime->getTime() + MENU_INTERVAL_TRANSITION;
	}
	// Are we choosing?
	if ( bConfirm )
	{
		menuState = MENU_DOGAMEPLAY; // uberControl will catch this, and use our current level selection
		IFreeMenuSamples();
	}
	// Are we picking a higher level?
	if ( bLeft || bDown )
	{
		timerNum = glTime->getTime() + MENU_INTERVAL_NUM;
		levelStart++;
		if ( levelStart > LEVEL_START_HIGH ) 
			levelStart = LEVEL_START_HIGH; // we can only start as so high a level
		else
			IPlayMenuSound(SOUND_MENU_SELECTLEVELBEEP);
	}
	// Are we picking a lower level?
	if ( bRight || bUp )
	{
		timerNum = glTime->getTime() + MENU_INTERVAL_NUM;
		levelStart--;
		if ( levelStart < 1 )
			levelStart = 1; // must start at the first level
		else
			IPlayMenuSound(SOUND_MENU_SELECTLEVELBEEP);
	}

	// REMIND TIMER CHECK HERE, RESET ABOVE
	if ( timerNum < glTime->getTime() )
	{
		
		TCHAR szMsg[MAX_PATH] = TEXT("");
		sprintf( szMsg, TEXT("%i" ), levelStart );
		string s;
		s.append(szMsg);
		ITextToSpeech talker;
		talker.speakText(s);

		timerNum = glTime->getTime() + MENU_INTERVAL_REMINDER;
		
	}
}

//
// HighScoresThink
//	... called by: Think()
//	... description: we're listening to the high scores
//
void CMenuControl::HighScoresThink()
{
	if ( !m_pHighScores )
	{
		m_pHighScores = new IHighScores( FILE_HIGHSCORES ); // create a new highscores object, link it to the member data
	}
	if ( timerScores < glTime->getTime() )
	{
		m_pHighScores->speakList(); // have text to speech speak the list--freezes program
		menuState = MENU_SELECTING; // go back to the main menu
		delete m_pHighScores; // kill our high scores object
		m_pHighScores = NULL; // reset the pointer
	}
}

//
// TrainingThink
//	... called by: Think()
//	... description: we're listening to the training
//
void CMenuControl::TrainingThink()
{
	if ( !bTrainingPlaying )
	{
		IPlayMenuSound( SOUND_STREAM_TRAINING ); // play the stream
		bTrainingPlaying = true; // so we only start the stream once
	}
	else if ( timerTrainingDone < glTime->getTime() || bBack )
	{
		menuState = MENU_SELECTING; // back to the main menu
		timerTransition = glTime->getTime() + MENU_INTERVAL_TRANSITION;
		bTrainingPlaying = false; // reset for next time
		IPlayMenuSound( SOUND_STREAM_MUSIC ); // restart the menu theme stream
	}
}

//
// GetSoundFromMenuSelect
//	... called by: any, utility
//	... description: get the sound index from the menu
//
int CMenuControl :: GetSoundFromMenuSelect()
{
	switch( menuMainSelect )
	{
	case MENU_SELECT_TRAINING:
		return SOUND_MENU_TRAINING;
	case MENU_SELECT_STARTGAME:
		return SOUND_MENU_STARTGAME;
	case MENU_SELECT_HIGHSCORES:
		return SOUND_MENU_HIGHSCORES;
	case MENU_SELECT_QUIT:
		return SOUND_MENU_QUIT;
	}
	return -1;
}
