
#ifndef __VEC3D_H_
#define __VEC3D_H_

#include <complex>
//-----------------------------------------------------------
// Class Vec3d
//
//	This is the vector class the game will use. It uses three float values, and includes
// all of the essential operators/methods/constructors.
//
//
//	UPDATES:
//
//	2/4/04 - Daniel: written
//
// This code was made by consulting a vector class used in the Half-Life engine
//-----------------------------------------------------------

class Vec3d
{
public:
	// CONSTRUCTORS
	inline Vec3d() { x = 0; y = 0; z = 0;} // default constructor
	inline Vec3d( float X, float Y, float Z ) { x=X; y=Y; z=Z; } // input constructor
	inline Vec3d( const Vec3d & vec ) { x = vec.x; y = vec.y; z = vec.z; } // copy constructor

	// OPERATORS
	inline Vec3d operator - () const { return Vec3d(-x,-y,-z); } // - operator (negate, not subtract)
	inline bool operator == ( const Vec3d & vec ) { return (vec.x==x && vec.y==y && vec.z==z); } // == operator
	inline bool operator != ( const Vec3d & vec ) { return !(*this==vec); } // != operator
	inline Vec3d operator + ( const Vec3d & vec ) { return Vec3d( x+vec.x, y+vec.y, z+vec.z ); } //+
	inline Vec3d operator - ( const Vec3d & vec ) { return Vec3d( x-vec.x, y-vec.y, z-vec.z ); } // subtract
	inline Vec3d operator * ( float m ) { return Vec3d( x*m, y*m, z*m ); } // multiply by scalar
	inline Vec3d operator / ( float m ) { return Vec3d( x/m, y/m, z/m ); } // divide by scalar

	// FUNCTIONS
	inline float Length() const { return (float)sqrt(x*x+y*y+z*z); } // return the length of the vector
	inline Vec3d Normalize() const  // return a unit vector in this direction
	{
		float m = 1/Length();
		return Vec3d( x*m,y*m,z*m );
	}

	// DATA
	float x, y, z;
};

#endif