

#include "CGameControl.h"
#include "ITime.h"
#include "CPlayer.h"
#include "ILevelLoader.h"
#include "stateflagdefs.h"
#include "globalconst.h"
#include "glfunc.h"
#include "Vec3.h"
#include "soundfunc.h"
#include "IInput.h"
#include "IHighScores.h"
#include "filedefs.h"
#include "ITextToSpeech.h"
#include <string.h>
#include <iostream>

// -------------------------------------
// CGameControl - functions definitions
//
// ... see header for more information
//
//	UPDATES:
//		- 2/7/04, Daniel: created
//		- 2/9/04, Daniel: CoinPickedUp, commented stuff
//		- 2/11/04, Daniel: racethink
//		- 2/12/04, Daniel: CalcScore, more to racethink, CheckCollision
//		- 2/17/04, Daniel: The level loading system now works, LoadLevel
//		- 2/18/04, Daniel: debug text graphics
//		- 2/19/04, Daniel: now handling level endings
//		- 2/22/04, Daniel: PlaySounds
//		- 2/24/04, Patrick: Draw changes
//		- 3/1/04,  Daniel: now tracks player's lives
//		- 3/21/04, Daniel: call to SwitchCheck
//		- 3/22/04, Daniel: SoundFunc stuff
//		- 3/31/04, Daniel: WarningThink
//		- 4/13/04, Daniel: PauseThink
//		- 4/17/04, Daniel: HighScoresThink, LevelSuccessThink
//
// -------------------------------------



//
//	Default constructor 
//			
//
CGameControl :: CGameControl() 
{
	entCount = -1; // array is not ready
	gameState = GAME_INIT; // set state to load level
	level = 1; // set level to the first
	score = 0;
	coins = 0;
	entArray = NULL; // initialize
	m_pHighScores = NULL;
	loader = new ILevelLoader();
	pauseTime = 0;
	pauseBeginTime = 0;
	levelDoneTime = 0;
	lives = NUM_LIVES_STANDARD;
	levelLength = 0;
	prePauseState = -1;

	name[0] = 'A';
	name[1] = 'A';
	name[2] = 'A';
	timerLetterRemind = 0.0f;
	nameState = 0;
	timerConfirm = 0.0f;
	timerBack = 0.0f;
	timerUp = 0.0f;
	timerDown = 0.0f;
	timerRight = 0.0f;
	timerLeft = 0.0f;

	bLevelEnd = false;
	timeTillSayScores = 0.0f;
	timeTillCanContinue = 0.0f;
}

//
//	Level start constructor
//		
//
CGameControl :: CGameControl( int startLevel )
{
	entCount = -1; // array is not ready 
	gameState = GAME_INIT; // set state to load level
	level = startLevel; // set level to given value
	score = 0;
	coins = 0;
	entArray = NULL; // initialie to null
	loader = new ILevelLoader();
	m_pHighScores = NULL;
	pauseTime = 0;
	pauseBeginTime = 0;
	levelDoneTime = 0;
	lives = NUM_LIVES_STANDARD;
	levelLength = 0;
	prePauseState = -1;

	name[0] = 'A';
	name[1] = 'A';
	name[2] = 'A';
	timerLetterRemind = 0.0f;
	nameState = 0;
	timerConfirm = 0.0f;
	timerBack = 0.0f;
	timerUp = 0.0f;
	timerDown = 0.0f;
	timerRight = 0.0f;
	timerLeft = 0.0f;

	bLevelEnd = false;
	timeTillSayScores = 0.0f;
	timeTillCanContinue = 0.0f;
}

void CGameControl :: CleanupEnts()
{
	
	for ( int i = 0; i < entCount; i++ )
		delete entArray[i];
	delete entArray;

	entArray = NULL;
}

void CGameControl :: CleanupLoader()
{

	loader->Cleanup();
	delete loader;

	loader = NULL;
}

//
// Think
//	... called by: CUberControl::Think()
//	... description: frame-by-frame logic
//
void CGameControl :: Think()
{

	switch ( gameState )
	{
		case GAME_INIT:
			LevelLoadData();
			IInitializeRaceSounds( entArray, entCount );
			//IPlayStationarySound(NULL, 30, SOUND_EVENT_LEVELEND );
			break;
		case GAME_LEVELINIT:
		//	SpeakLevelInfo();
			LevelGenerate();
			pauseTime = 0;
			pauseBeginTime = 0;
			//IInitializeRaceSounds( entArray, entCount );
			IResetRaceSounds(entArray,entCount);
			IPlayStationarySound(NULL,40,SOUND_EVENT_LEVELSTART);
			break;
		case GAME_STARTING:
			StartThink();
			break;
		case GAME_RACING:
			// "normal" execution loop
			RaceThink();
			//WarningThink();
			break;
		case GAME_PAUSE:
			PauseThink();
			break;
		case GAME_LEVEL_SUCCESS:
			LevelSuccessThink();
			/*IPlayStationarySound(NULL, 30, SOUND_EVENT_LEVELEND );
			gameState = GAME_LEVELINIT;
			CleanupEnts();
			level++;*/
			break;
		case GAME_HIGHSCORES:
			HighScoresThink();
			break;
		case GAME_DEATH:
			DeathThink();
			/*
			gameState = GAME_QUIT;
			IFreeRacingSamples();
			CleanupEnts();
			CleanupLoader();
			if ( m_pHighScores )
				delete m_pHighScores;
				*/
			break;
		case GAME_QUIT:
			break;

	}
}


//
// StartThink
//  ... called by: Think
//	... description: think code while in the starting racing mode
//
void CGameControl :: StartThink()
{
	if ( glInput->isKeyPressed( GKEY_ESC ) )
	{
	/*	IFreeRacingSamples();
		CleanupEnts();
		CleanupLoader();*/
		pauseBeginTime = glTime->getTime();
		gameState = GAME_PAUSE;
		IPlayStationarySound(NULL,50,SOUND_EVENT_PAUSE);
		IPauseSounds();
		prePauseState = GAME_STARTING;
		return;
	}
	if ( glTime->getTime() >=  timeTillStart )
	{
		gameState = GAME_RACING;
		levelStartTime = glTime->getTime();
	}
	for ( int i = 0; i < entCount; i++ )
	{
		if ( entArray[i]->pvars->obj_type == OBJ_CAMERA )
		{
			if ( entArray[i]->pvars->thinktime < glTime->getTime() )
				entArray[i]->Think();
		}
	}
}


//
// RaceThink
//  ... called by: Think
//	... description: think code while in normal racing mode
//
void CGameControl :: RaceThink()
{
	if ( glInput->isKeyPressed( GKEY_ESC ) )
	{
	/*	IFreeRacingSamples();
		CleanupEnts();
		CleanupLoader();
		*/
		pauseBeginTime = glTime->getTime();
		gameState = GAME_PAUSE;
		IPauseSounds();
		IPlayStationarySound(NULL,50,SOUND_EVENT_PAUSE);
		prePauseState = GAME_RACING;
		return;
	}
	for ( int i = 0; i < entCount; i++ )
	{
		CBaseEntity *pEnt = entArray[i];

		if ( pEnt->pvars->ent_flags & ENT_MOVING )
			CheckCollisions( pEnt ); // check for collisions

		if ( pEnt->isPlayer() )
		{
			((CPlayer*)pEnt)->HandleInput(); // do input for the player
			((CPlayer*)pEnt)->SwitchCheck();
		}

		// think at intervals
		if ( pEnt->pvars->thinktime < glTime->getTime() )
		{
			pEnt->Think(); // MAIN THINK CALL

			if ( pEnt->isPlayer() )
			{
				// get player stats update
				CPlayer *pPlayer = (CPlayer*)pEnt;
				coins = pPlayer->coins;
				lives = pPlayer->lives;
				pPlayer->level = level;

				if ( pPlayer->pvars->origin.z > levelLength ) // we're at the end of the level
				{
					// TO DO: call sounds, etc?
					IPlayStationarySound(NULL, 30, SOUND_EVENT_LEVELEND );
					gameState = GAME_LEVEL_SUCCESS;
					levelEndTime = glTime->getTime();
					levelDoneTime = (levelEndTime - levelStartTime) - pauseTime;
					CalcScore();
					coins = 0; // reset the coin values

				}
				if ( pPlayer->pvars->player_state == PL_DEAD )
				{
					IPauseSounds();
					IPlayStationarySound(NULL, 100, SOUND_EVENT_DEATH );	
					gameState = GAME_DEATH; // game is over
					levelEndTime = glTime->getTime(); // this is when the level has ended
					levelDoneTime = (levelEndTime - levelStartTime) - pauseTime; // end-start, minus paused time
					CalcScore(); // calculate score and bonustime
					coins = 0;

					if ( !m_pHighScores )
						m_pHighScores = new IHighScores( FILE_HIGHSCORES );
					if ( m_pHighScores->checkScore( score ) )
					{
						gameState = GAME_HIGHSCORES;
						IPauseSounds();
					}

				}
			}
		}
	}
}

//
// PauseThink
//  ... called by: Think
//	... description: think code while paused
//
void CGameControl :: PauseThink()
{
	if ( glTime->getTime() - pauseBeginTime > 0.5f )
	{
		if ( glInput->isKeyPressed( GKEY_ESC ) )
		{
			if ( !m_pHighScores )
				m_pHighScores = new IHighScores( FILE_HIGHSCORES );
			if ( m_pHighScores->checkScore( score ) )
			{
				gameState = GAME_HIGHSCORES;
				IPauseSounds();
				timerBack = glTime->getTime() + GAME_TIMER_BACK;
				return;
			}

			IFreeRacingSamples();
			CleanupEnts();
			CleanupLoader();
			gameState = GAME_QUIT;
			return;
		}
	}
	if ( glInput->isKeyPressed( GKEY_SPACE ) )
	{
		gameState = prePauseState;
		pauseTime += glTime->getTime() - pauseBeginTime;
		pauseBeginTime = 0;
		IUnpauseSounds();
		return;
	}
}

//
// HighScoresThink
//  ... called by: Think
//	... description: think code while processing high scores
//
void CGameControl :: HighScoresThink()
{
	if ( glInput->isKeyPressed( GKEY_ESC ) && timerBack <= glTime->getTime() )
	{
		if ( nameState == 0 )
		{
			m_pHighScores = NULL;
			gameState = GAME_DEATH;
			return;
		}
		else if ( nameState == 1 )
		{
			nameState = 0;
			// PLAY SOUND HERE
		}
		else if ( nameState == 2 )
		{
			nameState = 1;
			// PLAY SOUND HERE
		}

		timerBack = glTime->getTime() + GAME_TIMER_BACK;
	}

	if ( glInput->isKeyPressed( GKEY_SPACE ) && timerConfirm <= glTime->getTime() )
	{
		if ( nameState == 0 )
		{
			// PLAY SOUND HERE
			nameState = 1;
		}
		else if ( nameState == 1 )
		{
			// PLAY SOUND HERE
			nameState = 2;
		}
		else if ( nameState == 2 )
		{
			// PLAY SOUND HERE
			string s = "";

			char c[6];
			c[0] = name[0];
			c[1] = '.';
			c[2] = name[1];
			c[3] = '.';
			c[4] = name[2];
			c[5] = '.';
			s.append(c,6);

			m_pHighScores->checkScore( s, score );
			m_pHighScores = NULL;
			gameState = GAME_DEATH;
		}
	
		timerConfirm = glTime->getTime() + GAME_TIMER_CONFIRM;
	}
	
	if ( glInput->isKeyPressed( GKEY_RIGHT ) && timerRight <= glTime->getTime() )
	{


		timerRight = glTime->getTime() + GAME_TIMER_RIGHTLEFT;
	}

	if ( glInput->isKeyPressed( GKEY_LEFT ) && timerLeft <= glTime->getTime() )
	{


		timerLeft = glTime->getTime() + GAME_TIMER_RIGHTLEFT;
	}

	if ( glInput->isKeyPressed( GKEY_UP ) && timerUp <= glTime->getTime() )
	{
		name[nameState]--;
		if ( name[nameState] < 'A' )
			name[nameState] = 'Z';

		// PLAY BEEP HERE
		timerLetterRemind = 0.0f;

		timerUp = glTime->getTime() + GAME_TIMER_UPDOWN;
	}

	if ( glInput->isKeyPressed( GKEY_DOWN ) && timerDown <= glTime->getTime() )
	{
		name[nameState]++;
		if ( name[nameState] > 'Z' )
			name[nameState] = 'A';	

		// PLAY BEEP HERE
		timerLetterRemind = 0.0f;

		timerDown = glTime->getTime() + GAME_TIMER_UPDOWN;
	}

	if ( timerLetterRemind < glTime->getTime() )
	{
		ITextToSpeech speaker;
		string s = "";

		char c[6];
		c[0] = name[0];
		c[1] = '.';
		c[2] = name[1];
		c[3] = '.';
		c[4] = name[2];
		c[5] = '.';
		s.append(c,6);
		
		speaker.speakText(s);

		timerLetterRemind = glTime->getTime() + GAME_TIMER_LETTER;
	}
}

//
// LevelSuccessThink
//  ... called by: Think
//	... description: think code while processing high scores
//
void CGameControl :: LevelSuccessThink()
{
	if ( !bLevelEnd )
	{
		IPlayStationarySound(NULL, 30, SOUND_EVENT_LEVELEND );
		bLevelEnd = true;
		timeTillSayScores = glTime->getTime() + GAME_TIMER_SAYSCORE;
		timeTillCanContinue = glTime->getTime() + GAME_TIMER_SAYSCORE + 1.0f;
	}

	if ( timeTillSayScores < glTime->getTime() )
	{
		string s;

		TCHAR szMsg[MAX_PATH] = TEXT("");
		sprintf( szMsg, TEXT("%i" ), score );
		s.append(szMsg,sizeof(szMsg));
		ITextToSpeech speaker;
		speaker.speakText(s);
		
		timeTillSayScores = glTime->getTime() + 10.0f;
	}

	if ( (glInput->isKeyPressed( GKEY_SPACE ) || glInput->isKeyPressed( GKEY_ESC )) && timeTillCanContinue < glTime->getTime() )
	{
		gameState = GAME_LEVELINIT;
		CleanupEnts();
		level++;
		bLevelEnd = false;
	}
}


//
// DeathThink
//  ... called by: Think
//	... description: think code while dead
//
void CGameControl :: DeathThink()
{
	if ( glInput->isKeyPressed( GKEY_ESC ) || glInput->isKeyPressed( GKEY_SPACE ) )
	{
		gameState = GAME_QUIT;
		IFreeRacingSamples();
		CleanupEnts();
		CleanupLoader();
		if ( m_pHighScores )
			delete m_pHighScores;
	}

	// PLAY SOUND INSTRUCTION
}

//
// DoPhysics
//
//
void CGameControl :: DoPhysics()
{
	if ( gameState == GAME_RACING )
	{
		CBaseEntity* pEnt;
		for ( int i = 0; i < entCount; i++ )
		{
			pEnt = entArray[i];
			pEnt->Physics();
		}
	}
}

//
// Draw
//	... called by: CUberControl::Draw()
//	... description: handles drawing when we're in game mode
//
void CGameControl :: Draw()
{
	
	if ( gameState == GAME_INIT )
	{
		// special case, loading splash screen
		//IDrawLoading();
		return;
	}

}

//
//	PlaySounds
//
//
//
void CGameControl :: PlaySounds()
{
	if ( gameState == GAME_STARTING || gameState == GAME_RACING )
	{
		for ( int i = 0; i < entCount; i++ )
		{
			if ( entArray[i]->isPlayer() )
			{
				IUpdateSounds( entArray, entCount, entArray[i]->pvars );
				return;
			}
		}
	}
}

//
// CheckCollisions
//	... called by: CGameControl::Think()
//	... description: check for collisions
//
void CGameControl :: CheckCollisions( CBaseEntity *pEnt )
{
	
	for ( int i = 0; i < entCount; i++ )
	{
		CBaseEntity* pSecEnt = entArray[i];

		if( pSecEnt->pvars->ent_flags & ENT_COLLIDABLE )
		{
			Vec3d dist = pEnt->pvars->origin - pSecEnt->pvars->origin;
			if ( dist.Length() < (pEnt->pvars->radius + pSecEnt->pvars->radius ) )
			{
				// we have hit someone
				pEnt->Hit( pSecEnt );
				pSecEnt->Hit( pEnt );
			}
		}
	}
}

//
// LevelLoadData
//	... called by: CGameControl::Think()
//	... description: link to level loading system. fill loader with data from the file
//
void CGameControl :: LevelLoadData()
{
	
	loader->Load();
	
	gameState = GAME_LEVELINIT;
}

//
// LevelGenerate()
//	... called by: CGameControl::Think()
//	... description: link to level loading system. fill array with entities
//
void CGameControl :: LevelGenerate()
{
	entArray = loader->Generate(level);
	entCount = loader->getEntCount();
	levelLength = loader->getZEnd();

	for ( int i = 0; i < entCount; i++ )
	{
		if ( entArray[i]->isPlayer() )
		{
			CPlayer *pPlayer = (CPlayer*)entArray[i];
			pPlayer->lives = lives;
			break;
		}
	}

	gameState = GAME_STARTING;
	timeTillStart = glTime->getTime() + TIME_TILL_START;
}



//
// CalcScore()
//	... called by: CGameControl::RaceThink()
//	... description: calculate score/bonustime
//
void CGameControl :: CalcScore()
{
	// first, multiply the coins gotten in this level by the number of points per each
	int addScore = 0;
	addScore = coins*SCORE_PER_COIN;
	
	// get the difference between low and high times
	float possibleBonus = (levelLength/MIN_PLAYER_SPEED - ACCEL_TIME/2) - (levelLength/MAX_PLAYER_SPEED - ACCEL_TIME/2);
	// get the difference between low and our time -- this does not take into account how much you were hit, so it is possible to get no bonus
	bonusTime = (levelLength/MIN_PLAYER_SPEED - ACCEL_TIME/2) - levelDoneTime - pauseTime;

	if ( bonusTime > 0 )
	{
		// equation: the ratio of our bonus time to the possible is multiplied by the range, and added by 1 to make sure we're only increasing score
		float bonusMultiplier = 1.0f + SCORE_MULTIPLIER_RANGE * bonusTime/possibleBonus;
		addScore = (int)addScore*bonusMultiplier;
		score += addScore;
	}
	else
		score += addScore; // no bonus, just give them the coin points
}

//
// SpeakLevelInfo
//	... called by: Think
//	...	description: say what level we're starting
//
void CGameControl :: SpeakLevelInfo( )
{
	ITextToSpeech speaker;
	speaker.speakText("Level starting");
}

//
// WarningThink
//	... called by: CCoin::WarningThink()
//	...	description: what should we do when a coin is picked up?
//
void CGameControl :: WarningThink()
{
	CBaseEntity* pEnt = NULL;
	CPlayer* pPlayer = NULL;
	
	// get the player
	for ( int i = 0; i < entCount; i++ )
	{
		pEnt = entArray[i];

		if( pEnt->isPlayer() )
		{
			pPlayer = (CPlayer*)pEnt;
			break;
		}
	}
	
	CBaseEntity* pClosest = NULL;
	for ( int i = 0; i < entCount; i++ )
	{
		pEnt = entArray[i];
		
		// if it's a mine...
		if ( pEnt->pvars->obj_type == OBJ_OBSTACLE && !(pEnt->pvars->ent_flags & ENT_DEAD ) )
		{
			// if it's in our lane..
			if ( pEnt->pvars->origin.x == pPlayer->pvars->origin.x )
			{
				// if it's ahead of us...
				if ( pEnt->pvars->origin.z > pPlayer->pvars->origin.z )
				{
					// if we don't have anything yet
					if ( !pClosest )
					{
						pClosest = pEnt;
					}
					else 
					{
						// if this is closer than anything else yet
						if ( (pEnt->pvars->origin - pPlayer->pvars->origin).Length() < (pClosest->pvars->origin - pPlayer->pvars->origin).Length() )
						{
							pClosest = pEnt;
						}
					}
				}
			}
		}
	}

	// pClosest should now be the closest obstacle in front of us.
	if ( !pClosest )
		return; // nothing is in front of us, apparently

	Vec3d pointer = pClosest->pvars->origin - pPlayer->pvars->origin;
	float dist = pointer.Length();
	
	// t = d/v
	float time = dist/pPlayer->pvars->speed;

	float klaxInterval = -1;
	float volume = 50;
	
	if ( time >= 5 )
	{
		klaxInterval = -1;
	}
	if ( time < 5 && time >= 4 )
	{
		klaxInterval = 1.0;
	}
	if ( time < 4 && time >= 3 )
	{
		klaxInterval = 0.8;
	}
	if ( time < 3 && time >= 2 )
	{
		klaxInterval = 0.6;
	}
	if ( time < 2 && time >= 1 )
	{
		klaxInterval = 0.4;
	}
	if ( time < 1 )
	{
		klaxInterval = 0.2;
	}

	// determine what the warning level/klaxon interval is
	if ( klaxInterval < 0 )
		return;

	// time to play the sound!
	if ( glTime->getTime() - klaxonLast >= klaxInterval )
	{
		IPlayStationarySound( NULL, volume, SOUND_EVENT_WARNING );
		klaxonLast = glTime->getTime();
	}

}