
#ifndef __CGAMECONTROL_H_
#define __CGAMECONTROL_H_


#include "CBaseEntity.h"
#include "ILevelLoader.h"
#include "IHighScores.h"

//-----------------------------------------------------------
// Class CGameControl
//
//	This class controls the flow of the game. It's principal
// duty is to manage/create an array of game objects, allow
// them to manipulate each other, and draw the results.
//
//
//	UPDATES:
//	2/7/04 - Daniel: created, first pass
//	2/9/04 - Daniel: CoinPickedUp
//	2/12/04 - Daniel: timers, player data, CalcScore
//
//-----------------------------------------------------------

class CGameControl
{
public:
	// construction
	CGameControl(); // default constructor
	CGameControl( int startLevel ); // starting at a different level?
	void CleanupEnts();
	void CleanupLoader();

	// functions
	void Think(); // frame-to-frame logic
	void Draw(); // frame-to-frame drawing
	void PlaySounds(); // frame-to-frame sound updates
	void LevelLoadData(); // load/create array database
	void LevelGenerate();

	void RaceThink();
	void StartThink();
	void PauseThink();
	void HighScoresThink();
	void LevelSuccessThink();
	void DeathThink();

	void CalcScore();
	void DoPhysics();
	void SpeakLevelInfo();
	
	// rules - these are called to determine what happens in game situations
	void CheckCollisions( CBaseEntity *pEnt ); // check for collisions with this entity
	void WarningThink();
	// bool canPlayerMove( CPlayer* ppl );

	
	// timers
	float levelStartTime;
	float levelEndTime;
	float pauseBeginTime; // when did the last pause begin?
	float pauseTime; // total amount of time spent in pause
	float levelDoneTime; // calculated when we finish/die
	float bonusTime; // calculated when we finish/die
	float klaxonLast;

	float timeTillStart;
	float timeTillSayScores;
	float timeTillCanContinue;

	// bools
	bool bLevelEnd;

	// data:
	int level; // what level are we on?
	int score; // calculated at end of each level
	int coins; // for stats
	int lives; // player's lives

	int prePauseState; // to revert to after pause
	
	float levelLength; // gotten by load
	ILevelLoader* loader;

	int gameState; // state of the game
	int entCount; // how many objects in the array
	CBaseEntity** entArray; // main array

	IHighScores* m_pHighScores;
	char name[3];
	float timerLetterRemind;
	float timerBack;
	float timerConfirm;
	float timerUp;
	float timerDown;
	float timerRight;
	float timerLeft;
	int nameState;
};


#endif