
#include "CObstacle.h"
#include "CBaseEntity.h"
#include "globalconst.h"
#include "stateflagdefs.h"
#include "ITime.h"
#include "Vec3.h"
#include "soundfunc.h"

// -------------------------------------
// CObstacle - functions definitions
//
// ... see header for more information
//
//	UPDATES:
//		- 2/11/04, Daniel: created, sort of finished (first pass)
//		- 3/22/04, Daniel: Sound on hit
//
// -------------------------------------

//
// Default Constructor
//
//
CObstacle :: CObstacle()
{
	//Spawn();
}

//
// Origin Constructor
//
//
CObstacle :: CObstacle( Vec3d orig )
{
	//Spawn();
	//pvars->origin = orig;
}

//
// Spawn
//	... called by: constructor (why not just in constructor? because
//			there's always a chance we'd want to reset.)
//	... description: initialize pvars to what this object requires
//
void CObstacle :: Spawn()
{
	// object-specific traits
	pvars->obj_type				= OBJ_OBSTACLE; // object type
	pvars->player_state			= PL_NOT_PLAYER; // this is no player
	pvars->switch_state			= SWITCH_NO;
	pvars->ent_flags			= 0;			 // initialize
	pvars->ent_flags			= ((( pvars->ent_flags | ENT_COLLIDABLE )
									| ENT_VISIBLE )
									| ENT_AUDIBLE ); // flags
	pvars->ang_vel.y			= -0.33*PI;			// spinning (depends on whether implemented)
	pvars->ang_vel.x			= 0;
	pvars->ang_vel.z			= 0;
	pvars->radius				= RADIUS_OBSTACLE; // radius for collision
	pvars->speed				= 0;
	pvars->thinktime			= glTime->getTime();

}


//
// Think
//	... called by: CGameControl->Think()
//	...	description: do game logic at intervals, check for updates/state change, etc.
//
void CObstacle :: Think()
{
	if ( !(pvars->ent_flags & ENT_DEAD) )
	{
		// nothing to do here as of yet
		pvars->thinktime = glTime->getTime() + NT_OBSTACLE;
	}
}

//
// Hit
//	... called by: CGameControl->CheckCollisions()
//	... description: do game logic when a object collides with this one
//
void CObstacle :: Hit( CBaseEntity * victim )
{
	// call gamecontrol? play a sound? temporary explosion graphic? need to get sound/event system worked out.
	if ( victim->isPlayer() )
	{
		IPlayStationarySound(NULL, 100, SOUND_EVENT_HITOBSTACLE );
	}
	Remove();
}

//
// Remove
//	... called by: any, but in this case probably Hit()
//	... description: removes the coin from interaction
//
void CObstacle :: Remove()
{
	// we must disappear...
	pvars->ent_flags = (((pvars->ent_flags ^ ENT_COLLIDABLE )
								^ ENT_VISIBLE )
								^ ENT_AUDIBLE );
	// and die.
	pvars->ent_flags = pvars->ent_flags | ENT_DEAD;

}

