
#include "CUberControl.h"
#include "CGameControl.h"
#include "stateflagdefs.h"
#include "globalconst.h"

// -------------------------------------
// CUberControl - functions definitions
//
// ... see header for more information
//
//	UPDATES:
//		- 2/13/04, Daniel: created, alpha code (hopefully)
//		- 2/18/04, Daniel: changed draw to have a maximum framerate
//		- 4/2/04, Daniel: added menuControl, launch, cleanup for both
//	
// -------------------------------------


//
// Constructor
//	... called by: magic
//
CUberControl :: CUberControl ()
{
	exState = EX_MENU_INTRO;
	gameControl = NULL;
	menuControl = NULL;
	gameStartLevel = -1;
}

//
// Think
//  ... called by: main program execution
//	... description: frame by frame logic for the program
//
void CUberControl :: Think() 
{

	switch ( exState )
	{
	case EX_QUIT:
		break;
	case EX_GAME_INIT:
		LaunchGame();
		break;
	case EX_GAME:
		CheckGame();
		break;
	case EX_MENU_INTRO:
		LaunchMenu(); // HACK
		break;
	case EX_MENU_INIT:
		LaunchMenu();
		break;
	case EX_MENU:
		CheckMenu();
		break;
	}

}

//
// Draw
//  ... called by: main program execution
//	... description: draw each frame, menu or game
//
void CUberControl :: Draw()
{
	Think();

	if ( exState == EX_GAME )
	{
		gameControl->DoPhysics();
		gameControl->Draw();
		gameControl->PlaySounds();
	}


}


//
// LaunchGame
//	... called by: CUberControl::Think()
//	... description: switch to game mode
//
void CUberControl :: LaunchGame()
{
	if ( gameStartLevel >= 0 && gameStartLevel <= LEVEL_START_HIGH )
	{
		// start from a certain level (0 is training)
		gameControl = new CGameControl( gameStartLevel );
		exState = EX_GAME;
	}
	else
	{
		// level was invalid, start anyway, default is 1
		gameControl = new CGameControl();
		exState = EX_GAME;
	}
}


//
// LaunchMenu
//	... called by: CUberControl::Think()
//	... description: switch to menu
//
void CUberControl :: LaunchMenu()
{
	menuControl = new CMenuControl();
	exState = EX_MENU;
}


//
// CleanupGame
//	... called by: CUberControl::Think()
//	... description: kill the gameControl
//
void CUberControl :: CleanupGame()
{
	//gameControl->Cleanup();
	delete gameControl;
	gameControl = NULL;
	gameStartLevel = -1;
}


//
// CleanupMenu
//	... called by: CUberControl::Think()
//	... description: kill the menuControl
//
void CUberControl :: CleanupMenu()
{
	delete menuControl;
	menuControl = NULL;
}


//
// CheckGame
//	... called by: CUberControl::Think()
//	... description: do frame check on the game
//
void CUberControl :: CheckGame()
{
	gameControl->Think();
	if ( gameControl->gameState == GAME_QUIT )
	{
	//exState = EX_QUIT;
		exState = EX_MENU_INIT;
		CleanupGame();
	}
}


//
// CheckMenu
//	... called by: CUberControl::Think()
//	... description: do frame check on the menu
//
void CUberControl :: CheckMenu()
{
	menuControl->Think();
	if ( menuControl->menuState == MENU_DOQUIT )
	{
		exState = EX_QUIT; // this will trigger the exit
		CleanupMenu(); // clean it up, we're waiting to exit
	}
	/*else if ( menuControl->menuState == MENU_DOTRAINING )
	{
		gameStartLevel = 0; // 0 is the training level
		exState = EX_GAME_INIT; // start the game
		CleanupMenu();
	}*/
	else if ( menuControl->menuState == MENU_DOGAMEPLAY )
	{
		gameStartLevel = menuControl->levelStart; // user-chosen level
		exState = EX_GAME_INIT; // start the game
		CleanupMenu();
	}
}
