
#ifndef	__IENTVARS_H_
#define __IENTVARS_H_

#include "Vec3.h"

//-----------------------------------------------------------
// Class IEntVars
//
//	This is an interface class, because it is recognized both
// by CBaseEntity (and its derivatives) and by the engine code.
//	This class contains all data required for drawing and playing
// sounds.
//
//	UPDATES:
//	2/6/04 - Daniel: created, data moved from CBaseEntity
//	2/8/04 - Daniel: added thinktime, realized we don't need to
//				think every second...
//	2/9/04 - Daniel: added ang_vel, changed obj_state to obj_type, phystime
//	2/11/04 - Daniel: switch_state, killed phystime
//	2/21/04 - Daniel: channel_id
//
//-----------------------------------------------------------


class IEntVars
{
public:
	// PHYSICS
	Vec3d		origin; // this is the location vector
	Vec3d		orientation; // this is the direction the entity is "pointing" in
	float		speed; // this is the length of the velocity vector UNIT: FLPOINTS/S
	Vec3d		velocity; // this is directional (unit vector)
	Vec3d		ang_vel; // angular velocity UNIT: RAD/S
	float		radius; // for collision
	// STATES/FLAGS
	int			ent_flags; // flags
	int			obj_type; // state of the object
	int			player_state; // only for players
	int			switch_state; // when switching... objects could switch too, but not in our design
	// SOUND ONLY
	int channel_id;
	bool has_sound;
	// TIMERS
	float		thinktime; // time until we can think next
};


#endif