#include "IHighScores.h"

//-----------------------------------------------------------
// Class IHighScores - functions definitions
//
//	... see header for more information
//	
//
//	UPDATES:
//	3/29/04 - Matt: created, split up .h file
//  3/30/04 - Matt: speakList() first draft completed
//	3/31/04 - Matt: checkScore() completed
//	4/3/04 - Daniel: added error check to line size
//
//
//-----------------------------------------------------------


//Default Constructor
IHighScores::IHighScores()
{
}

//Constructor
//
//
IHighScores::IHighScores(string filePath)
{
	d_fileName = filePath;
	
	for (int i = 0; i < NUM_IN_LIST; i++)
	{
		nameList[i] = "";
		scoreList[i] = 0;
	}
	
	fillLists();

}

//writeList
//
//Writes High Score list to the same file the list was read
//from (d_filename)
//
void IHighScores::writeList()
{
	writeListToFile(d_fileName);
}

//writeList
//
//Writes High Score list to given fileName
//

void IHighScores::writeList(string fileName)
{
	writeListToFile(fileName);
}

//speakList
//
//
void IHighScores::speakList()
{
	ITextToSpeech tts;
	
	/*
	vector<string> temp;
	string line = "";
	
	IFileIO d_i(d_fileName);
	
	
	for (int i = 0; i < NUM_IN_LIST; i++)
	{
		if (!d_i.isFileEnded())
		{
			temp = d_i.ReadLine();
			line = "";
			for (int j = 0; j < temp.size(); j++)
			{
				line.append(temp[j]);
				line = line + " ";
			}
			
			tts.speakText(line);
		}
	}
	
	d_i.CloseFile();
	*/
	
	string line = "number ";

	for (int i = 0; i < NUM_IN_LIST; i++)
	{
		char place[2];
		itoa(i+1, place, 10);
		
		line.append(place);

		tts.speakText(line);
		tts.speakText(nameList[i]);

		char score[10];
		itoa(scoreList[i],score,10);
		tts.speakText(score);

		line = "number ";
	}


}




//checkScore
//
//parameters: players score
//
//Checks score to see if player acheived a top five (or how many are 
//in the list) score. If playerScore is greater than the lowest score
//in scoreList, returns true. Otherwise, returns false.
//
bool IHighScores::checkScore(int playerScore)
{
	for (int i = 0; i < NUM_IN_LIST; i++)
	{
		if (playerScore > scoreList[i])		//playerScore in top five
			return true;
	}

	return false;

}




//checkScore
//
//parameters: player's name (as a string) and score (as an int)
//
//Checks score to see if player acheived a top five (or how many are 
//in the list) score. If playerScore is greater than the lowest score
//in scoreList, playerName is added to nameList and playerScore is added
//to scoreList (both are added in sorted position of highest to lowest score).
//The new list is written to file, and a spoken message is played.
//
//If the score is not high enough, a spoken message is played.
void IHighScores::checkScore(string playerName, int playerScore)
{	
	ITextToSpeech tts;
	bool succeed = false;		//keeps track of whether score is high enough to break 
								//into High Score list

	for (int i = 0; i < NUM_IN_LIST; i++)
	{

		if (playerScore > scoreList[i])		//playerScore in top five
		{
			for (int j = (NUM_IN_LIST - 1); j > i; j--)
			{
				scoreList[j] = scoreList[j-1];		//move scores under playerScore down
				nameList[j] = nameList[j-1];		//move names down
			}
			scoreList[i] = playerScore;				//add playerScore to list
			nameList[i] = playerName;				//add playerName to list
			writeListToFile(d_fileName);			//write new list to file
			
			//speaks a congratulatory message
			//****MIGHT WANT TO RECORD PART OF THIS
			
			char place[2];
			itoa(i+1, place, 10);

			char score_string[10];
			itoa(playerScore, score_string, 10);
		
			tts.speakText("Congratulations! Your score of ");
			tts.speakText(score_string);
			tts.speakText("is number ");
			tts.speakText(place);
			tts.speakText("on the High Score List!");

			succeed = true;							//score did crack top score list
			
			i = NUM_IN_LIST;						//makes sure loop is broken
		}
		
		
	}

	if (!succeed)
		tts.speakText("Sorry, your score did not make the High Score List.");
	


}




//PRIVATE MEMBER FUNCTIONS
//fillList
//
//
void IHighScores::fillLists()
{
	IFileIO d_i;
	d_i.OpenFile(d_fileName);

	int i = 0;
	vector<string> s;

	while (!d_i.isFileEnded() && i < NUM_IN_LIST)
	{
		s = d_i.ReadLine();
		if ( s.size() < 3 )
			continue;
		nameList[i] = s[1]; // was 1
		scoreList[i] = atoi(s[2].c_str()); // was 2

		//cout<<nameList[i]<<" "<<scoreList[i]<<endl;
		i++;
	}
	
	d_i.CloseFile();
}


//writeListToFile
//
//
void IHighScores::writeListToFile(string fileName)
{
	OFileIO d_o(fileName);

	string line = "";
		
	for (int i = 0; i < NUM_IN_LIST; i++)
	{
		char place[2];
		itoa(i+1, place, 10);

		string name = nameList[i];

		char score[10];
		itoa(scoreList[i],score,10);
		
		line.append(place);
		line = line + " ";
		line.append(name);
		line = line + " ";
		line.append(score);

		d_o.WriteLine(line);
		line = "";
	}

	d_o.CloseFile();
}

