
#include "ILevelLoader.h"
#include "CBaseEntity.h"
#include "CPlayer.h"
#include "CObstacle.h"
#include "CCoin.h"
#include "filedefs.h"
#include "IScriptSegment.h"
#include "IScriptLine.h"
#include "IFileIO.h"
#include "stateflagdefs.h"
#include "globalconst.h"
#include "IEntVars.h"
#include "IRand.h"
#include "CCamera.h"
#include "leveldefs.h"

// string/vector headers
#include <string>
#include <vector>
#include <stdlib.h>
#include <iostream>
using namespace std;

// -------------------------------------
// ILevelLoader - functions definitions
//
// ... see header for more information
//
//	UPDATES:
//		2/14/04 - Daniel: Created
//		2/16/04	- Daniel: Load(), first pass
//		2/17/04 - Daniel: Load finished, Generate finished, compiled, tested, and commented up.
//		2/18/04 - Daniel: integrated IRand
//		2/19/04 - Daniel: zEnd
//		4/8/04 - Daniel: changed Generate to include different difficulties, also GetDifficultyFromLevel pseudo-algorithm
//
// -------------------------------------

extern IRand* glRand;
const int CAP_START = 15;

//
// Constructor
//
//
ILevelLoader::ILevelLoader()
{
	capacity = CAP_START;  // initialize capacity
	segArray = new IScriptSegment*[CAP_START]; // set up an array
	segCount = 0; // nothing yet
}

//
// Cleanup
//	... called by: any
//	... description: free the memory in the level database
//
void ILevelLoader:: Cleanup()
{
	for ( int i = 0; i < segCount; i++ )
	{
		segArray[i]->Cleanup();
		delete segArray[i];
	}
	delete segArray;
	segArray = NULL;
}


//
// Generate
//	... called by: CGameControl::LoadLevel()
//	... description: generate a level from the segments/groups
//
CBaseEntity** ILevelLoader :: Generate( int iLevel )
{
	int iLevelSegs[9]; // an array to count the number of segments of each difficulty

	// first, find out how many of each difficulty there are
	for ( int i = 0; i < 9; i++ )
		iLevelSegs[i] = 0; // initalize array
	for ( int i = 0; i < segCount; i++ )
	{
		if ( segArray[i]->difficulty > 0 && segArray[i]->difficulty < 10 )
			iLevelSegs[segArray[i]->difficulty - 1]++; 
	}

	//  now, split up the segments into arrays of the same difficulty
	IScriptSegment** ppFirstLevel = new IScriptSegment*[iLevelSegs[0]];
	IScriptSegment** ppSecondLevel = new IScriptSegment*[iLevelSegs[1]];
	IScriptSegment** ppThirdLevel = new IScriptSegment*[iLevelSegs[2]];
	IScriptSegment** ppFourthLevel = new IScriptSegment*[iLevelSegs[3]];
	IScriptSegment** ppFifthLevel = new IScriptSegment*[iLevelSegs[4]];
	IScriptSegment** ppSixthLevel = new IScriptSegment*[iLevelSegs[5]];
	IScriptSegment** ppSeventhLevel = new IScriptSegment*[iLevelSegs[6]];
	IScriptSegment** ppEighthLevel = new IScriptSegment*[iLevelSegs[7]];
	IScriptSegment** ppNinthLevel = new IScriptSegment*[iLevelSegs[8]];
	
	// load the segments of each difficulty into their respective arrays
	IScriptSegment** ppTemp;
	for ( int i = 0; i < 9; i++ )
	{
		switch (i)
		{
		case 0:
			ppTemp = ppFirstLevel;
			break;
		case 1:
			ppTemp = ppSecondLevel;
			break;
		case 2:
			ppTemp = ppThirdLevel;
			break;
		case 3:
			ppTemp = ppFourthLevel;
			break;
		case 4:
			ppTemp = ppFifthLevel;
			break;
		case 5:
			ppTemp = ppSixthLevel;
			break;
		case 6:
			ppTemp = ppSeventhLevel;
			break;
		case 7:
			ppTemp = ppEighthLevel;
			break;
		case 8:
			ppTemp = ppNinthLevel;
			break;
		}
		int index = 0;
		for ( int j = 0; j < segCount; j++ )
		{
			if ( segArray[j]->difficulty == i+1 )
			{
				ppTemp[index] = segArray[j];
				index++;
			}
		}
	}
	ppTemp = NULL;

	// now, begin the main loop to put segments into entities
	int difficulty = 0;
	int entCounter = 2; // start at 1... player is added manually
	int lineCounter = 0; // initialize to zero
	int index = 0; // initialize
	float xVal = 0; // start at 0,0,0
	float yVal = 0; // ...
	float zRel = 0; // ...
	CBaseEntity**	ppEnts = new CBaseEntity*[entCounter]; // RETURN VALUE, right now to a short-term array
	CBaseEntity**	ppTempEnts; // pointer to a temporary array for the segment being added
	CBaseEntity*	pNewEnt; // poointer the new entity we're creating 
	IEntVars*		pNewVars; // pointer to the new entvars associated with the new entity
	IScriptSegment* seg; // pointer to the segment we're reading 
	IScriptLine*	line; // pointer to the line we're reading

	pNewVars = new IEntVars; // make a new IEntVars
	CBaseEntity* pPlay = new CPlayer; // make a new CPlayer, pointed to by a CBaseEntity*
	pPlay->pvars = pNewVars; // hook in the entvars
	pPlay->Spawn(); // spawn
	ppEnts[0] = pPlay; // put it in the array

	pNewVars = new IEntVars; // another
	CBaseEntity* pCam = new CCamera; // this is for the camera
	pCam->pvars = pNewVars; // hook in
	pCam->Spawn(); // spawn it
	ppEnts[1] = pCam; // hardwire it in

	if ( segCount <= 0 )
		return NULL;

	while ( lineCounter < LEVEL_LINE_MIN ) // don't go more than a segment length above LEVEL_LINE_MIN
	{
		difficulty = GetDifficultyFromLevel(iLevel); // get a difficulty randomly
		switch (difficulty-1) // get the array associated with the difficulty
		{
		case 0:
			ppTemp = ppFirstLevel;
			break;
		case 1:
			ppTemp = ppSecondLevel;
			break;
		case 2:
			ppTemp = ppThirdLevel;
			break;
		case 3:
			ppTemp = ppFourthLevel;
			break;
		case 4:
			ppTemp = ppFifthLevel;
			break;
		case 5:
			ppTemp = ppSixthLevel;
			break;
		case 6:
			ppTemp = ppSeventhLevel;
			break;
		case 7:
			ppTemp = ppEighthLevel;
			break;
		case 8:
			ppTemp = ppNinthLevel;
			break;
		}
		index = glRand->Rand() % iLevelSegs[difficulty-1]; // select a random segment in the chosen difficulty
		seg = ppTemp[index]; // get the random segment

		int tempCap = seg->lineCount * 3; // WARNING: not applicable with more complicated models
		int tempCount = 0; // initialize, this will be used to know how many entities are in this segment
		ppTempEnts = new CBaseEntity*[tempCap]; // set up an array big enough to hold entities
		lineCounter += seg->lineCount; // we're adding the segment's length to the total length
		zRel += SEGMENT_SEPARATION; // segments are separated to ensure no conflicts
		
		for ( int i = 0; i < seg->lineCount; i++ ) // go through each line
		{
			line = seg->lineArray[i]; // get the line

			if ( line->commandType == COM_LINE ) // is this a normal line?
			{
				for ( int j = 0; j < 3; j++ ) // go from left to right
				{
					switch ( j )
					{
					case 0:
						xVal = TRACK_1_X; // track 1's x value
						break;
					case 1:
						xVal = TRACK_2_X; // track 2's x value
						break;
					case 2:
						xVal = TRACK_3_X; // track 3's x value
						break;
					}

					if ( line->objArray[j] == OBJ_COIN ) // the new object is a coin
					{
						pNewVars = new IEntVars; // create a new entvars
						Vec3d orig( xVal, COIN_OFFSET+yVal, zRel ); // create an origin vector
						pNewVars->origin = orig; // give the entvars its origin
						pNewEnt = new CCoin; // create a new CCoin object
						pNewEnt->pvars = pNewVars; // hook in the entvars
						pNewEnt->Spawn(); // spawn the object
						ppTempEnts[tempCount] = pNewEnt; // add the entity to the top of the array
						tempCount++; // increase the number of entities
					} 
					else if ( line->objArray[j] == OBJ_OBSTACLE )
					{
						pNewVars = new IEntVars; // create a new entvars
						Vec3d orig( xVal, yVal, zRel ); // create an origin vector
						pNewVars->origin = orig; // give the entvars its origin
						pNewEnt = new CObstacle; // create a new CCoin object
						pNewEnt->pvars = pNewVars; // hook in the entvars
						pNewEnt->Spawn(); // spawn the object
						ppTempEnts[tempCount] = pNewEnt; // add the entity to the top of the array
						tempCount++; // increase the number of entities
					}
				}
			}

			zRel += LINE_SEPARATION; // separate the next line from this one
		}
	
		CBaseEntity** ppNeoEnts = new CBaseEntity*[entCounter+tempCount]; // create a new, temporary array to be made up of the old array plus the new segment's array
		for ( int i = 0; i < entCounter; i++ ) // go through the old stuff
			ppNeoEnts[i] = ppEnts[i]; // switch them over
		for ( int i = 0; i < tempCount; i++ ) // go through the new stuff
			ppNeoEnts[i+entCounter] = ppTempEnts[i]; // switch them over

		delete [] ppEnts;
		delete [] ppTempEnts;

		ppEnts = ppNeoEnts; // switch the pointer
		entCounter += tempCount; // add the segment's count to the total
	}
	
	zEnd = ppEnts[entCounter-1]->pvars->origin.z + LINE_SEPARATION*3; // the end is a few lines past the last object
	genEntCount = entCounter; // update how many entities were generated

	return ppEnts; // return the pointer to CGameControl


}

//
// GetDifficultyFromLevel
//	... called by: Generate
//	... description: from the level number, determine which difficulty we should randomly select
//
int ILevelLoader :: GetDifficultyFromLevel( int level )
{
	// lots of floats..
	float fFirst = 0.0f;
	float fSecond = 0.0f;
	float fThird = 0.0f;
	float fFourth = 0.0f;
	float fFifth = 0.0f;
	float fSixth = 0.0f;
	float fSeventh = 0.0f;
	float fEighth = 0.0f;
	float fNinth = 0.0f;
	float fSector1 = 0.0f;
	float fSector2 = 0.0f;
	float fSector3 = 0.0f;
	float fSector4 = 0.0f;
	float fSector5 = 0.0f;
	float fSector6 = 0.0f;
	float fSector7 = 0.0f;
	float fSector8 = 0.0f;
	float fSector9 = 0.0f;

	if ( level >= LEVEL_START && level <= LEVEL_EQUIVALENT )
	{
		float fPerSector = (LEVEL_EQUIVALENT - LEVEL_START) / 9.0f; // how many sectors are there? number of levels divided by 9
		fSector1 = 1*fPerSector + LEVEL_START;
		fSector2 = 2*fPerSector + LEVEL_START;
		fSector3 = 3*fPerSector + LEVEL_START;
		fSector4 = 4*fPerSector + LEVEL_START;
		fSector5 = 5*fPerSector + LEVEL_START;
		fSector6 = 6*fPerSector + LEVEL_START;
		fSector7 = 7*fPerSector + LEVEL_START;
		fSector8 = 8*fPerSector + LEVEL_START;
		fSector9 = 9*fPerSector + LEVEL_START; // should be LEVEL_EQUIVALENT
	
		// the first difficulty level is constant in this range
		fFirst = 100.0f;

		// second goes from 0 to 100 from the start to sector 2
		if ( level >= fSector2 )
			fSecond = 100.0f;
		else if ( level >= LEVEL_START && level < fSector2 )
			fSecond = ((level - LEVEL_START)/(fSector2 - LEVEL_START))*(100.0f-0.0f) + 0.0f;

		// third goes from 0 to 100 from sector1 to sector 3
		if ( level >= fSector3 )
			fThird = 100.0f;
		else if ( level <= fSector1 )
			fThird = 0.0f;
		else if ( level > fSector1 && level < fSector3 )
			fThird = ((level - fSector1)/(fSector3 - fSector1))*(100.0f-0.0f) + 0.0f;

		// fourth goes from 0 to 100 from sector2 to sector4
		if ( level >= fSector4 )
			fFourth = 100.0f;
		else if ( level <= fSector2 )
			fFourth = 0.0f;
		else if ( level > fSector2 && level < fSector4 )
			fFourth = ((level - fSector2)/(fSector4 - fSector2))*(100.0f-0.0f) + 0.0f;

		// fifth goes from 0 to 100 from sector3 to sector5
		if ( level >= fSector5 )
			fFifth = 100.0f;
		else if ( level <= fSector3 )
			fFifth = 0.0f;
		else if ( level > fSector3 && level < fSector5 )
			fFifth = ((level - fSector3)/(fSector5 - fSector3))*(100.0f-0.0f) + 0.0f;

		// sixth goes from 0 to 100 from sector4 to sector6
		if ( level >= fSector6 )
			fSixth = 100.0f;
		else if ( level <= fSector4 )
			fSixth = 0.0f;
		else if ( level > fSector4 && level < fSector6 )
			fSixth = ((level - fSector4)/(fSector6 - fSector4))*(100.0f-0.0f) + 0.0f;

		// seventh goes from 0 to 100 from sector5 to sector7
		if ( level >= fSector7 )
			fSeventh = 100.0f;
		else if ( level <= fSector5 )
			fSeventh = 0.0f;
		else if ( level > fSector5 && level < fSector7 )
			fSeventh = ((level - fSector5)/(fSector7 - fSector5))*(100.0f-0.0f) + 0.0f;

		// eighth goes from 0 to 100 from sector6 to sector8
		if ( level >= fSector8 )
			fEighth = 100.0f;
		else if ( level <= fSector6 )
			fEighth = 0.0f;
		else if ( level > fSector6 && level < fSector8 )
			fEighth = ((level - fSector6)/(fSector8 - fSector6))*(100.0f-0.0f) + 0.0f;

		// ninth goes from 0 to 100 from sector7 to sector9
		if ( level >= fSector9 )
			fNinth = 100.0f;
		else if ( level <= fSector7 )
			fNinth = 0.0f;
		else if ( level > fSector9 && level < fSector7 )
			fNinth = ((level - fSector7)/(fSector9 - fSector7))*(100.0f-0.0f) + 0.0f;

	}
	else if ( level > LEVEL_EQUIVALENT /*&& level <= LEVEL_FINAL */)
	{
		// if we're above the equivalent level, just make everything equal
		fFirst = 100.0f;
		fSecond = 100.0f;
		fThird = 100.0f;
		fFourth = 100.0f;
		fFifth = 100.0f;
		fSixth = 100.0f;
		fSeventh = 100.0f;
		fEighth = 100.0f;
		fNinth = 100.0f;

	}


	float fTotal = fFirst + fSecond + fThird + fFourth + fFifth + fSixth + fSeventh + fEighth + fNinth;
	int iRandom = glRand->Rand() % 100; // get a random integer from 0 to 99

	// think of this as a stack of the values for each sector... we divide by the total to make it a percentage, and then we check to see
	// if our random number falls inside the given part of the stack.
	if ( iRandom <= 100*fFirst/fTotal )
		return 1;
	else if ( iRandom > 100*fFirst/fTotal && iRandom <= 100*(fFirst + fSecond)/fTotal )
		return 2;
	else if ( iRandom > 100*(fFirst + fSecond)/fTotal && iRandom <= 100*(fFirst + fSecond + fThird)/fTotal )
		return 3;
	else if ( iRandom > 100*(fFirst + fSecond + fThird)/fTotal && iRandom <= 100*(fFirst + fSecond + fThird + fFourth)/fTotal )
		return 4;
	else if ( iRandom > 100*(fFirst + fSecond + fThird + fFourth)/fTotal && iRandom <= 100*(fFirst + fSecond + fThird + fFourth + fFifth)/fTotal )
		return 5;
	else if ( iRandom > 100*(fFirst + fSecond + fThird + fFourth + fFifth)/fTotal && iRandom <= 100*(fFirst + fSecond + fThird + fFourth + fFifth + fSixth)/fTotal )
		return 6;
	else if ( iRandom > 100*(fFirst + fSecond + fThird + fFourth + fFifth + fSixth)/fTotal && iRandom <= 100*(fFirst + fSecond + fThird + fFourth + fFifth + fSixth + fSeventh)/fTotal )
		return 7;
	else if ( iRandom > 100*(fFirst + fSecond + fThird + fFourth + fFifth + fSixth + fSeventh)/fTotal && iRandom <= 100*(fFirst + fSecond + fThird + fFourth + fFifth + fSixth + fSeventh + fEighth)/fTotal )
		return 8;
	else if ( iRandom > 100*(fFirst + fSecond + fThird + fFourth + fFifth + fSixth + fSeventh + fEighth)/fTotal )
		return 9;
	else 
		return 1; // default case



}



//
// Load
//	... called by: CGameControl
//	... description: load and store all level information from a file
//
void ILevelLoader :: Load()
{
	IFileIO file(FILE_LEVEL); // open the level file
	vector<string> s; // create a vector of strings to be manipulated

	while ( true )
	{
		if ( file.isFileEnded() ) 
			break; // if the file is done, do nothing more
		s = file.ReadLine();
		
		if ( s.size() < 2)
			continue; // if it's a blank line, do nothing.

		if ( s[0] == "segment" && s[1] != "end" ) // a segment is starting
		{
			
			IScriptSegment *seg = new IScriptSegment( "stand-in" ); // s[2] is the seg_id
			
			// add the lines into the segment
			while ( true )
			{
				s = file.ReadLine(); // get the next line
				if ( s[0] == "difficulty" )
				{
					if ( s.size() < 2 )
						continue;
			
					if ( s[1] == "1" )
						seg->difficulty = 1;
					else if ( s[1] == "2" )
						seg->difficulty = 2;
					else if ( s[1] == "3" )
						seg->difficulty = 3;
					else if ( s[1] == "4" )
						seg->difficulty = 4;
					else if ( s[1] == "5" )
						seg->difficulty = 5;
					else if ( s[1] == "6" )
						seg->difficulty = 6;
					else if ( s[1] == "7" )
						seg->difficulty = 7;
					else if ( s[1] == "8" )
						seg->difficulty = 8;
					else if ( s[1] == "9" )
						seg->difficulty = 9;
				}
				if ( s[0] == "line" ) // a normal line command
				{
					IScriptLine* line = new IScriptLine; // create a new object
					line->commandType = COM_LINE; // initialize the type
					for ( int i = 0; i < 3; i++ ) // cycle from left to right on the line
					{
						if ( s[i+1] == "_" )
							line->objArray[i] = OBJ_NONE; // this entry has nothing
						if ( s[i+1] == "C" )
							line->objArray[i] = OBJ_COIN; // this entry has a coin
						if ( s[i+1] == "B" )
							line->objArray[i] = OBJ_OBSTACLE; // this entry has a mine
					}
					seg->AddLine( line ); // add the line to the segment
				} 
				else if ( s[0] == "segment" && s[1] == "end" ) // this is the end of the segment
				{
					break;
				}
			}
			// add to ILevelLoader's segment array
			AddSeg(seg);
		}
	}
}

//
// AddSeg
//	... called by: ILevelLoader::Load()
//	... description: add to the array
//
void ILevelLoader :: AddSeg( IScriptSegment *seg )
{
	if ( segCount >= capacity )
		IncreaseSegCap(); // need more capacity

	segArray[segCount] = seg; // add it in, segCount is above last used index

	segCount++;		// increase the tally
}

//
// IncreaseSegCap
//	... called by: CGameControl::AddSeg()
//	... description: increase the capacity of the array
//
void ILevelLoader :: IncreaseSegCap()
{
	capacity += CAP_START; // for now, go in increments, not double
	IScriptSegment **neo = new IScriptSegment*[capacity]; // new array of new size
	for ( int i = 0; i < segCount; i++ ) 
		neo[i] = segArray[i]; // copy everything over

	delete [] segArray;
	segArray = neo; // now change the pointer
}

//
// getEntCount
//	... called by: any
//	... description: utility
//
int ILevelLoader :: getEntCount()
{
	return genEntCount; // return
}

//
// getZEnd
//	... called by: any
//	... description: utility
//
float ILevelLoader :: getZEnd()
{
	return zEnd;
}
