#ifndef __GLAUX_H_
#define __GLAUX_H_

#include "Vec3.h"

const int WARP_NONE				= 0;
const int WARP_PERPSPIRAL		= 1;
const int WARP_AXIALSPIRAL		= 2;
const int WARP_CURVY			= 3;
const int WARP_TUNNEL			= 4;
const int NUM_WARPS				= 5;

const int TEXTURE_TRACKMAIN		= 0;
const int TEXTURE_LINE1			= 1;
const int TEXTURE_LINE2			= 2;
const int TEXTURE_SIDELOW		= 3;
const int TEXTURE_SIDEHIGH		= 4;
const int TEXTURE_CEILING		= 5;

//-----------------------------------------------------------
// Class GTrackSegment
//
//
//	NOTE: This class is not a primary OR secondary requirement, and
// comes courtesy of Daniel's free time.
//
//	A track segment, consists of a quad, a color, and a texture index.
//
//-----------------------------------------------------------

const float TRACK_SEG_LENGTH		= 20;

class GTrackSegment
{
public:
	float first[3];
	float firstNorm[3];
	float second[3];
	float secondNorm[3];
	float third[3];
	float thirdNorm[3];
	float fourth[3];
	float fourthNorm[3];

	float R;
	float G;
	float B;

	int textureIndex;
};

//-----------------------------------------------------------
// Class GTrackArray
//
//	NOTE: This class is not a primary OR secondary requirement, and
// comes courtesy of Daniel's free time.
//
//	With that said, it performs three roles: first, to create a "track"
// out of thin air when given only the length to the last object in the game.
// Secondly, it can warp that track in a few ways, changing the x and y coordinates
// of all track vertices according to some function of z. Finally, it can provide
// similar offsets for the locations of the objects and the player in the game, to be
// used when rendering. See the rendering code for how this works.
//
// NOTE 2: This is by no means how this *should* be done. The core model of the game
// is merely objects at conventions for certain x and z coordinates (with minor variations in y), so
// the track geometry, the track and object warp, and the very graphics themselves are all
// tacked on to the simplistic level model. (as opposed to say, some sort of "spine" system).
//
//-----------------------------------------------------------

class GTrackArray
{
public:
	GTrackArray() { bCreated = false; segNum = 0;} // constructor
	void Create( int levelLength ); // create a track with only the ground
	void Create2( int levelLength ); // create a track that's like a tunnel
	void Warp( float outSin, float inSin, float outCos, float inCos ); // PERPSPIRAL warp
	void Warp2( float radius, float frac ); // AXIALSPIRAL warp
	void Warp3( float outSin, float frac ); // CURVY warp
	void DoWarp( float radius, float frac, int type ); // picks a warp function from a warptype
	void Draw(); // draws the track (or does it?)

	// helper functions, see declarations for more information
	float getXOffsetFromZ( float z, float frac, float range );
	float getYOffsetFromZ( float z, float frac, float range );
	Vec3d getNormal( float x, float y, float z, float radius, float frac );
	float getXOffsetFromXYZ( float x, float y, float z, float radius, float frac );
	float getYOffsetFromXYZ( float x, float y, float z, float radius, float frac );
	float getRotationFromXYZ( float x, float y, float z, float radius, float frac);
	float getAxialXOffset( float x, float y, float z, float radius, float frac );
	float getAxialYOffset( float x, float y, float z, float radius, float frac );
	float getCurvyYOffset( float z, float radius, float frac );

	void Cleanup(); // free memory

	GTrackSegment* track; // pointer to an array of track segments
	int segNum;  // how many segments are in the track?
	int warpType; // what type of warp are we currently using?
	bool bCreated; // has the track been created?
};

#endif