#include "soundfunc.h"
#include "Vec3.h"
#include "IEntVars.h"
#include "CBaseEntity.h"
#include "fmod.h"

#include "string"

using namespace std;


// -------------------------------------
// soundfunc - functions definitions
//
// ... see header for more information
//
//	UPDATES:
//		- 2/21/04, Eden: created
//		- 2/22/04, Daniel: in the course of getting it to compile: no soundfunc, can't switch floats, need entCount in IInitializeSounds
//		- 2/24/04, Eden: changed the way samples are loaded to reduce channels
//		- 3/15/04, Eden: changed IUpdateSounds and IInitializeSounds to only load channels when objects are within "hearing range"
//		- 3/21/04, Eden: added IPlayStationarySounds and IPlay3DSounds
// -------------------------------------
#define TRUE true
#define FALSE false

//
// IInitializeSounds
//  ... called by: CGameControl
//	... description: Initializes sounds for all coins and obstacles. 
//					Assigns each to a separate sound channel.
//					Player is initialized to face forward.
//
void IInitializeSounds( CBaseEntity** entArray, int entCount )
{
	//Initialize FMOD
	FSOUND_Init(44100, 32, 0);

	//Make these pointers global somehow
	//Pointers to samples
	static const FSOUND_SAMPLE *sample_coin_1;
	static const FSOUND_SAMPLE *sample_coin_2;
	static const FSOUND_SAMPLE *sample_coin_3;
	static const FSOUND_SAMPLE *sample_obstacle_1;
	static const FSOUND_SAMPLE *sample_obstacle_2;
	static const FSOUND_SAMPLE *sample_obstacle_3;

	//Loading samples
	sample_coin_1 = FSOUND_Sample_Load(FSOUND_FREE, "coin1.wav", FSOUND_HW3D | FSOUND_LOOP_NORMAL, 0, 0);
	sample_coin_2 = FSOUND_Sample_Load(FSOUND_FREE, "coin2.wav", FSOUND_HW3D | FSOUND_LOOP_NORMAL, 0, 0);
	sample_coin_3 = FSOUND_Sample_Load(FSOUND_FREE, "coin3.wav", FSOUND_HW3D | FSOUND_LOOP_NORMAL, 0, 0);
	sample_obstacle_1 = FSOUND_Sample_Load(FSOUND_FREE, "obstacle1.wav", FSOUND_HW3D | FSOUND_LOOP_NORMAL, 0, 0);
	sample_obstacle_2 = FSOUND_Sample_Load(FSOUND_FREE, "obstacle2.wav", FSOUND_HW3D | FSOUND_LOOP_NORMAL, 0, 0);
	sample_obstacle_3 = FSOUND_Sample_Load(FSOUND_FREE, "obstacle3.wav", FSOUND_HW3D | FSOUND_LOOP_NORMAL, 0, 0);

	//Vectors for coins/obstacles
	float pos[3] = {0, 0, 0};
	//float vel[3] = {0, 0, 0}; //won't be needed since all coins/obstacles stationary 

	//Vectors for player
	float listener_pos[3] = {TRACK_2_X, 0, 0}; //player initial position
	float listener_vel[3] = {0, 0, 0}; //player initially stationary
	
	//Setting player position
	FSOUND_3D_Listener_SetAttributes(listener_pos, listener_vel, 0, 0, 1.0f, 0, 1.0f, 0);

}


//
// IUpdateSounds
//  ... called by: CGameControl
//	... description: Updates the player position for sound purposes
//
void IUpdateSounds( CBaseEntity** entArray, int entCount, IEntVars * player )
{
	float pos[3] = {0, 0, 0};

	//Go through all entities
	//Coins and obstacles will be given sounds and positions
	for ( int i = 0; i < entCount; i++ )
	{
		IEntVars *pVar = entArray[i]->pvars;

		//If the object is within "hearing range" and doesn't have a sound yet, give it a sound
		if( ( pVar->origin.z - player->origin.z < MAX_PLAYER_SPEED * 5 ) && ( pVar->origin.z - player->origin.z > MAX_PLAYER_SPEED * -1 ) && !(pVar->has_sound) )
		{

			if( pVar->obj_type == OBJ_COIN )	//If it's a coin
			{
				if ( pVar->origin.x == TRACK_1_X )
				{
					pVar->channel_id = FSOUND_PlaySound(FSOUND_FREE, sample_coin_1);
				} 
				else if ( pVar->origin.x == TRACK_2_X )
				{
					pVar->channel_id = FSOUND_PlaySound(FSOUND_FREE, sample_coin_2);
				}
				else if ( pVar->origin.x == TRACK_3_X )
				{
					pVar->channel_id = FSOUND_PlaySound(FSOUND_FREE, sample_coin_3);
				}

				//Get and set the position of the coin
				pos[0] = -pVar->origin.x;
				pos[1] = pVar->origin.y;
				pos[2] = -pVar->origin.z;
	
				FSOUND_3D_SetAttributes( pVar->channel_id, pos, NULL );
				pVar->has_sound = TRUE;
				//FSOUND_SetMute( pVar->channel_id, TRUE );
			}
			else if( pVar->obj_type == OBJ_OBSTACLE)	//If it's an obstacles
			{
				//Play the obstacle sound determined by the lane
				if ( pVar->origin.x == TRACK_1_X )
				{
					pVar->channel_id = FSOUND_PlaySound(FSOUND_FREE, sample_obstacle_1);
				} 
				else if ( pVar->origin.x == TRACK_2_X )
				{
					pVar->channel_id = FSOUND_PlaySound(FSOUND_FREE, sample_obstacle_2);
				}
				else if ( pVar->origin.x == TRACK_3_X )
				{
					pVar->channel_id = FSOUND_PlaySound(FSOUND_FREE, sample_obstacle_3);
				}


				//Get and set the position of the obstacle
				pos[0] = -pVar->origin.x;
				pos[1] = pVar->origin.y;
				pos[2] = -pVar->origin.z;

				FSOUND_3D_SetAttributes( pVar->channel_id, pos, NULL );
				pVar->has_sound = TRUE;
				//FSOUND_SetMute( pVar->channel_id, TRUE );
			}
		}

		else if( pVar->origin.z - player->origin.z <= MAX_PLAYER_SPEED * -1 && pVar->has_sound )
		{
			FSOUND_StopSound( pVar->channel_id );
			pVar->has_sound = FALSE;
		}
	}

/*
	for ( int i = 0; i < entCount; i++ )
	{
		IEntVars *pVar = entArray[i]->pvars;

		if ( !(pVar->ent_flags & ENT_AUDIBLE) )
			FSOUND_StopSound(pVar->channel_id );

		if( pVar->obj_type == OBJ_COIN || pVar->obj_type == OBJ_OBSTACLE)
		{
			if( pVar->origin.z - player->origin.z < MAX_PLAYER_SPEED * 5 )
			{
				FSOUND_SetMute( pVar->channel_id, FALSE );
			}
			else if( player->origin.z - pVar->origin.z > MAX_PLAYER_SPEED )
			{
				FSOUND_StopSound( pVar->channel_id );
			}
		}
	}
*/

	//update player position
	float listener_pos[3] = {0, 0, 0};
	listener_pos[0] = -player->origin.x;
	listener_pos[1] = player->origin.y;
	listener_pos[2] = -player->origin.z;

	//update player velocity
	float listener_vel[3] = {0, 0, 0};
	listener_vel[0] = -player->velocity.x;
	listener_vel[1] = player->velocity.y;
	listener_vel[2] = -player->velocity.z;

	//Set and Update
	FSOUND_3D_Listener_SetAttributes(listener_pos, listener_vel, 0, 0, 1.0f, 0, 1.0f, 0);

	FSOUND_Update();
}


//
// IPlayStationarySound
//  ... called by: CGameControl
//	... description: plays a single sound once with no 3D support
//
void IPlayStationarySound( float pos[3], int volume, string type )
{
	FSOUND_SAMPLE *sample;
	int channel;

	if(type == "hitObstacle")
		sample = FSOUND_Sample_Load(FSOUND_FREE, "hitObstacle.wav", FSOUND_2D, 0, 0);
	else if(type == "collectCoin")
		sample = FSOUND_Sample_Load(FSOUND_FREE, "collectcoin.wav", FSOUND_2D, 0, 0);
	else if(type == "death")
		sample = FSOUND_Sample_Load(FSOUND_FREE, "death.wav", FSOUND_2D, 0, 0);
	else if(type == "finishLevel")
		sample = FSOUND_Sample_Load(FSOUND_FREE, "finishLevel.wav", FSOUND_2D, 0, 0);
	else if(type == "pause")
		sample = FSOUND_Sample_Load(FSOUND_FREE, "pause.wav", FSOUND_2D, 0, 0);
	else if(type == "startLevel")
		sample = FSOUND_Sample_Load(FSOUND_FREE, "startLevel.wav", FSOUND_2D, 0, 0);

	channel = FSOUND_PlaySound(FSOUND_FREE, sample);
	FSOUND_SetVolume(channel, volume);
}


//
// IPlay3DSound
//  ... called by: CGameControl
//	... description: plays a single sound once with 3D support
//
void IPlay3DSound( float pos[3], float vel[3], int volume, string type )
{
	FSOUND_SAMPLE *sample;
	int channel;

	if(type == "moveLeft")
		sample = FSOUND_Sample_Load(FSOUND_FREE, "moveLeft.wav", FSOUND_HW3D, 0, 0);
	else if(type == "moveRight")
		sample = FSOUND_Sample_Load(FSOUND_FREE, "moveRight.wav", FSOUND_HW3D, 0, 0);


	channel = FSOUND_PlaySound(FSOUND_FREE, sample);
	FSOUND_SetVolume(channel, volume);
	FSOUND_3D_SetAttributes( channel, pos, vel );
}
